
/****************************************************************************/
/* Nombre: mainwindow.h                                                     */
/* Descripcion: biblioteca empleada para la ventana basica de la aplicacion */
/* Argumentos:	   															*/
/* Valor Devuelto:									                        */
/*								                                            */
/*				Trabajo Fin de Grado: David Sanz Cabreros					*/
/****************************************************************************/


//Definiciones previas
#ifndef MAINWINDOW_H
#define MAINWINDOW_H

/***************************** BIBLIOTECAS **********************************/
//Biblioteca que incluye todo lo necesario para ejecutar una ventana de windows
#include <QMainWindow>
#include <QtGui>

void MatrizTexto();
void MatrizTextoUp();
void MatrizTextoDown();
void MatrizIDStatesPadres();
void MatrizITStatesPadres();
void MatrizDatosSimulacionFinal(int NumSignals);
void LimpiarMemoria();


/******************************* CLASES *************************************/
//Definicion de las clases necesarias para Main Window
//Clase para tener una aplicacion Main Window
class MainWindow : public QMainWindow
{
    Q_OBJECT	//Definicion de Objeto

public:
    MainWindow();

protected:
    void closeEvent(QCloseEvent *event);	//Detecta cuando el usuario quiere cerrar la ventana

private slots:
    void newFile();							//Crea un nuevo archivo
    void open();							//Abre un archivo
    bool save();							//Guarda un archivo
    bool saveAs();							//Guarda un archivo como...
	void openRecentFile();					//Archivos recientemente abiertos
	void run();								//Ejecuta el simulador
    void about();							//Da informacion acerca de la aplicacion
    void documentWasModified();				//Detecta si el documento se ha modificado

private:
    void createActions();					//Crea acciones
    void createMenus();						//Crea menus
    void createToolBars();					//Crea la barra de herramientas
    void createStatusBar();					//Crea la barra de estado
    void readSettings();					//Lee los ajustes de la aplicacion y las preferencias del usuario
	void updateRecentFileActions();			//Carga los archivos recientes
    void writeSettings();					//Escribe los ajustes
    bool maybeSave();						//Pregunta al usuario si quiere guardar los cambios
    void loadFile(const QString &fileName);				//Carga un archivo
    bool saveFile(const QString &fileName);				//Guarda un archivo
    void setCurrentFile(const QString &fileName);		//Resetea el estado de algunas variables cuando el archivo se carga o guarda
	void simulacion(const QString &fileName);			//Funcion a la que accede tras selecionar RUN
	void TXTOut(QStringList ListTXT);

	////////////////////////////////////////////////////////////////////
	void ErroresYWarnings(QString ErrWar, int tipo);
	void LecturaArchivo(const QString &fileName);			//Funcion de Lectura del archivo para ver que hay e ir guardando la informacin
	void MostrarDatosGuardados();
	void OrdenandoBoxes();
	void BoxCabecera();
	void BoxPorts();
	void BoxCode();
	void EjecBoxCabecera();
	void EjecBoxPorts();
	void EjecBoxCode();
	void SimulacionDeclarativa();
	void NextBox(int NextPadre, int &ItHijo);
	void SaveSignals(int IteradorMemoria, int modo);
	void MostrarSignals();
	void SimulacionOperativa();
	void BoxState();
	void MostrarDatosInicioSimOperativa();
	void PresimulacionOperativa();
	void SimulacionOperativaON();
	void AnalizadorEventos();
	void ValoresPorDefecto();
	void MostrarEventos();
	void GuardarSimulacion(int CYCLE);
	void GuardarActual(int CYCLE);
	void GuardarDespues(int CYCLE);
	void MostrarSignalPorCicloSimulacion();
	void MostrarVerificacionesARealizar();
	void RealizarVerificaciones();
	void ComprobarCambiosEnSimulacion(int &CAMBIOS);


	void ExprPreviaToken(QString &frase);
	void ExprToken(QString &frase);
	void ExprAToken(QString &frase);
	void ExprBToken(QString &frase);
	void ExprCToken(QString &frase);
	void ExprDToken(QString &frase);
	void ExprEToken(QString &frase);
	void ExprFToken(QString &frase);
	void ExprGToken(QString &frase);
	void ConstantToken(QString &frase);
	void GroupToken(QString &frase);
	void SignalToken(QString &frase);



	void ExprPrevia(const QString &Expresion);
	void Expr(const QString &Expresion);				//Funcion para analizar la primera expresion
	void ExprA(const QString &exprA);					//Funcion para analizar las expresiones A
	void ExprB(const QString &exprB);					//Funcion para analizar las expresiones B
	void ExprC(const QString &exprC);					//Funcion para analizar las expresiones C
	void ExprD(const QString &exprD);					//Funcion para analizar las expresiones D
	void ExprE(const QString &exprE);					//Funcion para analizar las expresiones E
	void ExprF(const QString &exprF);					//Funcion para analizar las expresiones F
	void ExprG(const QString &exprG);					//Funcion para analizar las expresiones G
	void Group(const QString &Grupo);					//Funcion para analizar GROUPS
	void Signal(const QString &Senal);					//Funcion para analizar SIGNALS
	void Constant(const QString &Constante);			//Funcion para analizar CONSTANTES
	////////////////////////////////////////////////////////////////////

	QString strippedName(const QString &fullFileName);	//QString que acorta el nombre del archivo

	QGroupBox *CreateEditGroup();		//Definicion del nombre del box que crea la ventana de edicion
	QGroupBox *CreateOutGroup();		//Definicion del nombre del box que crea la ventana de salida
	QGroupBox *CreateProcessGroup();	//Definicion del nombre del box que crea la ventana de procesamiento
	QGroupBox *Edit;				//Definicion del nombre del box de edicion
	QGroupBox *Out;					//Definicion del nombre del box de salida
	QGroupBox *Process;				//Definicion del nombre del box de procesamiento
    QPlainTextEdit *textEdit;		//Definicion del nombre del editor de texto
	QListWidget *textOut;		//Definicion del nombre del editor de texto que muestra la salida
	QListWidget *textProcess;		//Definicion del nombre del editor de texto donde se mostrara el proceso
    QString curFile;				//Cadena de caracteres para guardar el archivo en ejecucion
	QString FILE;					//Cadena de caracteres con el archivo que se va a simular

    QMenu *fileMenu;			//Menu archivos
	QMenu *recentFilesMenu;		//Menu de archivos recientes
    QMenu *editMenu;			//Menu editar
    QMenu *helpMenu;			//Menu ayuda
    QToolBar *fileToolBar;		//Archivos (barra de herramientas)
    QToolBar *editToolBar;		//Editar (barra de herramientas)
    QAction *newAct;			//Accion nuevo 
    QAction *openAct;			//Accion abrir 
    QAction *saveAct;			//Accion guardar 
    QAction *saveAsAct;			//Accion guardar como 
    QAction *exitAct;			//Accion cerrar 
    QAction *cutAct;			//Accion cortar
    QAction *copyAct;			//Accion copiar
    QAction *pasteAct;			//Accion pegar
	QAction *runAct;			//Accion ejecutar simulador
    QAction *aboutAct;			//Accion acerca de
    QAction *aboutQtAct;		//Accion acerca de Qt
	QAction *separatorAct;		//Accion de separar en el menu
	//QList *BOX;

	//Enumeracion de los archivos mas recientes
	enum { MaxRecentFiles = 5 };
     QAction *recentFileActs[MaxRecentFiles];
};

#endif
