/****************************************************************************/
/* Nombre: mainwindow.cpp                                                   */
/* Descripcion: archivo cpp en el cual quedan definidos los comandos		*/
/*				para crear la ventana bsica de la aplicacin				*/
/* Argumentos:	   															*/
/* Valor Devuelto:									                        */
/*								                                            */
/*				Trabajo Fin de Grado: David Sanz Cabreros					*/
/****************************************************************************/



/***************************** BIBLIOTECAS **********************************/
//Biblioteca con la definicion de la clase QtGui
#include <QtGui>
//Biblioteca mainwindow.h
#include "mainwindow.h"
#include "globalDefs.h"
#include "shared.h"
//#include "globalComponents.cpp"



#include "stdlib.h"

//Valor ValorConstant;	//Definicion de la clase en la que se va a guardar el valor constante
QStringList SimProcess;
QStringList OutProcess;
QStringList TokenProcess;

int NumDec;															//Variable para guardar el numero decimal que va pasando por los analizadores de expresiones
int HayComparacion, HayLogica;										//Variable para guardar si hay logica o comparacion en los analizadores (0: NO HAY - 1: SI HAY)
bool ComparacionExpresiones;										//Variable para saber la logica de la logica o la comparacion que devuelve el analizador de expresiones
QString NameSignalTOKEN;											//Nombre de la seal devuelto por el analizador de expresiones
const int MSG_Lectura=0, MSG_War=2, MSG_Err=1;						//Variables para saber si mandamos un Warning o un Error en la simulacin
const QString TYPETestBench="TestBench", TYPEDiseno="Design";		//Variables para guardar el tipo de padre
const int filasMatricesTexto=50;									//Numero de filas de las matrices de texto y textoDOWN al inicializar
const int filasMatricesTextoUp=10;									//Numero de filas de las matrices de textoUP
int TimeCycleClock;													//Variable para saber el valor del ciclo del reloj
QString UnidadReloj;												//Variable para saber la unidad del ciclo de reloj
int CYCLE=0;														//Variable para saber en que ciclo estamos
int ASYNC=0, SYNC=1, AMBOS=2;										//Variables para decidir que tipos de seales se guardan o no
int repeticiones=0;													//Variable para saber si existe un bucle infinito en el diseo o no


QFile fOut("OutFileSimulation.txt");
QTextStream salidaTXT(&fOut);

/*********************** FUNCIONES DEL MAIN WINDOW **************************/
//Funcion principal del main window
MainWindow::MainWindow()
{
	//Variables para los editores de texto
    textEdit = new QPlainTextEdit;
	textOut	= new QListWidget;
	textProcess = new QListWidget;

	//Convertimos textOut y textProcess en un editor de solo lectura
	//textOut->setReadOnly(true);
	textOut->setSelectionMode(QAbstractItemView::NoSelection);
	textProcess->setSelectionMode(QAbstractItemView::NoSelection);

	//Creamos la pantalla principal del Main Windows
	//Para ello creamos un grid en el que se incluyen los 3 widgets que crean los diferentes grupos (layouts)
	QGridLayout *grid = new QGridLayout;
	grid->addWidget(CreateEditGroup(),1,0);		//Grupo de edicion
	grid->addWidget(CreateOutGroup(),2,0);		//Grupo de salida
	grid->addWidget(CreateProcessGroup(),3,0);	//Grupo de Proceso
	setLayout(grid);							//Establecemos el layout

	//En la clase main Windows no podemos poner layouts como hariamos en un dialogo
	//Para insertar los 3 layouts tenemos que crear un nuevo widget, guardar en el
	//el grid y una vez hecho ponerlo en el centro de la pantalla
	QWidget *widget = new QWidget();
	widget->setLayout(grid);
	setCentralWidget(widget);

	//Llamadas a las funciones que crean las acciones, menus, barras de herramientas y barra de estado
    createActions();
    createMenus();
    createToolBars();
    createStatusBar();

	//Llamada a la funcion que lee los ajustes de la aplicacion
    readSettings();

	//Conexion signal-slot entre el documento del editor de texto y el slot (funcion) que comprueba si se ha modificado
    connect(textEdit->document(), SIGNAL(contentsChanged()),
            this, SLOT(documentWasModified()));

	//Funcion que establece el titulo de la aplicacion
    setCurrentFile("");
	setWindowTitle(tr("Simulador de circuitos digitales para diagramas ASM"));
    setUnifiedTitleAndToolBarOnMac(true);

}

//Box encargado de crear el grupo de edicion
QGroupBox *MainWindow::CreateEditGroup()
{
	Edit = new QGroupBox(tr("Ventana de edicion del archivo .vdo"));	//Creamos una variable para el box
	QVBoxLayout *boxEdit = new QVBoxLayout;		//Lo convertimos en un box vertical
	boxEdit->addWidget(textEdit);				//Aadimos el widget QPlainTextEdit
	Edit->setLayout(boxEdit);					//Establecemos el layout
	return Edit;
}

//Box encargado de crear el grupo de salida
QGroupBox *MainWindow::CreateOutGroup()
{
	Out = new QGroupBox(tr("Ventana de salida del archivo tras la simulacin"));	//Creamos una variable para el box
	QVBoxLayout *boxOut = new QVBoxLayout;		//Lo convertimos en un box vertical
	boxOut->addWidget(textOut);					//Aadimos el widget QPlainTextEdit
	Out->setLayout(boxOut);						//Establecemos el layout
	return Out;
}

//Box encargado de crear el grupo de procesamiento
QGroupBox *MainWindow::CreateProcessGroup()
{
	Process = new QGroupBox(tr("Ventana con el proceso de la simulacin"));			//Creamos una variable para el box	
	QVBoxLayout *boxProcess = new QVBoxLayout;	//Lo convertimos en un box vertical
	boxProcess->addWidget(textProcess);			//Aadimos el widget QPlainTextEdit
	Process->setLayout(boxProcess);				//Establecemos el layout
	return Process;
}


//Funcion que detecta si el usuario quiere cerrar la ventana
void MainWindow::closeEvent(QCloseEvent *event)
{
//Llama a la funcion maybeSave para recordar al usuario si se quieren guardar los cambios
    if (maybeSave()) {
        writeSettings();
        event->accept();
    } else {
        event->ignore();
    }
}

//Funcion que crea un nuevo archivo
void MainWindow::newFile()
{
    if (maybeSave()) {
        textEdit->clear();	//Limpia el editor de texto
        setCurrentFile("");
    }
}

//Funcion que abre un archivo ya existente
void MainWindow::open()
{
    if (maybeSave()) {
		//Abrir ficheros solo con extension *.vdo
        QString fileName = QFileDialog::getOpenFileName(this, tr("Open source file"),0, tr("Source files (*.vdo)"), 0, 0);
        if (!fileName.isEmpty())
            loadFile(fileName);
    }
}

//Funcion que guarda un archivo
bool MainWindow::save()
{
    if (curFile.isEmpty()) {
        return saveAs();
    } else {
        return saveFile(curFile);
    }
}

//Funcion para guardar archivo inexistente con un nombre
bool MainWindow::saveAs()
{
    QString fileName = QFileDialog::getSaveFileName(this);
    if (fileName.isEmpty())
        return false;

    return saveFile(fileName);
}

//Funcion para poder abrir archivos recientemente cargados
 void MainWindow::openRecentFile()
 {
     QAction *action = qobject_cast<QAction *>(sender());
     if (action)
         loadFile(action->data().toString());
 }

//Funcion que ejecuta la aplicacion
void MainWindow::run()
{
	//Si no hay archivo en ejecucion (esta vacio)
    if (curFile.isEmpty()) {
		QMessageBox runBox;
		runBox.setText("No hay ningun archivo seleccionado");
		runBox.exec();
		statusBar()->showMessage(tr("No hay ningun archivo seleccionado"));
    } else {

		//Si hay archivo en ejecucion 
		QMessageBox runBox;
		runBox.setText("Simulando el archivo.");
		runBox.exec();
		statusBar()->showMessage(tr("Simulando el archivo"));

		//Mandamos el archivo a la funcion simulacion
        return simulacion(curFile);
	}
}

//Funcion que da informacin acerca de la aplicacion
void MainWindow::about()
{
   QMessageBox::about(this, tr("About Application"),
            tr("Simulador de circuitos digitales integrable en un compilador de diagramas ASM "
			   ". La aplicacion se encarga de mostrar un archivo .vdo, analizarlo y mostrar "
			   "su procesamiento de validacin y el archivo LOG de salida que muestra si hay "
			   "errores o no en el archivo .vdo"));
}

//Funcion que comprueba si se ha modificado el documento
void MainWindow::documentWasModified()
{
    setWindowModified(textEdit->document()->isModified());
}

//Funcion que crea las acciones
void MainWindow::createActions()
{
	//Accion de archivo nuevo
    newAct = new QAction(QIcon(":/images/new.png"), tr("&New"), this);
    newAct->setShortcuts(QKeySequence::New);
    newAct->setStatusTip(tr("Crear un archivo nuevo"));
    connect(newAct, SIGNAL(triggered()), this, SLOT(newFile()));

	//Accion de abrir un archivo existente
    openAct = new QAction(QIcon(":/images/open.png"), tr("&Open..."), this);
    openAct->setShortcuts(QKeySequence::Open);
    openAct->setStatusTip(tr("Abrir un archivo existente"));
    connect(openAct, SIGNAL(triggered()), this, SLOT(open()));

	//Accion de guardar el archivo
    saveAct = new QAction(QIcon(":/images/save.png"), tr("&Save"), this);
    saveAct->setShortcuts(QKeySequence::Save);
    saveAct->setStatusTip(tr("Guardar el archivo en el disco"));
    connect(saveAct, SIGNAL(triggered()), this, SLOT(save()));

	//Accion de guardar el archivo con nombre
    saveAsAct = new QAction(tr("Save &As..."), this);
    saveAsAct->setShortcuts(QKeySequence::SaveAs);
    saveAsAct->setStatusTip(tr("Guardar archivo como..."));
    connect(saveAsAct, SIGNAL(triggered()), this, SLOT(saveAs()));

	//Accion para mostrar los archivos recientemente abiertos
	for (int i = 0; i < MaxRecentFiles; ++i) {
         recentFileActs[i] = new QAction(this);
         recentFileActs[i]->setVisible(false);
         connect(recentFileActs[i], SIGNAL(triggered()),
                 this, SLOT(openRecentFile()));
     }

	//Accion para cerrar la aplicacion
    exitAct = new QAction(tr("E&xit"), this);
    exitAct->setShortcuts(QKeySequence::Quit);
    exitAct->setStatusTip(tr("Salir de la aplicacion"));
    connect(exitAct, SIGNAL(triggered()), this, SLOT(close()));

	//Accion de cortar
    cutAct = new QAction(QIcon(":/images/cut.png"), tr("Cu&t"), this);
    cutAct->setShortcuts(QKeySequence::Cut);
    cutAct->setStatusTip(tr("Cortar texto seleccionado"));
    connect(cutAct, SIGNAL(triggered()), textEdit, SLOT(cut()));

	//Accion de copiar
    copyAct = new QAction(QIcon(":/images/copy.png"), tr("&Copy"), this);
    copyAct->setShortcuts(QKeySequence::Copy);
    copyAct->setStatusTip(tr("Copiar el texto seleccionado"));
    connect(copyAct, SIGNAL(triggered()), textEdit, SLOT(copy()));

	//Accion de pegar
    pasteAct = new QAction(QIcon(":/images/paste.png"), tr("&Paste"), this);
    pasteAct->setShortcuts(QKeySequence::Paste);
    pasteAct->setStatusTip(tr("Pegar"));
    connect(pasteAct, SIGNAL(triggered()), textEdit, SLOT(paste()));

	//Accion de ejecutar simulador
	runAct = new QAction(QIcon(":/images/run.png"), tr("&Run"), this);
    runAct = new QAction(tr("Run"), this);
    runAct->setStatusTip(tr("Ejecuta el simulador"));
    connect(runAct, SIGNAL(triggered()), this, SLOT(run()));

	//Accion que muestra la ventana de informacion acerca de la aplicacin
    aboutAct = new QAction(tr("&About"), this);
    aboutAct->setStatusTip(tr("Muestra la informacion acerca de la aplicacion"));
    connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

	//Accion que muestra informacion acerca de la biblioteca de Qt
    aboutQtAct = new QAction(tr("About &Qt"), this);
    aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
    connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));

	//Se habilitan las opciones copiar y pegar cuando hay texto selecionado
    cutAct->setEnabled(false);
    copyAct->setEnabled(false);
    connect(textEdit, SIGNAL(copyAvailable(bool)),
            cutAct, SLOT(setEnabled(bool)));
    connect(textEdit, SIGNAL(copyAvailable(bool)),
            copyAct, SLOT(setEnabled(bool)));
}

//Funcion que crea la barra de menu principal
void MainWindow::createMenus()
{
	//Pestaa Archivo
    fileMenu = menuBar()->addMenu(tr("&File"));
    fileMenu->addAction(newAct);					//Nuevo
    fileMenu->addAction(openAct);					//Abrir
    fileMenu->addAction(saveAct);					//Guardar
    fileMenu->addAction(saveAsAct);					//Guardar como
	separatorAct = fileMenu->addSeparator();		//Separador
    for (int i = 0; i < MaxRecentFiles; ++i)		//Aadir los archivos mas recientemente abiertos
		fileMenu->addAction(recentFileActs[i]);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAct);					//Cerrar
	updateRecentFileActions();						//Cargar los archivos mas recientes

	//Pestaa editar
    editMenu = menuBar()->addMenu(tr("&Edit"));
    editMenu->addAction(cutAct);					//Cortar
    editMenu->addAction(copyAct);					//Copiar
    editMenu->addAction(pasteAct);					//Pegar

    menuBar()->addSeparator();						

	//Pestaa Simulador
    fileMenu = menuBar()->addMenu(tr("&Simulator"));
    fileMenu->addAction(runAct);					//Run
    fileMenu->addSeparator();						//Separador


	//Pestaa ayuda
    helpMenu = menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(aboutAct);					//Acerca de la aplicacion
    helpMenu->addAction(aboutQtAct);				//Acerca de Qt
}

//Funcion que crea la barra de herramientas
void MainWindow::createToolBars()
{
	//Barra de herramientas archivo
    fileToolBar = addToolBar(tr("File"));
    fileToolBar->addAction(newAct);					//Nuevo
    fileToolBar->addAction(openAct);				//Abrir
    fileToolBar->addAction(saveAct);				//Guardar

	//Barra de herramientas editar
	editToolBar = addToolBar(tr("Edit"));
    editToolBar->addAction(cutAct);					//Cortar
    editToolBar->addAction(copyAct);				//Copiar
    editToolBar->addAction(pasteAct);				//Pegar

	//Barra de herramientas simulador
	editToolBar = addToolBar(tr("Simulator"));
    editToolBar->addAction(runAct);					//Run
}

//Funcion que crea barra de estado
void MainWindow::createStatusBar()
{
    statusBar()->showMessage(tr("Ready"));
}

//Funcion que lee las propiedades del usuario y caracteristicas de la aplicacion
void MainWindow::readSettings()
{
    QSettings settings("UVa", "TFGSimuCircuitosDigitales");
    QPoint pos = settings.value("pos", QPoint(200, 200)).toPoint();
    QSize size = settings.value("size", QSize(400, 400)).toSize();
    resize(size);
    move(pos);
}

//Funcion que reescribe las propiedades del usuario y caracteristicas de la aplicacion
void MainWindow::writeSettings()
{
    QSettings settings("UVa", "TFGSimuCircuitosDigitales");
    settings.setValue("pos", pos());
    settings.setValue("size", size());
}

//Funcion que guarda cambios pendientes
bool MainWindow::maybeSave()
{
    if (textEdit->document()->isModified()) {
        QMessageBox::StandardButton ret;
		//Ventana para decidir si guardar o no los cambios
        ret = QMessageBox::warning(this, tr("Application"),
                     tr("El documento se ha modificado.\n"
                        "Quieres guardar los cambios?"),
                     QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);
        if (ret == QMessageBox::Save)
            return save();
        else if (ret == QMessageBox::Cancel)
            return false;
    }
    return true;
}

//Funcion que carga un archivo existente
void MainWindow::loadFile(const QString &fileName)
{
    QFile file(fileName);
    if (!file.open(QFile::ReadOnly | QFile::Text)) {
        QMessageBox::warning(this, tr("Application"),
                             tr("No se puede cargar el archivo %1:\n%2.")
                             .arg(fileName)
                             .arg(file.errorString()));
        return;
    }

    QTextStream in(&file);
#ifndef QT_NO_CURSOR
    QApplication::setOverrideCursor(Qt::WaitCursor);
#endif
    textEdit->setPlainText(in.readAll());
#ifndef QT_NO_CURSOR
    QApplication::restoreOverrideCursor();
#endif

    setCurrentFile(fileName);
    statusBar()->showMessage(tr("Archivo cargado"), 2000);
}

//Funcion que guarda el archivo
bool MainWindow::saveFile(const QString &fileName)
{
    QFile file(fileName);
    if (!file.open(QFile::WriteOnly | QFile::Text)) {
        QMessageBox::warning(this, tr("Application"),
                             tr("No se puede guardar el archivo %1:\n%2.")
                             .arg(fileName)
                             .arg(file.errorString()));
        return false;
    }

    QTextStream out(&file);
#ifndef QT_NO_CURSOR
    QApplication::setOverrideCursor(Qt::WaitCursor);
#endif
    out << textEdit->toPlainText();
#ifndef QT_NO_CURSOR
    QApplication::restoreOverrideCursor();
#endif

    setCurrentFile(fileName);
    statusBar()->showMessage(tr("Archivo guardado"), 2000);
    return true;
}

//Funcion que resetea el estado de algunas variables
void MainWindow::setCurrentFile(const QString &fileName)
{
    curFile = fileName;
    textEdit->document()->setModified(false);
    setWindowModified(false);

	QSettings settings;
    QStringList files = settings.value("recentFileList").toStringList();
    files.removeAll(fileName);
    files.prepend(fileName);
    while (files.size() > MaxRecentFiles)
        files.removeLast();
    settings.setValue("recentFileList", files);
    foreach (QWidget *widget, QApplication::topLevelWidgets()) {
        MainWindow *mainWin = qobject_cast<MainWindow *>(widget);
        if (mainWin)
            mainWin->updateRecentFileActions();
    }

	//Si no hay archivo en ejecucion lo guarda muestra el nombre untilted.txt
    QString shownName = curFile;
    if (curFile.isEmpty())
        shownName = "untitled.vdo";
    setWindowFilePath(shownName);
}

//Funcion que carga los archivos recientemente cargados
void MainWindow::updateRecentFileActions()
 {
     QSettings settings;
     QStringList files = settings.value("recentFileList").toStringList();

     int numRecentFiles = qMin(files.size(), (int)MaxRecentFiles);

     for (int i = 0; i < numRecentFiles; ++i) {
         QString text = tr("&%1 %2").arg(i + 1).arg(strippedName(files[i]));
         recentFileActs[i]->setText(text);
         recentFileActs[i]->setData(files[i]);
         recentFileActs[i]->setVisible(true);
     }
     for (int j = numRecentFiles; j < MaxRecentFiles; ++j)
         recentFileActs[j]->setVisible(false);

     separatorAct->setVisible(numRecentFiles > 0);
 }

//Funcion que acorta el nombre del archivo
QString MainWindow::strippedName(const QString &fullFileName)
{
    return QFileInfo(fullFileName).fileName();
}

//Funcion con la que llamamos a la funcion del simulador tras apretar RUN
void MainWindow::simulacion(const QString &fileName)
{
    //Guardamos el archivo en la variable FILE
	FILE = fileName;

	//Hacemos el archivo de salida txt de escritura
	fOut.open(QFile::WriteOnly | QFile::Text);

	//Si hay fallo de apertura mostramos
	if (fOut.open(QFile::WriteOnly | QFile::Text)) {	} 
	else 
	{
		ErroresYWarnings("Error! Fallo al abrir el archivo de salida", MSG_Err);
	}
  
	//Mostramos en la barra de estado el estado actual
	statusBar()->showMessage(tr("Simulando el archivo seleccionado"));

	//Limpiamos todo lo que hay en TextProcess
	textProcess->clear();
	textOut->clear();
	
	//Mostramos por pantalla en el cuadro de texto inferior la direccion del archivo que estamos simulando
	QStringList IniProcess;
	QString Null;
	IniProcess << ("//Simulando el archivo con direccion: ");
	IniProcess.append(FILE);
	IniProcess.append("---------------------------------------------");
	textProcess->addItems(IniProcess);


	//Ejecutamos el simulador pasandole el archivo de texto
	LecturaArchivo(FILE);
	//Ejecutamos la funcin que muestra los datos guardados
	MostrarDatosGuardados();
	
	//Ordenamos los Boxes en un QMultiHash para saber que posicin de memoria es de cada TYPE BOX
	OrdenandoBoxes();
	//Llamada a la funcin que detecta cuales son los iteradores de los Boxes PADRE y HEADER
	BoxCabecera();

	//Simulamos en forma declarativa el circuito
	SimulacionDeclarativa();
	//Ejecutamos la funcion que muestra las seales guardadas
	MostrarSignals();

	//Simulamos en forma operativa el circuito
	SimulacionOperativa();

	//Muestro las seales por ciclo
	MostrarSignalPorCicloSimulacion();

	//Muestro verificaciones a realizar
	MostrarVerificacionesARealizar();

	//RealizoVerificaciones
	RealizarVerificaciones();

	QString SimulationGlobalProcess;
	SimulationGlobalProcess.append("\nNumero de ciclos de simulacion: ");
	SimulationGlobalProcess.append(QString::number(CYCLE));

	TokenProcess.append(SimulationGlobalProcess);

	
	//Llama a la funcin que muestra todos los Warning y errores de la simulacion
	ErroresYWarnings(Null, MSG_Lectura);


//	QString Process;
	//Process.append("\n//-----------PRUEBAS EXPRESIONES--------------//\n");
	//QString Frase="inB[0];";
	//ExprPreviaToken(Frase);
	//Process.append(QString::number(NumDec));
	//Process.append("\n");
	//Frase="inB[1];";
	//ExprPreviaToken(Frase);
	//Process.append(QString::number(NumDec));
	//Process.append("\n");
	//Frase="inB[2];";
	//ExprPreviaToken(Frase);
	//Process.append(QString::number(NumDec));
	//Process.append("\n");
	//Frase="inB[3];";
	//ExprPreviaToken(Frase);
	//Process.append(QString::number(NumDec));
	//Process.append("\n");
	//Frase="inB[8];";
	//ExprPreviaToken(Frase);
	//Process.append(QString::number(NumDec));
	//Process.append("\n");
	//Frase="2'D16769025;";
	//ExprPreviaToken(Frase);
	//Process.append(QString::number(NumDec));
	//Process.append("\n");
	//Frase="2'O43;";
	//ExprPreviaToken(Frase);
	//Process.append(QString::number(NumDec));
	//Process.append("\n");
	//Frase="2'B001010;";
	//ExprPreviaToken(Frase);
	//Process.append(QString::number(NumDec));
	//Process.append("\n");
	////TokenProcess.append(Process);
	//textOut->addItems(TokenProcess);
	TokenProcess.clear();
	//QString hola="hola";
	//QStringList yoyo;
	//yoyo.append(hola);

	//for (int i = 0; i < yoyo.size(); ++i)
	//{
 //     salidaTXT << yoyo.at(i) << '\n';
	//}
	fOut.close();
}

//Funcion para escribir en el txt de salida.
void MainWindow::TXTOut(QStringList ListOut)
{
	for (int i = 0; i < ListOut.size(); ++i)
	{
      salidaTXT << ListOut.at(i) << "\n";
	}
}

////////////////////////////////////////////////////////////////////////

//Funcin que recibe un String con un error o un warning y los almacena en class Fallos
//( tipo 0 = lectura, tipo 1 = Error, tipo 2 = Warning) 
void MainWindow::ErroresYWarnings(QString ErrWar, int tipo)
{
	int NumWar=0, NumErr=0;
	QString Warnings;
	QString Errores;
	QStringList ErrProcess, WarProcess, WarErrProcess;

	ErrProcess.clear();
	WarProcess.clear();
	WarErrProcess.clear();

	//LECTURA
	//Simplemente se muestran por pantalla todos los errores y warnings
	if (tipo == 0)
	{	
		WarErrProcess.append("\n//-------------- WARNINGS Y ERRORES --------------//");
		textOut->addItems(WarErrProcess);
		TXTOut(WarErrProcess);
		WarErrProcess.clear();

		NumErr=FALLOS.Errores.size();
		NumWar=FALLOS.Warnings.size();
		if (NumWar!=0)
		{
			for(int i=0; i<NumWar; i++)
			{
				WarProcess.append(FALLOS.Warnings.at(i));
			}
		}
		if (NumErr!=0)
		{
			for(int j=0; j<NumErr; j++)
			{
				ErrProcess.append(FALLOS.Errores.at(j));
			}
		}

		Errores=("Numero de Errores: ");
		Errores.push_back(QString::number(NumErr));
		Warnings=("Numero de Warnings: ");
		Warnings.push_back(QString::number(NumWar));

		WarErrProcess.append(Warnings);
		WarErrProcess.append(Errores);

		textOut->addItems(WarProcess);
		TXTOut(WarProcess);
		textOut->addItems(ErrProcess);
		TXTOut(ErrProcess);
		textOut->addItems(WarErrProcess);
		TXTOut(WarErrProcess);
	}
	//ERROR
	if (tipo == 1)
	{
		FALLOS.Errores.push_back(ErrWar);
	}
	//WARNING
	if (tipo == 2)
	{
		FALLOS.Warnings.push_back(ErrWar);
	}
}

//Funcin encargada de vaciar la memoria totalmente
void LimpiarMemoria()
{
	NumDec=0;														
	HayComparacion=0;
	HayLogica=0;										
	ComparacionExpresiones=false;										
	NameSignalTOKEN.clear();

	//-----------------------------------CLASS SIGNAL-----------------------------------------//

	SignalClass.NombreSignal.clear();
	SignalClass.CaractSignal.clear();
	SignalClass.OutputList.clear();
	SignalClass.InputList.clear();
	SignalClass.InoutList.clear();
	SignalClass.WireList.clear();
	SignalClass.CaractInput.clear();
	SignalClass.CaractOutput.clear();
	SignalClass.CaractInout.clear();
	SignalClass.CaractWire.clear();
	SignalClass.SyncInput.clear();
	SignalClass.SyncOutput.clear();
	SignalClass.SyncInout.clear();
	SignalClass.SyncWire.clear();
	SignalClass.InputValor.clear();
	SignalClass.OutputValor.clear();
	SignalClass.InoutValor.clear();
	SignalClass.WireValor.clear();
	SignalClass.InputValorAnt.clear();
	SignalClass.OutputValorAnt.clear();
	SignalClass.InoutValorAnt.clear();
	SignalClass.WireValorAnt.clear();
	SignalClass.InputValorDesp.clear();
	SignalClass.OutputValorDesp.clear();
	SignalClass.InoutValorDesp.clear();
	SignalClass.WireValorDesp.clear();
	SignalClass.SignalUPEvent.clear();
	SignalClass.SignalUPEventValor.clear();
	SignalClass.SignalDOWNEvent.clear();
	SignalClass.SignalDOWNEventValor.clear();
	SignalClass.SignalDefaultName.clear();
	SignalClass.SignalDefaultValor.clear();

	//------------------------------------CLASS PARAMETRO---------------------------------------//

	ParametroToken.FraseParametro.clear();
	ParametroToken.NumParametro=0;
	ParametroToken.VariableList.clear();

	//------------------------------------CLASS BOX--------------------------------------//
	BOX.ID.clear();
	BOX.TYPE.clear();
	BOX.TEXTMatrix.clear();
	BOX.TEXTUPMatrix.clear();
	BOX.TEXTDOWNMatrix.clear();
	BOX.NEXT.clear();
	BOX.NEXT0.clear();
	BOX.NEXT1.clear();
	BOX.COMMENT.clear();
	BOX.TOUCH.clear();

	//-----------------------------------CLASS PAGES--------------------------------------//
	PAGES.ID.clear();
	PAGES.NAME.clear();

	//---------------------------------CLASS ORDENBOX---------------------------------------//
	ORDENBOX.IterBox=0;
	ORDENBOX.IdCabeceras.clear();
	ORDENBOX.ItCabeceras.clear();
	ORDENBOX.POSICION.clear();
	ORDENBOX.TypeCabeceras.clear();
	ORDENBOX.IDStatePadresMatrix.clear();
	ORDENBOX.ITStatePadresMatrix.clear();

	//---------------------------------CLASS TESTBENCH---------------------------------------//
	TESTBENCH.Nombre.clear();
	TESTBENCH.Cantidad=0;

	//-------------------------------CLASS VERIFICACIONES---------------------------------------//
	VERIFICACIONES.Ciclo.clear();
	VERIFICACIONES.NameSignal.clear();
	VERIFICACIONES.Valor.clear();

	//-------------------------------CLASS DATOSSIMULACIONES---------------------------------------//
	SIMULACION.SignalSim.clear();
	SIMULACION.ValoresSim.clear();

	//-----------------------------------CLASS FALLOS---------------------------------------------//
	FALLOS.Errores.clear();
	FALLOS.Warnings.clear();


}

//Esta funcion recibe un archivo de texto, lo abre, lo recorre interpretando cada BOX y cada linea de este. Toda la informacin la almacena en
//las diferentes variables.
void MainWindow::LecturaArchivo(const QString &FILE)
{
	int longline=200;		//Variable para definir la longitud de cada linea de lectura

	//Vamos aadiendo cosas a la lista SimProcess
	SimProcess << ("//Leyendo, interpretando y guardando el archivo de texto en la memoria...");
	SimProcess.append("---------------------------------------------");
	textProcess->addItems(SimProcess);
	SimProcess.clear();

	//Mensaje para la ventana de salida
	OutProcess.append("//--------------------------ARCHIVO DE SALIDA----------------------------//");
	textOut->addItems(OutProcess);
	TXTOut(OutProcess);
	OutProcess.clear();

	//Llamamos a la funcin que limpia la memoria para leer un nuevo archivo
	LimpiarMemoria();

	//Variables "globales" de la funcion
	iteradorBox=0;
	int iteradorPages=0;
	int filT=0, colT=0;
	int filU=0, colU=0;
	int filD=0, colD=0;

	//Inicializa la matriz en la que se van a guardar todos los textos. Matriz 10x300 con todas las casillas null
	MatrizTexto();
	MatrizTextoUp();
	MatrizTextoDown();
	
	//Abrimos el archivo con la direccion de FILE seleccionada en las funciones anteriores
	QFile archivo(FILE);
    archivo.open(QIODevice::ReadOnly);

	//Establecemos la secuencia de texto para leer
    QTextStream textoLeer(&archivo);

	//Creamos diferentes strings para diferenciar entre las diferentes secciones y cajas
	QString Pages="Pages";			//Paginas 
	QString Box="Box";				//Box (cajas)
	QString IDPages="ID";			//ID Pagina
	QString NamePages="Name";		//Nombre Pagina

	//Hacemos un bucle que vaya leyendo las lineas de texto desde la primera (i=1) hasta encontrar la palabra Pages
	int i=1, NULO=0;						//Variables para el bucle DO-While
	int Comentario;
	
	//Creamos un string para cada linea
	QString LineaTexto;

	//Hacemos un bucle Do-While leyendo en cada pasada una linea desde i=1 hasta while
	do{
		//Aadimos la linea en el texto de proceso que o estara vacia si es la primera iteracion
		//o tendra toda la iteracion anterior
		if (LineaTexto.isNull())
		{
			//Salimos del bucle cuando hemos acabado de leer el archivo. Para ello hacemos un contador
			//y si nos encontramos 100 frases seguidas nulas se supone que se ha acabado. Se manda un valor de i
			//superior al de la condicin y se sale
			NULO=NULO+1;
			if(NULO==100)
			{
				i=100001;
			}
		}
		else
		{
			//Si la frase no esta nula no hemos acabado, y se reinicia la cuenta de nulo
			NULO=0;
			
			//Se comprueba si es un comentario
			Comentario=GetToken(LineaTexto,false);
			if(Comentario==TOKEN_COMMENT)
			{
				//Mostramos toda la iteracin anterior
				SimProcess.append(LineaTexto);
				//Mostramos la Lista de proceso
				textProcess->addItems(SimProcess);
			}
			
			//Limpiamos la lista de proceso
			SimProcess.clear();
		}

		//Leemos la linea de 75 caracteres y guardamos en el QString
		LineaTexto=textoLeer.readLine(longline);
		int LlaveFinalBox=LineaTexto.indexOf("}",Qt::CaseInsensitive);
		if(LlaveFinalBox<3)
		{
			SimProcess.clear();
		}
		//Leemos una linea. Si es un modulo Pages o Box y lo analizamos
		int page = LineaTexto.indexOf(Pages,Qt::CaseInsensitive);
		int box = LineaTexto.indexOf(Box,Qt::CaseInsensitive);
		
		//Si encontramos un modulo Pages le analizamos
		//Sera del tipo:
		//	Pages{
		//		Id= ; Name= ;
		//	}
		if (page != -1)
		{
			//Mensaje para la ventana de salida
			//OutProcess.append("//-----------------Nuevo Pages encontrado---------------//");
			textOut->addItems(OutProcess);
			TXTOut(OutProcess);
			OutProcess.clear();

			//Mensaje para la ventana de Proceso
			SimProcess.append("---------------------------------------------");
			QString PalabraPages="Palabra Pages encontrada en la linea: ";
			PalabraPages.insert(38,Pages.number(i));
			SimProcess.append(PalabraPages);
			//Aumentamos la busqueda en una linea para guardar el Id y el Nombre de la pgina
			SimProcess.append("---------------------------------------------");
			
			//Leemos la linea de 75 caracteres y guardamos en el QString
			LineaTexto=textoLeer.readLine(longline);
			i++;
			
			//Guardamos en la clase Pages el ID y el Nombre
			QString IdentPages, NamePages;
			int idPages;
			bool ok;

			//Secionamos ambos
			IdentPages=LineaTexto;
			NamePages=LineaTexto;
			IdentPages=IdentPages.section(";",0,0);
			NamePages=NamePages.section(";",1,1);
			IdentPages=IdentPages.section("=",1);
			NamePages=NamePages.section("=",1);

			//Cnvertimos el id en un int y guardamos ambos
			idPages=IdentPages.toInt(&ok, 10);
			PAGES.ID.push_back(idPages);
			PAGES.NAME.push_back(NamePages);
			
			////Aadimos la linea en el texto de proceso
			QString PagesID, PagesName;
			PagesID="Id encontrado. Linea:  . ID = ";
			PagesID.insert(22,Box.number(i));
			PagesID.push_back(Box.number(PAGES.ID.at(iteradorPages)));
			PagesName="Name encontrado. Linea:  . NAME = ";
			PagesName.insert(24,Box.number(i));
			PagesName.push_back(PAGES.NAME.at(iteradorPages));
			SimProcess.append(PagesID);
			SimProcess.append(PagesName);
			SimProcess.append("---------------------------------------------");

			//Leemos la siguiente linea de 75 caracteres y si es un } salimos
			i++;
			LineaTexto=textoLeer.readLine(longline);
			i++;
			int EndCorch = LineaTexto.indexOf("}",Qt::CaseInsensitive);
			if (EndCorch!=-1)
			{
				SimProcess.append("Fin del modulo Pages");
				SimProcess.append("---------------------------------------------");
			}

			//Como salimos del modulo Pages, aumentamos el iterador para la proxima vez guardar en otra posicion del vector
			iteradorPages++;
			//Mostramos la Lista de proceso
			textProcess->addItems(SimProcess);
			//Limpiamos la lista de proceso
			SimProcess.clear();
		}

		//Si encontramos un modulo Box lo analizamos
		else if (box != -1)
		{
			int itBox;

			/****************Declarative section*******************/
			//Creamos diferentes strings para diferenciar entre los diferentes TYPE de cajas de esta zona
			QString Header="Header";				//Cabecera
			QString Ports="Ports";					//Ports
			QString Code="Code";					//Code
			QString ThreadSync="ThreadSync";		//ThreadSync
			QString Event="Event";					//Event
			QString Defaults="Default";			    //Default
			QString Instance="Instance";			//Instance
			QString MetaState="MetaState";			//MetaState
			QString StateAsyncOps="StateAsyncOps";	//StateAsyncOps
			QString Initial="Initial";				//Initial
			QString Decision="Decision";			//Decision
			QString State="State";					//State
			QString CondSyncOps="CondSyncOps";		//CondSyncOps
			QString	CondAsyncOps="CondAsyncOps";	//CondAsyncOps
			QString SyncOps="SyncOps";				//SyncOps
			QString	AsyncOps="AsyncOps";			//AsyncOps


			//Creamos diferentes strings para diferenciar entre los diferentes apartados de las cajas
			QString Text="Text";			//Text
			QString TextUp="TextUp";		//TextUp
			QString TextDown="TextDown";	//TextDown
			QString Next="Next";			//Next

			
			int DeclarativeSection;		//Definimos una variable para el switch del tipo de Box

			//////////////////////////////// Definiciones ID ////////////////////////////////
			QString Identificador;		//String para guardar el ID una vez seccionado
			QString PalabraID;			//String para mostrar por pantalla un texto + ID
			int id;
			bool ok;

			//////////////////////////////// Definiciones TYPE ////////////////////////////////
			int header, ports, code, threadsync, events, defaults, instance, metastate;		//Variables para definir el TYPE
			int stateAsyncOps, state, decision, condsyncOps, condasyncOps, syncOps, asyncOps, initial;	//Variables para definir el TYPE
			
			QString Expresion;	//String para guardar la expresion 

			//Strings para mostras frases por pantalla de la zona en la que nos encontramos dentro del Box
			QString TextoBox;
			QString CajaHeader, BoxPorts, BoxEvent, BoxThreadSync, BoxCode, BoxDefaults, BoxInstance, BoxMetaState;
			QString BoxStateAsyncOps, BoxInitial, BoxDecision, BoxState, BoxSyncOps, BoxAsyncOps, BoxCondSyncOps, BoxCondAsyncOps;
			QString BoxError;

			//////////////////////////////// Definiciones TEXT ////////////////////////////////
			QString Final, RESULT, EXPRESION;			//String para  mostrar el resultado
			QString TipoTexto, TypeTextoBox;
			int UP, DOWN, lengLineaTexto, k, lengExpresion, l, finFrase, m;
			int END, ENDSIMU;		//Ints para saber si el texto es el final en un metastate

			//////////////////////////////// Definiciones NEXT ////////////////////////////////
			QString SiguienteBox;	//String para guardar el next una vez seccionado
			QString NextTextBox;	//String para mostrar texto dentro del next
			int siguiente;			//Int con el valor del next

			BOX.NEXT.insert(iteradorBox,-1); 
			BOX.NEXT0.insert(iteradorBox,-1); 
			BOX.NEXT1.insert(iteradorBox,-1); 

			//////////////////////////////// Definiciones Llave Fin ////////////////////////////////
			QString LlaveTextBox;

			//Mensaje para la ventana de salida
			//OutProcess.append("//-----------------Nuevo Box encontrado---------------//");
			textOut->addItems(OutProcess);
			TXTOut(OutProcess);
			OutProcess.clear();

			//Mostramos por pantalla la linea en la que hemos encontrado una nueva caja box
			SimProcess.append("---------------------------------------------");
			QString PalabraBox("Palabra Box encontrada en la linea: ");
			PalabraBox.insert(36,Box.number(i));
			SimProcess.append(PalabraBox);
			SimProcess.append("---------------------------------------------");
			textProcess->addItems(SimProcess);
			SimProcess.clear();

			//Leemos la linea de 75 caracteres siguiente y comprobamos que hemos encontrado dentro de BOX
			LineaTexto=textoLeer.readLine(longline);
			i++;
		
			for(itBox=0;itBox<2000;itBox++)
			{
				//PODEMOS ENCONTRARNOS DIVERSAS COSAS:
				//	- ESPACIO	(0)
				//	- ID		(1)
				//	- TYPE		(2)
				//	- TEXT		(3)
				//	- NEXT		(4)
				//	- }			(5)

				//Buscamos en la linea diferentes palabras para saber que nos encontramos
				int IDBox=LineaTexto.indexOf("Id",Qt::CaseInsensitive);
				int TypeBox=LineaTexto.indexOf("Type",Qt::CaseInsensitive);
				int TextBox=LineaTexto.indexOf("Text",Qt::CaseInsensitive);
				int NextBox=LineaTexto.indexOf("Next",Qt::CaseInsensitive);
				int LlaveBox=LineaTexto.indexOf("}",Qt::CaseInsensitive);

				int NextZero=0, NextOne=0;


				//Dependiendo lo que hayamos, modificamos la variable PalabraEnBox y analizamos el caso
				int PalabraEnBox=0;
				if(IDBox!=-1) { PalabraEnBox=1; }
				else if(TypeBox!=-1) { PalabraEnBox=2; }
				else if(TextBox!=-1) { PalabraEnBox=3; }
				else if(NextBox!=-1) { PalabraEnBox=4; }
				else if(LlaveBox!=-1) { PalabraEnBox=5; }

				int SaltoLinea,CR,cuantosSaltosLinea;
				int j;
				QString exprPrevia;

				switch (PalabraEnBox)
				{
					case 0:
						//////////////////////////////// ESPACIO ////////////////////////////////
						//Leemos la siguiente linea de caracteres para saber que nos encontramos a continuacion en el BOX
						LineaTexto=textoLeer.readLine(longline);
						i++;
						break;
					case 1:
						//////////////////////////////// ID ////////////////////////////////
						//Guardamos el ID en la class Box, en la posicin iteradorBox de los vectores
						Identificador=LineaTexto.section(";",0,0);
						Identificador=Identificador.section("=",1);
						id=Identificador.toInt(&ok, 10);
						BOX.ID.push_back(id);


						//Mostramos en el cuadro de Proceso
						PalabraID="Id encontrado. Linea:  . ID = ";
						PalabraID.insert(22,Box.number(i));
						PalabraID.push_back(Box.number(BOX.ID.at(iteradorBox)));
						SimProcess.append(PalabraID);
						SimProcess.append("---------------------------------------------");

						//Leemos la siguiente linea de caracteres para saber que nos encontramos a continuacion en el BOX
						LineaTexto=textoLeer.readLine(longline);
						i++;
						break;
					case 2:
						//////////////////////////////// TYPE ////////////////////////////////
						//Diferenciamos entre los diferentes TYPE de cajas
						header = LineaTexto.indexOf(Header,Qt::CaseInsensitive);
						ports = LineaTexto.indexOf(Ports,Qt::CaseInsensitive);
						code = LineaTexto.indexOf(Code,Qt::CaseInsensitive);
						threadsync = LineaTexto.indexOf(ThreadSync,Qt::CaseInsensitive);
						events = LineaTexto.indexOf(Event,Qt::CaseInsensitive);
						defaults = LineaTexto.indexOf(Defaults,Qt::CaseInsensitive);
						instance = LineaTexto.indexOf(Instance,Qt::CaseInsensitive);
						metastate = LineaTexto.indexOf(MetaState,Qt::CaseInsensitive);
						stateAsyncOps = LineaTexto.indexOf(StateAsyncOps,Qt::CaseInsensitive);
						state = LineaTexto.indexOf(State,Qt::CaseInsensitive);
						decision = LineaTexto.indexOf(Decision,Qt::CaseInsensitive);
						condsyncOps = LineaTexto.indexOf(CondSyncOps,Qt::CaseInsensitive);
						condasyncOps = LineaTexto.indexOf(CondAsyncOps,Qt::CaseInsensitive);
						syncOps = LineaTexto.indexOf(SyncOps,Qt::CaseInsensitive);
						asyncOps = LineaTexto.indexOf(AsyncOps,Qt::CaseInsensitive);
						initial = LineaTexto.indexOf(Initial,Qt::CaseInsensitive);

						//Si encontramos la cabecera procedemos a analizarla
						//La cabecera tendra fijo: Type, Next y }. Si ademas tiene Text se analiza 
						if (header!=-1)
						{
							//TYPE: Mostramos el tipo de caja
							CajaHeader="Caja tipo Header. Linea: ";
							CajaHeader.insert(25,Box.number(i));
							SimProcess.append(CajaHeader);
							SimProcess.append("---------------------------------------------");

							//Guardamos en la clase de informacin de BOX
							BOX.TYPE.push_back(Header);
						}
						//Una vez pasada la caja Header, pueden aparecer diversas cajas
						else
						{
							//Inicializamos la variable del Switch
							DeclarativeSection=0;
							//Le damos un valor u otro dependiendo el tipo de Box
							if (ports!=-1){ DeclarativeSection=1; }
							else if (code!=-1) { DeclarativeSection=2; }
							else if (threadsync!=-1){ DeclarativeSection=3; }
							else if (events!=-1) { DeclarativeSection=4; }
							else if (defaults!=-1){ DeclarativeSection=5; }
							else if (instance!=-1) { DeclarativeSection=6; }
							else if (metastate!=-1) {DeclarativeSection=7; }
							else if (stateAsyncOps!=-1) { DeclarativeSection=8; }
							else if (state!=-1){ DeclarativeSection=9; }
							else if (decision!=-1) { DeclarativeSection=10; }
							else if (condasyncOps!=-1) { DeclarativeSection=11; }
							else if (condsyncOps!=-1) {DeclarativeSection=12; }
							else if (asyncOps!=-1) { DeclarativeSection=13; }
							else if (syncOps!=-1) {DeclarativeSection=14; }
							else if (initial!=-1) {DeclarativeSection=15; }

							//Switch dependiendo del type
							switch (DeclarativeSection)
							{
								case 1:
									//PORTS
									BoxPorts="Box Type Ports. Linea: ";
									BoxPorts.push_back(QString::number(i));
									SimProcess.append(BoxPorts);
									SimProcess.append("---------------------------------------------");
									
									//Guardamos en la clase de informacin de BOX
									BOX.TYPE.push_back(Ports);
									break;
								case 2:
									//CODE
									BoxCode=("Box Type Code. Linea: ");
									BoxCode.push_back(QString::number(i));
									SimProcess.append(BoxCode);
									SimProcess.append("---------------------------------------------");
									
									//Guardamos en la clase de informacin de BOX
									BOX.TYPE.push_back(Code);
									break;
								case 3:
									//THREADSYNC
									BoxThreadSync=("Box Type ThreadSync. Linea: ");
									BoxThreadSync.push_back(QString::number(i));
									SimProcess.append(BoxThreadSync);
									SimProcess.append("---------------------------------------------");
									
									//Guardamos en la clase de informacin de BOX
									BOX.TYPE.push_back(ThreadSync);
									break;
								case 4:
									//EVENT
									BoxEvent=("Box Type Event. Linea: ");
									BoxEvent.push_back(QString::number(i));
									SimProcess.append(BoxEvent);
									SimProcess.append("---------------------------------------------");
									
									//Guardamos en la clase de informacin de BOX
									BOX.TYPE.push_back(Event);
									break;
								case 5:
									//DEFAULT
									BoxDefaults=("Box Type Default. Linea: ");
									BoxDefaults.push_back(QString::number(i));
									SimProcess.append(BoxDefaults);
									SimProcess.append("---------------------------------------------");
									
									//Guardamos en la clase de informacin de BOX
									BOX.TYPE.push_back(Defaults);
									break;
								case 6:
									//INSTANCE
									BoxInstance=("Box Type Instance. Linea: ");
									BoxInstance.push_back(QString::number(i));
									SimProcess.append(BoxInstance);
									SimProcess.append("---------------------------------------------");
									
									//Guardamos en la clase de informacin de BOX
									BOX.TYPE.push_back(Instance);
									break;
								case 7:
									//META STATE
									BoxMetaState=("Box Type Meta State. Linea: ");
									BoxMetaState.push_back(QString::number(i));
									SimProcess.append(BoxMetaState);
									SimProcess.append("---------------------------------------------");
									
									//Guardamos en la clase de informacin de BOX
									BOX.TYPE.push_back(MetaState);
									break;
								case 8:
									//STATE ASYNC OPS
									BoxStateAsyncOps=("Box Type StateAsyncOps. Linea: ");
									BoxStateAsyncOps.push_back(QString::number(i));
									SimProcess.append(StateAsyncOps);
									SimProcess.append("---------------------------------------------");
									
									//Guardamos en la clase de informacin de BOX
									BOX.TYPE.push_back(StateAsyncOps);
									break;
								case 9:
									//STATE
									BoxState=("Box Type State. Linea: ");
									BoxState.push_back(QString::number(i));
									SimProcess.append(BoxState);
									SimProcess.append("---------------------------------------------");
									
									//Guardamos en la clase de informacin de BOX
									BOX.TYPE.push_back(State);
									break;
								case 10:
									//DECISION
									BoxDecision=("Box Type Decision. Linea: ");
									BoxDecision.push_back(QString::number(i));
									SimProcess.append(BoxDecision);
									SimProcess.append("---------------------------------------------");
									
									//Guardamos en la clase de informacin de BOX
									BOX.TYPE.push_back(Decision);
									break;
								case 11:
									//CONDASYNCOPS
									BoxCondAsyncOps=("Box Type CondAsyncOps. Linea: ");
									BoxCondAsyncOps.push_back(QString::number(i));
									SimProcess.append(BoxCondAsyncOps);
									SimProcess.append("---------------------------------------------");
									
									//Guardamos en la clase de informacin de BOX
									BOX.TYPE.push_back(CondAsyncOps);
									break;
								case 12:
									//CONDSYNCOPS
									BoxCondSyncOps=("Box Type CondSyncOps. Linea: ");
									BoxCondSyncOps.push_back(QString::number(i));
									SimProcess.append(BoxCondSyncOps);
									SimProcess.append("---------------------------------------------");
									
									//Guardamos en la clase de informacin de BOX
									BOX.TYPE.push_back(CondSyncOps);
									break;
								case 13:
									//ASYNCOPS
									BoxAsyncOps=("Box Type AsyncOps. Linea: ");
									BoxAsyncOps.push_back(QString::number(i));
									SimProcess.append(BoxAsyncOps);
									SimProcess.append("---------------------------------------------");
									
									//Guardamos en la clase de informacin de BOX
									BOX.TYPE.push_back(AsyncOps);
									break;
								case 14:
									//SYNCOPS
									BoxSyncOps=("Box Type SyncOps. Linea: ");
									BoxSyncOps.push_back(QString::number(i));
									SimProcess.append(BoxSyncOps);
									SimProcess.append("---------------------------------------------");
									
									//Guardamos en la clase de informacin de BOX
									BOX.TYPE.push_back(SyncOps);
									break;
								case 15:
									//INITIAL
									BoxInitial=("Box Type Initial. Linea: ");
									BoxInitial.push_back(QString::number(i));
									SimProcess.append(BoxInitial);
									SimProcess.append("---------------------------------------------");
									
									//Guardamos en la clase de informacin de BOX
									BOX.TYPE.push_back(Initial);
									break;
								default:
									//ERROR
									BoxError=("ERROR! TYPE Box no implementado. Linea: ");
									BoxError.push_back(QString::number(i));
									SimProcess.append(BoxError);
									SimProcess.append("---------------------------------------------");
									break;
							}
						}
						//Leemos la siguiente linea de caracteres para saber que nos encontramos a continuacion en el BOX
						LineaTexto=textoLeer.readLine(longline);
						i++;
						break;
					case 3:
						//////////////////////////////// TEXT ////////////////////////////////
						//Podemos encontrarnos 3 tipos de texto: UP, DOWN o Normal
						//Buscamos las palabras para saber que tipo de texto son
						TipoTexto=LineaTexto.section('"',0,0);
						UP=TipoTexto.indexOf("Up",Qt::CaseInsensitive);
						DOWN=TipoTexto.indexOf("Down",Qt::CaseInsensitive);

						//Reiniciamos las filas
						filT=0;
						filU=0;
						filD=0;
						//Limpiamos el string con el Resultado
						RESULT.clear();
						EXPRESION.clear();

						if (UP!=-1) { TypeTextoBox=("TextoUp "); }
						else if (DOWN!=-1) { TypeTextoBox=("TextoDown "); }
						else { TypeTextoBox=("Texto "); }

						//Dependiendo del tipo de caja encontrada mostramos en que linea nos hemos encontrado su texto
						if (header!=-1) { TextoBox=("de la caja HEADER analizandose. Linea: "); }
						else if (DeclarativeSection==1) { TextoBox=("de la caja PORTS analizandose. Linea: "); }
						else if (DeclarativeSection==2) { TextoBox=("de la caja CODE analizandose. Linea: "); }
						else if (DeclarativeSection==3) { TextoBox=("de la caja THREADSYNC analizandose. Linea: "); }
						else if (DeclarativeSection==4) { TextoBox=("de la caja EVENTS analizandose. Linea: "); }
						else if (DeclarativeSection==5) { TextoBox=("de la caja DEFAULTS analizandose. Linea: "); }
						else if (DeclarativeSection==6) { TextoBox=("de la caja INSTANCE analizandose. Linea: "); }
						else if (DeclarativeSection==7) { TextoBox=("de la caja METASTATE analizandose. Linea: "); }
						else if (DeclarativeSection==8) { TextoBox=("de la caja STATEASYNCOPS analizandose. Linea: "); }
						else if (DeclarativeSection==9) { TextoBox=("de la caja STATE analizandose. Linea: "); }
						else if (DeclarativeSection==10) { TextoBox=("de la caja DECISION analizandose. Linea: "); }
						else if (DeclarativeSection==11) { TextoBox=("de la caja CONDASYNCOPS analizandose. Linea: "); }
						else if (DeclarativeSection==12) { TextoBox=("de la caja CONDSYNCOPS analizandose. Linea: "); }
						else if (DeclarativeSection==13) { TextoBox=("de la caja ASYNCOPS analizandose. Linea: "); }
						else if (DeclarativeSection==14) { TextoBox=("de la caja SYNCOPS analizandose. Linea: "); }
						else if (DeclarativeSection==15) { TextoBox=("de la caja INITIAL analizandose. Linea: "); }

						//Aadimos a la frase con el tipo de texto el tipo de caja y la linea y mostramos
						TextoBox.push_back(Box.number(i));
						TypeTextoBox.push_back(TextoBox);
						SimProcess.append(TypeTextoBox);

						//Como solo queremos la expresin de despues de text, que sera del tipo ( Text = "a<=5"; )
						//leemos digito a digito del texto hasta encontrar una comilla. lo que nos encontremos antes lo borramos
						lengLineaTexto=LineaTexto.length();
						finFrase=0;
						for(k=0;k<=lengLineaTexto;k++)
						{
							if(LineaTexto[k] == '"')
							{
								//Se borran las comillas iniciales
								Expresion=LineaTexto.remove(0,k+1);
								//Se buscan las siguientes comillas
								lengExpresion=Expresion.length();
								for(l=0;l<=lengExpresion;l++)
								{
									if (Expresion[l] == '"')	
									{ 
										Expresion=Expresion.section('"',0,0);	
										finFrase=1;
										l=lengExpresion;
									}
								}
								k=lengLineaTexto;
							}
						}

OTRA:
						//Ya tenemos la frase. La guardamos
						//Comprobamos si es el texto indicador del final de un metastate
						END=Expresion.indexOf("end",Qt::CaseInsensitive);
						ENDSIMU=Expresion.indexOf("end simulation",Qt::CaseInsensitive);
						
						//Si tenemos end o end simulation lo guardamos y no hacemos mas
						if (END!=-1 || ENDSIMU!=-1)
						{

							//Dependiendo el tipo de texto que sea
							if (UP!=-1)
							{
								if (ENDSIMU!=-1) 
								{ BOX.TEXTUPMatrix[filU][colU].replace("null","end simulation"); }
								else { BOX.TEXTUPMatrix[filU][colU].replace("null","end"); }
							}
							else if (DOWN!=-1)
							{
								if (ENDSIMU!=-1) { BOX.TEXTDOWNMatrix[filD][colD].replace("null","end simulation"); }
								else { BOX.TEXTDOWNMatrix[filD][colD].replace("null","end"); }
							}
							else
							{
								if (ENDSIMU!=-1) { BOX.TEXTMatrix[filT][colT].replace("null","end simulation"); }
								else { BOX.TEXTMatrix[filT][colT].replace("null","end"); }
							}
						}
						else
						{
							//Comprobamos si hay saltos de linea
							SaltoLinea=Expresion.indexOf("%CR%",Qt::CaseInsensitive);
							//Si tenemos los contamos
							if(SaltoLinea!=-1) { cuantosSaltosLinea=Expresion.count("%CR%"); }
							else { cuantosSaltosLinea=0; }
							CR=0;

							//Dependiendo el numero de saltos de linea hacemos un bucle. Si no hay solo lo haremos 1 vez
							for(j=-1;j<cuantosSaltosLinea;j++)
							{
								//Si hay salto de linea secciona y guarda
								if(cuantosSaltosLinea!=0)
								{
									exprPrevia=Expresion.section("%CR%",CR,CR);
									EXPRESION=exprPrevia;
									CR++;
									//ExprPreviaToken(exprPrevia,false);
								}
								//Si no hay manda
								else 
								{ 
									EXPRESION=Expresion;
									//ExprPreviaToken(Expresion,false); 
								}

								////Dependiendo lo que sea guardamos un resultado u otro
								//if (ValorConstant.haySignal==1) 
								//{
								//	//Si hay seales no muestra nada
								//	RESULT=(ValorConstant.Signals); 
								//}
								//else if (ValorConstant.hayInterrogante==1)
								//{
								//	//Si hay interrogante, hay que mostrar si la condicin es verdadera o falsa y el resultado
								//	RESULT=(Expresion.number(ValorConstant.decimal));
								//}
								//else if (ValorConstant.haylogica==1)
								//{
								//	//Si hay logica && o ||
								//	if (ValorConstant.logicaexpr==true) { RESULT=("Log. True"); }
								//	else if(ValorConstant.logicaexpr==false) { RESULT=("Log. False"); }
								//}
								//else if (ValorConstant.haycomparacion==1)
								//{
								//	//Si hay algun tipo de comparacin ==,<,>,>=,<=...
								//	if (ValorConstant.comparacion==true) { RESULT=("Comp. True"); }
								//	else if(ValorConstant.comparacion==false) { RESULT=("Comp. False"); }
								//}
								//else if (ValorConstant.haylogica==0 && ValorConstant.haycomparacion==0)
								//{
								//	//Si no hay ni una ni otra
								//	RESULT=(Expresion.number(ValorConstant.decimal));
								//}

								//Guardamos en la posicion de la matriz de textos dependiendo si es UP, DOWN o Normal
								if (UP!=-1) { BOX.TEXTUPMatrix[filU][colU].replace("null",EXPRESION); }
								else if (DOWN!=-1) { BOX.TEXTDOWNMatrix[filD][colD].replace("null",EXPRESION); }
								else { BOX.TEXTMatrix[filT][colT].replace("null",EXPRESION); }

								//BOX.TEXTMatrix[filT][colT].replace("null",RESULT);
								filT++;
								filU++;
								filD++;
								
								//Mostramos por pantalla el resultado
								Final=("Texto: ");
								Final.push_back(EXPRESION);
								SimProcess.append(Final);
							}
						}
						
						//Si habiamos encontrado el " final, la frase acaba en esa linea y podemos salir del switch TEXT
						if (finFrase == 1)
						{
							SimProcess.append("---------------------------------------------");
							//Leemos la siguiente linea de caracteres para saber que nos encontramos a continuacion en el BOX
							LineaTexto=textoLeer.readLine(longline);
							i++;
							break;
						}
						else
						{
							//Como la frase aun sigue se repite hasta encontrar los " finales
							LineaTexto=textoLeer.readLine(longline);
							i++;
							lengLineaTexto=LineaTexto.length();
							finFrase=0;
							for(m=0;m<=lengLineaTexto;m++)
							{
								if (LineaTexto[m] == '"')	
								{ 
									Expresion=LineaTexto.section('"',0,0);	
									finFrase=1;
									m=lengLineaTexto;
								}
							}
							if (finFrase==0)	{ Expresion=LineaTexto;	}
							//Una vez tenemos la frase volvemos a la parte de ver si es end o no y si hay %CR%
							goto OTRA;
						}
											
					case 4:
						//////////////////////////////// NEXT ////////////////////////////////
						//Puede ser Next normal, Next0 o Next1
						//El next siempre aparece delante de un Next1.... Por eso, cada vez que hay un next, en NEXT0, NEXT1... se guarda un 0 para no perder el orden del iteradorBox
						//Si luego hay Next1 o alguno, se remplaza el Next1 por el 0. Sino hay se quedan los 0
						NextZero=-1;
						NextOne=-1;
						NextZero=LineaTexto.indexOf("Next0",Qt::CaseInsensitive);
						NextOne=LineaTexto.indexOf("Next1",Qt::CaseInsensitive);

						//Guardamos el NEXT en la class Box, en la posicin iteradorBox de los vectores
						SiguienteBox=LineaTexto.section(";",0,0);
						SiguienteBox=SiguienteBox.section("=",1);
						siguiente=SiguienteBox.toInt(&ok, 10);

						//Dependiendo el next encontrado aade en la posicin iteradorBox de los 3 Next o un 0 o el valor de Next
						
						if(NextZero!=-1) 
						{ 
							BOX.NEXT0.replace(iteradorBox,siguiente); 
						}
						else if(NextOne!=-1) 
						{ 
							BOX.NEXT1.insert(iteradorBox,siguiente);
						}
						else 
						{ 
							BOX.NEXT.insert(iteradorBox,siguiente); 
						}
						/*BOX.ID(iteradorBox)=PalabraID.number(SiguienteBox);*/
						
						
						if(NextZero!=-1) 
						{ 
							NextTextBox=("Next0 de este Box. Linea: . NEXT0 = ");
							NextTextBox.insert(26,QString::number(i));
							NextTextBox.push_back(Box.number(BOX.NEXT0.at(iteradorBox))); 
						}
						else if(NextOne!=-1) 
						{ 
							NextTextBox=("Next1 de este Box. Linea: . NEXT1 = ");
							NextTextBox.insert(26,QString::number(i));
							NextTextBox.push_back(Box.number(BOX.NEXT1.at(iteradorBox))); 
						}
						else 
						{ 
							NextTextBox=("Next de este Box. Linea: . NEXT = ");
							NextTextBox.insert(25,Box.number(i));
							NextTextBox.push_back(Box.number(BOX.NEXT.at(iteradorBox)));
						}
						SimProcess.append(NextTextBox);
						SimProcess.append("---------------------------------------------");

						//Leemos la siguiente linea de caracteres para saber que nos encontramos a continuacion en el BOX
						LineaTexto=textoLeer.readLine(longline);
						i++;
						break;
					case 5:
						////////////////////////////////   }  ////////////////////////////////
						itBox=2000;		//Variable para salir del BOX
						LlaveTextBox=("Fin del modulo. Linea: ");
						LlaveTextBox.push_back(Box.number(i));
						SimProcess.append(LlaveTextBox);
						SimProcess.append("---------------------------------------------");
						i++;
						break;
					default:
						/////////////Mensaje de error por linea no interpretada////////////
						ErroresYWarnings("Error!: Linea dentro de BOX no implantada en el reconocimiento", MSG_Err);
						break;
				} //END SWITCH	
					//		//Leemos la siguiente linea. Tiene que aparecer Text o sino hay texto Next
					//		LineaTexto=textoLeer.readLine(75);

					//		//Actualizamos las variables
					//		int text = LineaTexto.indexOf(Text,Qt::CaseInsensitive);
					//		int next = LineaTexto.indexOf(Next,Qt::CaseInsensitive);
					//		int EndCorch = LineaTexto.indexOf("}",Qt::CaseInsensitive);

					//		//TEXT: Si encontramos texto lo analizamos
					//		if (text!=-1)
					//		{
					//			//Mostramos que hay texto
					//			QString TextoHeader("Texto de la caja Header. Linea: ");
					//			TextoHeader.insert(32,Box.number(i));
					//			SimProcess.append(TextoHeader);

					//			//Variables para saber si es texto superior o inferior y lo mostramos
					//			int textup = LineaTexto.indexOf(TextUp,Qt::CaseInsensitive);
					//			int textdown = LineaTexto.indexOf(TextDown,Qt::CaseInsensitive);

					//			if (textup!=-1)
					//				SimProcess.append("Es texto superior (TextUp)");
					//			else if (textdown!=-1)
					//				SimProcess.append("Es texto inferior (TextDown)");
					//			else
					//				SimProcess.append("Es texto normal");
					//
					//			SimProcess.append("---------------------------------------------");
					//			i++;
					//
					//			//Leemos la siguiente linea para saber si hay Next
					//			LineaTexto=textoLeer.readLine(75);

					//			//NEXT: lo analizamos si aparece
					//			int next = LineaTexto.indexOf(Next,Qt::CaseInsensitive);
					//			if (next!=-1)
					//			{
					//				QString NextHeader=("Next de la caja Header. Linea: ");
					//				NextHeader.insert(32,Box.number(i));
					//				SimProcess.append(NextHeader);
					//				SimProcess.append("---------------------------------------------");
					//				i++;
					//			}

					//			//Leemos la siguiente linea de texto, que sera un corchete }
					//			LineaTexto=textoLeer.readLine(75);

					//			int EndCorch = LineaTexto.indexOf("}",Qt::CaseInsensitive);
					//			if (EndCorch!=-1)
					//			{
					//				SimProcess.append("Fin del modulo Header");
					//				i++;
					//				SimProcess.append("---------------------------------------------");
					//			}		
					//		}

					//		//Si no encontramos texto encontraremos next y lo analizamos
					//		else if (next!=-1)
					//		{
					//			QString NextHeader=("Next de la caja Header. Linea: ");
					//			NextHeader.insert(32,Box.number(i));
					//			SimProcess.append(NextHeader);
					//			SimProcess.append("---------------------------------------------");
					//			i++;

					//			//Leemos la siguiente linea
					//			LineaTexto=textoLeer.readLine(75);

					//			//Encontramos un corchete } y lo mostramos
					//			int EndCorch = LineaTexto.indexOf("}",Qt::CaseInsensitive);
					//			if (EndCorch!=-1)
					//			{
					//				SimProcess.append("Fin del modulo Header");
					//				i++;
					//				SimProcess.append("---------------------------------------------");
					//			}
					//		}
					//		//Si no hay ni texto ni next habra un corchete que cierra el box cabecera
					//		else if (EndCorch!=-1)
					//		{
					//			SimProcess.append("Fin del modulo Header");
					//			i++;
					//			SimProcess.append("---------------------------------------------");
					//			//Leemos la linea en blanco despues del fin de corchetes
					//			//LineaTexto=textoLeer.readLine(75);
					//			//i++;
					//		}				
					//	}
					//}

					////Salidos ya del TYPE que nos hayamos encontrado y de analizar su texto 
					////pasamos a analizar el next y el fin del box
				
					//SimProcess.append("---------------------------------------------");
					//	break;




					

				
					
					//Leemos la siguiente linea para saber si hay Next
					//LineaTexto=textoLeer.readLine(75);

					////NEXT: lo analizamos si aparece
					//int next = LineaTexto.indexOf(Next,Qt::CaseInsensitive);
					//if (next!=-1)
					//{
					//	QString NextBox=("Next de este Box. Linea: ");
					//	NextBox.insert(25,Box.number(i));
					//	SimProcess.append(NextBox);
					//	SimProcess.append("---------------------------------------------");
					//	i++;
					//}

					////Leemos la siguiente linea de texto, que sera un corchete }
					//LineaTexto=textoLeer.readLine(75);

					//int EndCorch = LineaTexto.indexOf("}",Qt::CaseInsensitive);
					//if (EndCorch!=-1)
					//{
					//	SimProcess.append("Fin del modulo (Port, Code, ThreadSync, Event, Defaults o Instance)");
					//	i++;
					//	SimProcess.append("---------------------------------------------");
					//}		

					////Mostramos todo
					////textProcess->addItems(SimProcess);
			} //END FOR (for para leer dentro de un box)
			//i++;
			//Salimos del Box asiq esta variable vuelve a -1
			header=-1;
			//Guardamos un 0 en la posicin iteradorBox del Touch para inicializarlo
			BOX.TOUCH.push_back(0);
			//Como salimos del Box, aumentamos el iterador para no seguir guardando en el mismo vector de la class Box
			iteradorBox++;
			//Reiniciamos las filas y aumentamos las columnas para el siguiente box
			filT=0;
			colT++;
			filU=0;
			colU++;
			filD=0;
			colD++;
		}
		else
		{
			i++;
		}

		textProcess->addItems(SimProcess);
		SimProcess.clear();

		if (i==99999)
		{
			QString ErrorMsg=("ERROR!\t Archivo excesivamente largo. Aumentar variable i en bucle while de LecturaArchivo();");
			ErroresYWarnings(ErrorMsg,MSG_Err);
		}
	}while(i<=100000);	//End Do-While con valor suficientemente grande como para no fallar nunca

	//Limpiamos la lista de proceso y mostramos el mensaje final de la simulacin
	SimProcess.clear();
	SimProcess.append("//Fin de la simulacin en modo lectura de archivo");
	textProcess->addItems(SimProcess);
	SimProcess.clear();

	//Cerramos el archivo
	archivo.close();
}

//Funcion para mostrar todos los datos guardados a la hora de la lectura del archivo 
void MainWindow::MostrarDatosGuardados()
{
	QString Datos;
	SimProcess.append("");
	SimProcess.append("//--------------DATOS GUARDADOS AL LEER EL ARCHIVO-----------------//");
	Datos=(" - Numero de BOX leidos: ");
	Datos.push_back(QString::number(BOX.ID.size()));
	SimProcess.append(Datos);
	Datos.clear();
	Datos=(" - Numero de PAGES leidos: ");
	Datos.push_back(QString::number(PAGES.ID.size()));
	SimProcess.append(Datos);
	//for(int x=0;x<BOX.ID.size();x++)
	//{
	//	SimProcess.append("");
	//	SimProcess.append(QString::number(BOX.ID.at(x)));
	//	SimProcess.append(BOX.TYPE.at(x));
	//	//SimProcess.append(BOX.TEXT.at(x));
	//	
	//	if (BOX.NEXT.at(x)!=-1)			{ SimProcess.append(QString::number(BOX.NEXT.at(x)));	} 
	//	if (BOX.NEXT0.at(x)!=-1)		{ SimProcess.append(QString::number(BOX.NEXT0.at(x)));	}
	//	if (BOX.NEXT1.at(x)!=-1)		{ SimProcess.append(QString::number(BOX.NEXT1.at(x)));	}
	//}

	QString MostrarTexto, Size, Orden=("\t\t"), Separador=("  -");
	int size, sumador=0;
	bool ok;
	SimProcess.append(" - Matriz de TEXT: ");
	Separador=Separador.repeated(220);
	SimProcess.append(Separador);
	for(int ini=0; ini<BOX.TEXTMatrix.size(); ini++)
	{
		MostrarTexto.push_back("|   ");
		for (int ji=0; ji<BOX.ID.size(); ji++)
		{
			//Orden=("_");
			Size=BOX.TEXTMatrix[ini][ji];
			size=Size.toInt(&ok,10);
			//orden=orden-size;
			//for (int k=0; k<=orden; k++)
			//{
				//Orden.push_back(Orden);
			//}
			MostrarTexto.push_back(BOX.TEXTMatrix[ini][ji]);
			//MostrarTexto.push_back(Orden);
			if		(size<=4)	{ MostrarTexto.push_back("\t");	}
			else if (size<=24)	{ MostrarTexto.push_back("");	}
			else				{ MostrarTexto.push_back("");		}	

			if(BOX.TEXTMatrix[ini][ji]=="null")
			{
				sumador=sumador+1;
			}
			else
			{
				sumador=0;
			}
		}
		MostrarTexto.push_back("|");

		if (sumador>=BOX.ID.size())
		{
			MostrarTexto.clear();
		}
		else
		{
			SimProcess.append(MostrarTexto);
			MostrarTexto.clear();
		}
	}
	SimProcess.append(Separador);
	sumador=0;
	SimProcess.append("\n - Matriz de TEXT UP: ");
	SimProcess.append(Separador);
	for(int ini=0; ini<BOX.TEXTUPMatrix.size(); ini++)
	{
		MostrarTexto.push_back("|   ");
		for (int ji=0; ji<BOX.ID.size(); ji++)
		{
			MostrarTexto.push_back(BOX.TEXTUPMatrix[ini][ji]);
			Size=BOX.TEXTMatrix[ini][ji];
			size=Size.toInt(&ok,10);
			if		(size<=4)	{ MostrarTexto.push_back("\t");	}
			else if (size<=22)	{ MostrarTexto.push_back("");	}
			else				{ MostrarTexto.push_back("");		}

			if(BOX.TEXTUPMatrix[ini][ji]=="null")
			{
				sumador=sumador+1;
			}
			else
			{
				sumador=0;
			}
		}
		MostrarTexto.push_back("|");

		if (sumador>=BOX.ID.size())
		{
			MostrarTexto.clear();
		}
		else
		{
			SimProcess.append(MostrarTexto);
			MostrarTexto.clear();
		}
	}
	SimProcess.append(Separador);
	SimProcess.append("\n - Matriz de TEXT DOWN: ");
	SimProcess.append(Separador);
	sumador=0;
	for(int ini=0; ini<BOX.TEXTDOWNMatrix.size(); ini++)
	{
		MostrarTexto.push_back("|   ");
		for (int ji=0; ji<BOX.ID.size(); ji++)
		{
			MostrarTexto.push_back(BOX.TEXTDOWNMatrix[ini][ji]);
			Size=BOX.TEXTMatrix[ini][ji];
			size=Size.toInt(&ok,10);
			if		(size<=4)	{ MostrarTexto.push_back("\t");	}
			else if (size<=22)	{ MostrarTexto.push_back("");	}
			else				{ MostrarTexto.push_back("");		}

			if(BOX.TEXTDOWNMatrix[ini][ji]=="null")
			{
				sumador=sumador+1;
			}
			else
			{
				sumador=0;
			}
		}
		MostrarTexto.push_back("|");


		if (sumador>=BOX.ID.size())
		{
			MostrarTexto.clear();
		}
		else
		{
			SimProcess.append(MostrarTexto);
			MostrarTexto.clear();
		}
	}
	SimProcess.append(Separador);
	textOut->addItems(SimProcess);
	TXTOut(SimProcess);
	SimProcess.clear();
}

//Esta funcion se encarga de recorrer el vector TYPE de la clase Box BOX y guarda en un QMultiHash las posiciones de cada tipo
//Tambien llama a la funcin que identifica cuales son los ID's Cabecera
void MainWindow::OrdenandoBoxes()
{

	QStringList OrdenProcess;	//Lista para mostrar el proceso de esta funcin
	QString TIPO;				//String para guardar el tipo de BOX
	QString MostrarProceso;		//String para guardar el proceso y mostrarlo
	int TypeSw=0;				//Variable del Switch

	QList<int> LIST;			//Listado para mostrar las posiciones de cada tipo
	QString POS;				//String para mostrar las posiciones de cada tipo

	//Creamos diferentes strings para diferenciar entre los diferentes TYPE de cajas de esta zona
	QString Header="Header";				//Cabecera
	QString Ports="Ports";					//Ports
	QString Code="Code";					//Code
	QString ThreadSync="ThreadSync";		//ThreadSync
	QString Event="Event";					//Event
	QString Defaults="Default";				//Default
	QString Instance="Instance";			//Instance
	QString MetaState="MetaState";			//MetaState
	QString StateAsyncOps="StateAsyncOps";	//StateAsyncOps
	QString Initial="Initial";				//Initial
	QString Decision="Decision";			//Decision
	QString State="State";					//State
	QString CondSyncOps="CondSyncOps";			//CondSyncOps
	QString	CondAsyncOps="CondAsyncOps";		//CondAsyncOps
	QString SyncOps="SyncOps";				//SyncOps
	QString	AsyncOps="AsyncOps";			//AsyncOps


	//Mensaje cabecera de la funcin para saber que estamos aqui
	OrdenProcess.append("");
	OrdenProcess.append("//-------------- Posicin en la memoria del TYPE de cada BOX --------------//");

	//Inicializacin de variables
	ORDENBOX.IterBox=0;
	ORDENBOX.POSICION.clear();

	//Recorre el vector TYPE para ver que tipo de boxes hay y dependiendo el que sea guarda su posicin en QMultiHash
	for(int x=0;x<BOX.TYPE.size();x++)
	{
		TIPO=BOX.TYPE.at(x);

		if (TIPO==Header) { TypeSw=1; }
		else if (TIPO==Ports) { TypeSw=2; }
		else if (TIPO==Code) { TypeSw=3; }
		else if (TIPO==ThreadSync) { TypeSw=4; }
		else if (TIPO==Event) { TypeSw=5; }
		else if (TIPO==Defaults) { TypeSw=6; }
		else if (TIPO==Instance) { TypeSw=7; }
		else if (TIPO==MetaState) { TypeSw=8; }
		else if (TIPO==StateAsyncOps) { TypeSw=9; }
		else if (TIPO==Initial) { TypeSw=10; }
		else if (TIPO==Decision) { TypeSw=11; }
		else if (TIPO==State) { TypeSw=12; }
		else if (TIPO==CondAsyncOps) { TypeSw=13; }
		else if (TIPO==CondSyncOps) { TypeSw=14; }
		else if (TIPO==AsyncOps) { TypeSw=15; }
		else if (TIPO==SyncOps) { TypeSw=16; }

		switch (TypeSw)
		{
			case 1:
				ORDENBOX.POSICION.insert("Header",ORDENBOX.IterBox);
				ORDENBOX.IterBox++;
				break;
			case 2:
				ORDENBOX.POSICION.insert("Ports",ORDENBOX.IterBox);
				ORDENBOX.IterBox++;
				break;
			case 3:
				ORDENBOX.POSICION.insert("Code",ORDENBOX.IterBox);
				ORDENBOX.IterBox++;
				break;
			case 4:
				ORDENBOX.POSICION.insert("ThreadSync",ORDENBOX.IterBox);
				ORDENBOX.IterBox++;
				break;
			case 5:
				ORDENBOX.POSICION.insert("Event",ORDENBOX.IterBox);
				ORDENBOX.IterBox++;
				break;
			case 6:
				ORDENBOX.POSICION.insert("Default",ORDENBOX.IterBox);
				ORDENBOX.IterBox++;
				break;
			case 7:
				ORDENBOX.POSICION.insert("Instance",ORDENBOX.IterBox);
				ORDENBOX.IterBox++;
				break;
			case 8:
				ORDENBOX.POSICION.insert("MetaState",ORDENBOX.IterBox);
				ORDENBOX.IterBox++;
				break;		
			case 9:
				ORDENBOX.POSICION.insert("StateAsyncOps",ORDENBOX.IterBox);
				ORDENBOX.IterBox++;
				break;
			case 10:
				ORDENBOX.POSICION.insert("Initial",ORDENBOX.IterBox);
				ORDENBOX.IterBox++;
				break;
			case 11:
				ORDENBOX.POSICION.insert("Decision",ORDENBOX.IterBox);
				ORDENBOX.IterBox++;
				break;
			case 12:
				ORDENBOX.POSICION.insert("State",ORDENBOX.IterBox);
				ORDENBOX.IterBox++;
				break;	
			case 13:
				ORDENBOX.POSICION.insert("CondAsyncOps",ORDENBOX.IterBox);
				ORDENBOX.IterBox++;
				break;
			case 14:
				ORDENBOX.POSICION.insert("CondSyncOps",ORDENBOX.IterBox);
				ORDENBOX.IterBox++;
				break;
			case 15:
				ORDENBOX.POSICION.insert("AsyncOps",ORDENBOX.IterBox);
				ORDENBOX.IterBox++;
				break;
			case 16:
				ORDENBOX.POSICION.insert("SyncOps",ORDENBOX.IterBox);
				ORDENBOX.IterBox++;
				break;	
		}
	}

	//Bucle para mostrar los tipos de cajas y sus posiciones
	for (int j=0; j<16; j++)
	{
		if (j==0) { POS=Header; }
		else if (j==1) { POS=Ports; }
		else if (j==2) { POS=Code; }
		else if (j==3) { POS=ThreadSync; }
		else if (j==4) { POS=Event; }
		else if (j==5) { POS=Defaults; }
		else if (j==6) { POS=Instance; }
		else if (j==7) { POS=MetaState; }
		else if (j==8) { POS=StateAsyncOps; }
		else if (j==9) { POS=Initial; }
		else if (j==10) { POS=Decision; }
		else if (j==11) { POS=State; }
		else if (j==12) { POS=CondSyncOps; }
		else if (j==13) { POS=CondAsyncOps; }
		else if (j==14) { POS=SyncOps; }
		else if (j==15) { POS=AsyncOps; }

		MostrarProceso="Posiciones de las cajas  en el archivo : ";
		MostrarProceso.insert(24,POS);
		LIST = ORDENBOX.POSICION.values(POS);
		
		//Si de un tipo no hay boxes no se muestra
		if(LIST.isEmpty()) {	}
		else
		{
			//Recorremos la lista guardandolo en un String
			for(int i=0; i<LIST.size(); i++)
			{
				MostrarProceso += QString::number(LIST[i]);
				if(i<LIST.size()-1) { MostrarProceso += "," ; }
			}

			OrdenProcess.append(MostrarProceso);
		}
	}
	//Aadimos el QStringList al widget con el proceso
	textOut->addItems(OrdenProcess);
	TXTOut(OrdenProcess);
}

//Esta funcion se encarga de detectar que ID's son padres y cuales no. Los junta con los de las cabeceras HEADER
//y los guarda en la clase ORDENBOX para ejecutarlos cuando sea necesario
void MainWindow::BoxCabecera()
{
	//Definicin de variables
	QStringList OrdenProcess;				//Lista para guardar y mostrar el proceso de esta funcin
	QString IdOrden, ItOrden;				//String para ir creando las cadenas de texto a mostrar
	QList<int> LISTPosPadres, LISTIdPadres;	//Listas para guardar los iteradores y juntarlos				
	int Hijo=0;								//Variable para detectar si un Id es hijo o padre
	int Id=0;								//Variable para guardar el Id
	int Next=0, Next0=0, Next1=0;			//Variables para guardar los Next's
	int ID=0;								//Variable para guardar cada iterador para mostrar el ID correspondiente

	//Limpiamos todos los Id's que esten guardados como cabeceras anteriormente (de otros programas) asi como la lista de proceso
	ORDENBOX.ItCabeceras.clear();
	OrdenProcess.clear();
	LISTPosPadres.clear();
	LISTIdPadres.clear();
	IdOrden.clear();
	ItOrden.clear();

	IdOrden="Id Padres: ";
	ItOrden="It Padres: ";
	OrdenProcess.append(" ");
	OrdenProcess.append("//-------------- ID's de los BOXES PADRES--------------//");

	//Recorremos los ID de la clase Box BOX 
	for (int i=0; i<BOX.ID.size(); i++)
	{
		Hijo=0;
		Id=BOX.ID[i];
		//Recorremos los NEXT de la clase Box BOX y comprobamos si alguno coincide con un ID
		for (int j=0; j<BOX.NEXT.size(); j++)
		{
			Next=BOX.NEXT[j];
			Next0=BOX.NEXT0[j];
			Next1=BOX.NEXT1[j];
			
			//Si no coincide el Id con ningun Next es un padre
			if (Next!=Id && Next0!=Id && Next1!=Id) 
			{ 
				Hijo=0;
			}

			//Si coincide con algun next ese box es un hijo, y pasa al siguiente ID
			else
			{
				Hijo=1;
				j=BOX.NEXT.size();
			}
		}

		//Si no se ha encontrado ningun Next igual al ID, ese ID es un padre y se guarda en la lista de padres
		if (Hijo!=1)
		{
			//Se guarda la posicin del iterador en la memoria
			LISTPosPadres.push_back(i);
			//Se guarda el ID
			LISTIdPadres.push_back(BOX.ID[i]);
		}
	}

	if (LISTPosPadres.length()==0)
	{
		//WARNING!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		//NO HAY PADRES
		QString Warning=("WARNING!\t No hay boxes padre");
		ErroresYWarnings(Warning,MSG_War);
	}

	//Se guarda en la clase ORDENBOX los iteradores padres y cabeceras para poder ejecutarlos en otra funcin a partir de las listas
	ORDENBOX.ItCabeceras.clear();
	ORDENBOX.IdCabeceras.clear();
	for(int m=0; m<LISTPosPadres.size(); m++)
	{
		//Se guardan las posiciones de los iateradores
		ORDENBOX.ItCabeceras.push_back(LISTPosPadres[m]);
		//Se guardan los IDs
		ORDENBOX.IdCabeceras.push_back(LISTIdPadres[m]);

		//Se recorre la lista para mostrar por pantalla los iteradores de las cabeceras y los padres
		ID=ORDENBOX.ItCabeceras[m];
		ItOrden += QString::number(ORDENBOX.ItCabeceras[m]);
		if(m<LISTPosPadres.size()-1) { ItOrden += ", " ; }
		IdOrden += QString::number(ORDENBOX.IdCabeceras[m]);
		if(m<LISTIdPadres.size()-1) { IdOrden += ", " ; }
	}

	OrdenProcess.append(IdOrden);
	OrdenProcess.append(ItOrden);
	textOut->addItems(OrdenProcess);
	TXTOut(OrdenProcess);
}

//Esta funcion es la encargada de simular el circuito digital de forma declarativa. Partiendo desde los boxes padres,
//recorre el diagrama por orden NEXT->ID de caja en caja buscando todas las seales. Para ello, lo recorre en diferentes modos, 
//buscando por orden: inputs&outputs, wires. 
//SOLO HAY QUE RECORRER LA PARTE DE DISEO DEL CIRCUITO, NO EL TEST BENCH. POR LO TANTO SE RECORRE UNA VEZ PRIMERA DETECTANDO
//SI UN PADRE ES DE TEST BENCH O DE DISEO. LA DIFERENCIA ENTRE AMBOS ES QUE EL TEST BENCH ACABA CON UN END SIMULATION,
//POR LO TANTO HAY QUE RECORRER UNO A UNO EL DIAGRAMA Y SI SE ENCUENTRA AL FINAL UN END SIMULATION
//TODOS LOS PADRES QUE HAN LLEVADO A ESE BOX SON DEL TEST BENCH Y SE IGNORAN AL RECORRER EN BUSCA DE SEALES
//Las seales aparecen definidas en bloques Ports (inputs&outputs), Code (wire), initial, default, AsyncOps, SyncOps, Decision y Instance.
//Hay que recorrer todo el diagrama, sin pasar 2 veces por la misma caja. Para eso, cada vez que se toca una caja se deja
//una seal (un 1 por ejemplo en esa posicin) para que si ya hemos pasado no volvamos a pasar.
//En el modo busqueda Ports se guardan las seales dependiendo del tipo
//En el modo busqueda Code igual
//En el modo busqueda initial y default se buscan sus valores iniciales y por defecto
//En el modo busqueda Async-Sync se buscan que seales aparecen y se determina si son sincronas o asincronas
//En el modo busqueda Decision (no se si hace falta)
//CUANDO ACABE ESTA SIMULACION SE DEBE HABER RECORRIDO TODO EL DIAGRAMA SABIENDO QUE SEALES HAY Y 
//DE QUE TIPO (IN-OUT... SYNC-ASYNC) SON
void MainWindow::SimulacionDeclarativa()
{
	const int Toco=1;											//Variable para tocar al pasar por un box
	int modo=0;													//Variable para elegir el modo de busqueda
	int IteraPadre=0, NumPadres=0;								//Variables para saber los padres
	int IDPadre=0, NextPadre=0, Next0Padre=0, Next1Padre=0;		//Variables para los padres
	int IDSiguiente=0, IteraSiguiente=0;						//Variables para los box siguientes al padre
	int NextSiguiente=0, Next0Siguiente=0, Next1Siguiente=0;	//Variables para los box siguientes al padre
	int Itera0Siguiente=0, Itera1Siguiente=0;					//Variables para elegir el camino de una decision
	QString TYPE, MostrarType;

	//Variables de Simulacion declarativa, modo busqueda Ports
	int Tocado, k, ITERADOR, StateExiste=0;;
	QString State, Error;



	//Variables para ir viendo el proceso de simulacin
	QString SimDeclarativa;
	QStringList SimulacionDeclarativa;

	SimulacionDeclarativa.append("\n//--------------SIMULACION DECLARATIVA-----------------//");
	textProcess->addItems(SimulacionDeclarativa);
	textOut->addItems(SimulacionDeclarativa);
	TXTOut(SimulacionDeclarativa);
	SimulacionDeclarativa.clear();
	SimDeclarativa.clear();

	//Calculamos el numero de padres que hay para el bucle for
	NumPadres=ORDENBOX.ItCabeceras.size();

	//Recorremos tantas veces como padres hay
	for(int i=0; i<NumPadres; i++)
	{
		//Recorremos tantas veces como modos hay
		for(modo=1;modo<=3;modo++)
		{
			//Actualizamos valores del padre que toca
			IteraPadre=ORDENBOX.ItCabeceras.at(i);		//Iterador en memoria
			IDPadre=BOX.ID.at(IteraPadre);				//Id del padre
			NextPadre=BOX.NEXT.at(IteraPadre);			//Next del padre
			Next0Padre=BOX.NEXT0.at(IteraPadre);		//Next0 del padre 
			Next1Padre=BOX.NEXT1.at(IteraPadre);		//Next1 del padre

			//Se guarda en el String el padre
			SimDeclarativa.append("PADRE: ");
			SimDeclarativa.append(QString::number(IDPadre));
			SimDeclarativa.append(", ");
			SimDeclarativa.append(QString::number(NextPadre));
			SimDeclarativa.append(", ");
			SimDeclarativa.append(QString::number(Next0Padre));
			SimDeclarativa.append(", ");
			SimDeclarativa.append(QString::number(Next1Padre));
		
			//Se muestra y se borra
			SimulacionDeclarativa.append(SimDeclarativa);
			textOut->addItems(SimulacionDeclarativa);
			TXTOut(SimulacionDeclarativa);
			SimulacionDeclarativa.clear();
			SimDeclarativa.clear();

			//Ponemos todos los touch a 0 porque reiniciamos modo
			for(int j=0; j<=BOX.TOUCH.size(); j++)
			{
				if(BOX.TOUCH.at(j)==Toco)
				{
					BOX.TOUCH[j]=0;
				}
			}

			//MODOS DE BUSQUEDA:
			//	- PORTS ----------- modo = 1
			//	- CODE ------------ modo = 2
			//	- ASYNC-SYNC ------ modo = 3
			switch (modo)
			{
				case 1:
					SimulacionDeclarativa.append("     a) MODO PORTS");
					TYPE="Ports";
VUELTACODE:
					//Tocamos al padre
					BOX.TOUCH[IteraPadre]=Toco;
					//Si el Next del padre no es -1 seguimos
					if(NextPadre!=-1)
					{
VUELTA:
						//Llamamos a la funcin para saber el iterador correspondiente al Next de el box actual	
						NextBox(NextPadre,IteraSiguiente);

						//Comprobamos si ya hemos pasado por aqui
						//Si ya lo hemos tocado no hacemos nada
						if(BOX.ID.at(IteraSiguiente)==Toco) {}
						//Si no lo hemos tocado, lo tocamos, cogemos sus datos y buscamos el siguiente
						else
						{
							//Toco el BOX
							BOX.TOUCH[IteraSiguiente]=Toco;

							//Comprobamos su modo, y si coincide con el typo de BOX en el que estamos se analiza
							//su texto o no
							if ((modo==1) || (modo==2))
							{
								if (TYPE==BOX.TYPE.at(IteraPadre))		
								{ 
									MostrarType.clear();
									MostrarType="               + ";
									MostrarType.push_back(TYPE);
									SimDeclarativa.append(MostrarType);
									SimDeclarativa.append("\n");
									SaveSignals(IteraPadre,modo);
								}
							}
							//Sino, estamos en modo SYNC-ASYNC
							else
							{
								//Si el tipo es uno de los siguientes se analiza
								if (	(BOX.TYPE.at(IteraPadre) == "AsyncOps")
									||	(BOX.TYPE.at(IteraPadre) == "SyncOps")
									||	(BOX.TYPE.at(IteraPadre) == "CondSyncOps")
									||	(BOX.TYPE.at(IteraPadre) == "CondAsyncOps")
									||	(BOX.TYPE.at(IteraPadre) == "StateAsyncOps")
									||	(BOX.TYPE.at(IteraPadre) == "Event"))
								{ 
									MostrarType.clear();
									MostrarType="               + ";
									MostrarType.push_back(TYPE);
									SimDeclarativa.append(MostrarType);
									SimDeclarativa.append("\n");
									SaveSignals(IteraPadre,modo);
								}
							}

							//Consigo los datos del hijo
							IDSiguiente=BOX.ID.at(IteraSiguiente);			//Id del hijo
							NextSiguiente=BOX.NEXT.at(IteraSiguiente);		//Next del hijo
							Next0Siguiente=BOX.NEXT0.at(IteraSiguiente);	//Next0 del hijo 
							Next1Siguiente=BOX.NEXT1.at(IteraSiguiente);	//Next1 del hijo

							//Les muestro por pantalla
							SimDeclarativa.append("          - HIJO: ");
							SimDeclarativa.append(QString::number(IDSiguiente));
							SimDeclarativa.append(", ");
							SimDeclarativa.append(QString::number(NextSiguiente));
							SimDeclarativa.append(", ");
							SimDeclarativa.append(QString::number(Next0Siguiente));
							SimDeclarativa.append(", ");
							SimDeclarativa.append(QString::number(Next1Siguiente));
							SimulacionDeclarativa.append(SimDeclarativa);
							SimDeclarativa.clear();

							//Los datos del hijo pasan a ser los del padre
							NextPadre=NextSiguiente;
							Next0Padre=Next0Siguiente;
							Next1Padre=Next1Siguiente;
							IteraPadre=IteraSiguiente;		
						}
					}
					//Si no estamos en una decisin o en un end simulation, NEXT siempre es distinto de -1
					//Si NEXT es -1 es que estamos en una decision y aparecen NEXT0 y NEXT1
					if (NextPadre==-1)
					{
						//Mando los next de 0 y 1 para saber cuales son sus siguientes iteraciones y comprobar si estan tocados
						NextBox(Next0Padre,Itera0Siguiente);
						NextBox(Next1Padre,Itera1Siguiente);

						//Como estamos en decision vamos por un lado y comprobamos si esta tocado y por el otro igual
						if(Next0Padre!=-1)
						{
							//Si no esta tocado actualizo NextPadre con Next0Padre y vuelvo con goto
							if(BOX.TOUCH.at(Itera0Siguiente)!=Toco)
							{
								NextPadre=Next0Padre;
								goto VUELTA;
							}
						}
						if(Next1Padre!=-1)
						{
							//Si no esta tocado actualizo NextPadre con Next1Padre y vuelvo con goto
							if(BOX.TOUCH.at(Itera1Siguiente)!=Toco)
							{
								NextPadre=Next1Padre;
								goto VUELTA;
							}
						}
					}

					//Si el Next es 0 puede ser o un END SIMULATION o un STATE que vuelve a otro descrito antes o no
					//La diferencia es que cuando es END SIMULATION, NEXT1 = 0
					//Cuando no lo es NEXT0 = -1 y NEXT1 = -1
					if(NextPadre==0)
					{
						//END SIMULATION
						if ((Next1Padre == 0) || (Next0Padre == 0))
						{
							if(BOX.TEXTMatrix[0][IteraPadre]=="end simulation")
							{
								//Se guarda el tipo de padre
								ORDENBOX.TypeCabeceras.push_back(TYPETestBench);

								//Como el test bench no forma parte de la simulacin declarativa borramos todo
								//su proceso de recorrido y no seguimos buscando en ms modos
								SimDeclarativa.clear();
								SimulacionDeclarativa.clear();
								SimDeclarativa.append("END SIMULATION. Padre de test bench");
								SimulacionDeclarativa.append(SimDeclarativa);
								SimulacionDeclarativa.append("---------------------------------------");
								modo=3;
								SimDeclarativa.clear();
								break;
							}
						}
						//STATE
						if ((Next0Padre == -1) && (Next1Padre == -1))
						{
							//Se guarda el tipo de padre
							ORDENBOX.TypeCabeceras.push_back(TYPEDiseno);

							//Se coge el nombre del estado, se busca un estado con mismo nombre y si esta tocado se sale.
							//Si no esta tocado se sigue
							SimDeclarativa.append("               + STATE: ");
							State=BOX.TEXTMatrix[0][IteraPadre];
							Tocado=0;
							for(k=0; k<=BOX.TOUCH.size(); k++)
							{
								//Si hay un estado igual SE MIRA SI ESTA TOCADO O NO
								if(BOX.TEXTMatrix[0][k]==State)
								{
									SimDeclarativa.push_back(BOX.TEXTMatrix[0][k]);
									StateExiste=1;
									//Si esta tocado se seala
									if(BOX.TOUCH.at(k)==Toco)
									{
										SimDeclarativa.push_back(" -> TOCADO");
										Tocado=1;
										k=BOX.TOUCH.size();
									}
									//Sino, se guarda el iterador donde esta el estado igual
									else
									{
										SimDeclarativa.push_back(" -> SIN TOCAR");
										ITERADOR=k;	
									}
									SimulacionDeclarativa.append(SimDeclarativa);
									SimDeclarativa.clear();
								}
							}

							//Si no lo hay es un error porque los diagramas de diseo no tienen fin
							if (StateExiste==0)
							{
								Error=("WARNING!\t No hay State con el mismo nombre;");
								ErroresYWarnings(Error,MSG_Err);
								Error.clear();
							}

							//Si esta tocado salimos
							if (Tocado==1){}
							//Sino, se saca el ID, su next y todo del estado con mismo nombre y se vuelve a VUELTA
							else
							{
								IDPadre=BOX.ID.at(ITERADOR);			//Id 
								NextPadre=BOX.NEXT.at(ITERADOR);		//Next
								Next0Padre=BOX.NEXT0.at(ITERADOR);		//Next0  
								Next1Padre=BOX.NEXT1.at(ITERADOR);		//Next1
								goto VUELTA;
							}
						}	
					}
					//Si NextPadre es > 0 significa que no hemos acabado el diagrama y que hay un box siguiente.
					//Repetimos el proceso
					if(NextPadre>0)
					{	
						goto VUELTA;
					}

					SimulacionDeclarativa.append(SimDeclarativa);
					textOut->addItems(SimulacionDeclarativa);
					TXTOut(SimulacionDeclarativa);
					SimDeclarativa.clear();
					SimulacionDeclarativa.clear();
					//Sino salimos del modo PORTS
					break;
				case 2:
					SimulacionDeclarativa.append("     b) MODO CODE");
					TYPE="Code";
					goto VUELTACODE;
					break;
				case 3:
					SimulacionDeclarativa.append("     c) MODO ASYNC-SYNC");
					goto VUELTACODE;
					break;

				SimulacionDeclarativa.append("//--------------------------------//");
			}
		}
	}

	SimulacionDeclarativa.append(SimDeclarativa);
	textOut->addItems(SimulacionDeclarativa);
	TXTOut(SimulacionDeclarativa);
	SimulacionDeclarativa.clear();
	SimDeclarativa.clear();
}

//Funcion para mostrar todas las seales guardadas en la simulacion declarativa
void MainWindow::MostrarSignals()
{
	QStringList SimulacionDeclarativa;
	QString SimDeclarativa;

	//Se muestran todas las seales al finalizar la simulacin declarativa
	SimulacionDeclarativa.append("//----------------------------- Lista de signals -------------------------------//");
	SimDeclarativa.append(" - Inputs:\n");
	for (int l=0; l<SignalClass.InputList.size(); l++)
	{
		SimDeclarativa.push_back("\t");
		SimDeclarativa.push_back(SignalClass.InputList.at(l));
		SimDeclarativa.push_back("\t");
		SimDeclarativa.push_back(SignalClass.SyncInput.at(l));
		SimDeclarativa.push_back("\t");
		SimDeclarativa.push_back(SignalClass.CaractInput.at(l));
		SimDeclarativa.push_back("\n");
	}
	SimulacionDeclarativa.append(SimDeclarativa);
	SimDeclarativa.clear();
	SimDeclarativa=(" - Outputs:\n");
	for (int l=0; l<SignalClass.OutputList.size(); l++)
	{
		SimDeclarativa.push_back("\t");
		SimDeclarativa.push_back(SignalClass.OutputList.at(l));
		SimDeclarativa.push_back("\t");
		SimDeclarativa.push_back(SignalClass.SyncOutput.at(l));
		SimDeclarativa.push_back("\t");
		SimDeclarativa.push_back(SignalClass.CaractOutput.at(l));
		SimDeclarativa.push_back("\n");
	}
	SimulacionDeclarativa.append(SimDeclarativa);
	SimDeclarativa.clear();
	SimDeclarativa=(" - Wires:\n");	
	for (int l=0; l<SignalClass.WireList.size(); l++)
	{
		SimDeclarativa.push_back("\t");
		SimDeclarativa.push_back(SignalClass.WireList.at(l));
		SimDeclarativa.push_back("\t");
		SimDeclarativa.push_back(SignalClass.SyncWire.at(l));
		SimDeclarativa.push_back("\t");
		SimDeclarativa.push_back(SignalClass.CaractWire.at(l));
		SimDeclarativa.push_back("\n");
	}
	SimulacionDeclarativa.append(SimDeclarativa);
	SimDeclarativa.clear();
	SimDeclarativa=(" - Inouts:\n");
	for (int l=0; l<SignalClass.InoutList.size(); l++)
	{
		SimDeclarativa.push_back("\t");
		SimDeclarativa.push_back(SignalClass.InoutList.at(l));
		SimDeclarativa.push_back("\t");
		SimDeclarativa.push_back(SignalClass.SyncInout.at(l));
		SimDeclarativa.push_back("\t");
		SimDeclarativa.push_back(SignalClass.CaractInout.at(l));
		SimDeclarativa.push_back("\n");
	}


	SimulacionDeclarativa.append(SimDeclarativa);
	textOut->addItems(SimulacionDeclarativa);
	TXTOut(SimulacionDeclarativa);
	SimulacionDeclarativa.clear();
	SimDeclarativa.clear();
}

//Esta funcin se emplea para recorrer los diagramas. Recibe el next del padre y devuelve el Iterador del hijo en memoria
void MainWindow::NextBox(int NextPadre, int &ItHijo)
{
	ItHijo=-1;
	for(int i=0; i<BOX.ID.size(); i++)
	{
		if(BOX.ID.at(i)==NextPadre)
		{
			ItHijo=i;
		}
	}

	//Si no se encuentra al hijo se avisa por pantalla
	if(ItHijo==-1)
	{
		QString Warning=("WARNING!\t Hijo no encontrado con ID: ");
		Warning.push_back(QString::number(NextPadre));
		ErroresYWarnings(Warning,MSG_War);
	}

}

//Funcion encargada de guardar las seales en la simulacin declarativa
//Le llega un int con la posicin en memoria que tiene que analizar
//y un modo de busqueda, lo analiza en busca del tipo de seal dependiendo el modo y lo guarda.
//En modo 1 = Ports busca inputs y outputs. Si hay otras muestra error
//En modo 2 = Code busca wire e inouts. Si hay otras muestra error
//En modo 3 = Async-Sync busca entre las seales que ya hay las que encuentra y decide del tipo que son
void MainWindow::SaveSignals(int IteradorMemoria, int modo)
{
	//Variables de la funcion
	int Token;											//Variable para guardar el token recibido
	QString frase, caracteristicas, nombreSignal;		//Variable para guardar Strings empleados en la funcion

	int Encontrado=0, Sincronismo=0;

	SignalClass.NombreSignal.clear();
	TokenProcess.clear();
	//TokenProcess.append("SAVE SIGNALS");
	
	//Dependiendo el modo busca una cosa u otra en los textos del IteradorMemoria
	if (modo==1)
	{
		//PORTS
		//Recorremos la matriz de texto en busca de toda la informacin almacenada
		for(int i=0; i<BOX.TEXTMatrix.size(); i++)
		{
			//Si el texto es diferente de null se guarda en frase
			if ((BOX.TEXTMatrix[i][IteradorMemoria])!="null")
			{
				frase=BOX.TEXTMatrix[i][IteradorMemoria];
			}
			//Sino no se guarda
			else
			{
				frase.clear();
				caracteristicas.clear();
			}

			//Si el String frase esta vacio es que hemos encontrado null y no se analiza. Si no esta null se analiza
			if(frase.isNull()==false)
			{
				//Se manda al analizador de tokens para saber que tipo de seal es
				Token = GetToken(frase, FALSE);

				//Si es INPUT
				if (Token == TOKEN_INPUT)	
				{ 
					//Se borra la palabra input y se almacena el tipo en la clase signal
					GetToken(frase, TRUE);
					SignalClass.NumParametro=1;
				}
				//Si es Output
				else if (Token == TOKEN_OUTPUT)	
				{ 
					GetToken(frase, TRUE);
					SignalClass.NumParametro=2;
				}
				//Si es inout o wire, como en PORTS no pueden aparecer se manda error
				else if ((Token == TOKEN_INOUT) || (Token==TOKEN_WIRE))	
				{ 
					ErroresYWarnings("Error! En BOX Ports aparece un INOUT o WIRE",MSG_Err);
				}
				//Si no es nada es que hemos encontrado texto sin definir tipo
				else
				{
					ErroresYWarnings("Error! En BOX Ports aparece texto que no es una seal",MSG_Err);
				}

				//Busca si despues del tipo de variable hay o no parentesis con sus caracteristicas
				Token = GetToken(frase, FALSE);
				if (Token == TOKEN_QPARENOPEN) 
				{
					GetToken(frase, TRUE);

					//Para guardar las caracteristicas vamos almacenando en un String
					caracteristicas="[";

					//Se manda la expresin
					ExprPreviaToken(frase);
					caracteristicas.push_back(QString::number(NumDec));

					//Comprobamos si hay : expr o ]
					Token = GetToken(frase, FALSE);

					if (Token == TOKEN_DOSPUNTOS)
					{
						GetToken(frase, TRUE);
						caracteristicas.push_back(":");

						//Se manda la expresin
						ExprPreviaToken(frase);
						caracteristicas.push_back(QString::number(NumDec));

						Token = GetToken(frase, FALSE);
						if (Token == TOKEN_QPARENCLOSE)	
						{
							GetToken(frase, TRUE);
							caracteristicas.push_back("]");
						}
					}

					else if (Token == TOKEN_QPARENCLOSE)
					{
						GetToken(frase, TRUE);
						caracteristicas.push_back("]");
					}
				}
				//Si no hay nada se guarda un espacio
				else
				{
					caracteristicas=(" ");
				}

				//Si despues de encontrar el tipo y las caracteristicas encontramos letras seran sus nombres. Las analizamos
				//y se guardan
				if (GetToken(frase,FALSE)==TOKEN_WORD)
				{
SIGNALSNAMEPORTS:
					//Bucle para conseguir la frase entera
					int j=0;
					while (frase[j].isLetter()) 
					{
						nombreSignal.push_back(frase[j]);
						j++;	
					}
					//Se guarda en la clase y se limpia la variable local
					SignalClass.NombreSignal=nombreSignal;
					nombreSignal.clear();

					//Se borra la palabra anterior y se buscan comas para ver si hay ms palabras. Si hay se borra
					GetToken(frase,TRUE);
					if (GetToken(frase,FALSE)==TOKEN_COMA)
					{
						GetToken(frase,TRUE);
					}

					//Se guarda dependiendo de si es Input o Output
					if (SignalClass.NumParametro==1)
					{
						SignalClass.InputList.push_back(SignalClass.NombreSignal);
						SignalClass.CaractInput.push_back(caracteristicas);
					}
					if (SignalClass.NumParametro==2)
					{
						SignalClass.OutputList.push_back(SignalClass.NombreSignal);
						SignalClass.CaractOutput.push_back(caracteristicas);

					}
					//Si encontramos letras se reinicia el analizador
					if (GetToken(frase,FALSE)==TOKEN_WORD)
					{
						goto SIGNALSNAMEPORTS;
					}
				}
			}
		}
	}
	else if (modo==2)
	{
		//CODE
		//Recorremos la matriz de texto en busca de toda la informacin almacenada
		for(int i=0; i<BOX.TEXTMatrix.size(); i++)
		{
			//Si el texto es diferente de null se guarda en frase
			if ((BOX.TEXTMatrix[i][IteradorMemoria])!="null")
			{
				frase=BOX.TEXTMatrix[i][IteradorMemoria];
			}
			//Sino no se guarda
			else
			{
				frase.clear();
				caracteristicas.clear();
			}

			//Si el String frase esta vacio es que hemos encontrado null y no se analiza. Si no esta null se analiza
			if(frase.isNull()==false)
			{
				//Se manda al analizador de tokens para saber que tipo de seal es
				Token = GetToken(frase, FALSE);

				//Si es INPUT
				if (Token == TOKEN_INOUT)	
				{ 
					//Se borra la palabra inout y se almacena el tipo en la clase signal
					GetToken(frase, TRUE);
					SignalClass.NumParametro=3;
				}
				//Si es WIRE
				else if (Token == TOKEN_WIRE)	
				{ 
					GetToken(frase, TRUE);
					SignalClass.NumParametro=4;
				}
				//Si es inout o wire, como en PORTS no pueden aparecer se manda error
				else if ((Token == TOKEN_INPUT) || (Token==TOKEN_OUTPUT))	
				{ 
					ErroresYWarnings("Error! En BOX CODE aparece un INPUT o OUTPUT",MSG_Err);
				}
				//Si no es nada es que hemos encontrado texto sin definir tipo
				else
				{
					ErroresYWarnings("Error! En BOX code aparece texto que no es una seal",MSG_Err);
				}

				//Busca si despues del tipo de variable hay o no parentesis con sus caracteristicas
				Token = GetToken(frase, FALSE);
				if (Token == TOKEN_QPARENOPEN) 
				{
					GetToken(frase, TRUE);

					//Para guardar las caracteristicas vamos almacenando en un String
					caracteristicas="[";

					//Se manda la expresin
					ExprPreviaToken(frase);
					caracteristicas.push_back(QString::number(NumDec));

					//Comprobamos si hay : expr o ]
					Token = GetToken(frase, FALSE);

					if (Token == TOKEN_DOSPUNTOS)
					{
						GetToken(frase, TRUE);
						caracteristicas.push_back(":");

						//Se manda la expresin
						ExprPreviaToken(frase);
						caracteristicas.push_back(QString::number(NumDec));

						Token = GetToken(frase, FALSE);
						if (Token == TOKEN_QPARENCLOSE)	
						{
							GetToken(frase, TRUE);
							caracteristicas.push_back("]");
						}
					}

					else if (Token == TOKEN_QPARENCLOSE)
					{
						GetToken(frase, TRUE);
						caracteristicas.push_back("]");
					}
				}
				//Si no hay nada se guarda un espacio
				else
				{
					caracteristicas=(" ");
				}

				//Si despues de encontrar el tipo y las caracteristicas encontramos letras seran sus nombres. Las analizamos
				//y se guardan
				if (GetToken(frase,FALSE)==TOKEN_WORD)
				{
SIGNALSNAMECODE:
					//Bucle para conseguir la frase entera
					int j=0;
					while (frase[j].isLetter()) 
					{
						nombreSignal.push_back(frase[j]);
						j++;	
					}
					//Se guarda en la clase y se limpia la variable local
					SignalClass.NombreSignal=nombreSignal;
					TokenProcess.append(nombreSignal);
					nombreSignal.clear();

					//Se borra la palabra anterior y se buscan comas para ver si hay ms palabras. Si hay se borra
					GetToken(frase,TRUE);
					if (GetToken(frase,FALSE)==TOKEN_COMA)
					{
						GetToken(frase,TRUE);
					}

					//Se guarda dependiendo de si es Inout o Wire
					if (SignalClass.NumParametro==3)
					{
						SignalClass.InoutList.push_back(SignalClass.NombreSignal);
						SignalClass.CaractInout.push_back(caracteristicas);
					}
					if (SignalClass.NumParametro==4)
					{
						SignalClass.WireList.push_back(SignalClass.NombreSignal);
						SignalClass.CaractWire.push_back(caracteristicas);

					}
					//Si encontramos letras se reinicia el analizador
					if (GetToken(frase,FALSE)==TOKEN_WORD)
					{
						goto SIGNALSNAMECODE;
					}
				}
			}
		}
	}
	else if (modo==3)
	{
		//Async-Sync
		//En este modo tiene que ver el tipo de box que es (asincrono o no), luego ver que nombre de seal aparece,
		//ver si esta guardada y si es asi guardarla como syncrona o asyncrona en su posicin.

		//Se inicializa todo como 0
		for (int l=0; l<SignalClass.InputList.size(); l++)
		{
			SignalClass.SyncInput.push_back("ASYNC");
		}
		for (int l=0; l<SignalClass.OutputList.size(); l++)
		{
			SignalClass.SyncOutput.push_back("ASYNC");
		}
		for (int l=0; l<SignalClass.InoutList.size(); l++)
		{
			SignalClass.SyncInout.push_back("ASYNC");
		}
		for (int l=0; l<SignalClass.WireList.size(); l++)
		{
			SignalClass.SyncWire.push_back("ASYNC");
		}

		//Dependiendo el tipo se guarda como Async o Sync 
		//SI SE QUIERE AADIR ALGUN BOX A MAYORES HAY QUE AADIRLO TAMBIEN EN LA FUNCION SIMULACION DECLARATIVA ALGO DESPUES DE "VUELTA:   "
		if		((BOX.TYPE.at(IteradorMemoria) == "AsyncOps")
			||	(BOX.TYPE.at(IteradorMemoria) == "CondAsyncOps")
			||	(BOX.TYPE.at(IteradorMemoria) == "StateAsyncOps"))
		{
			Sincronismo=1;
			//Recorremos la matriz de texto en busca de toda la informacin almacenada
			for(int i=0; i<BOX.TEXTMatrix.size(); i++)
			{
				//Si la caja es tipo Event o CondSyncOps se toma la parte inferior
				if	(BOX.TYPE.at(IteradorMemoria) == "CondAsyncOps")
				{
					//Si el texto es diferente de null se guarda en frase
					if ((BOX.TEXTDOWNMatrix[i][IteradorMemoria])!="null")
					{
						frase=BOX.TEXTDOWNMatrix[i][IteradorMemoria];
					}
					//Sino no se guarda
					else
					{
						frase.clear();
				
					}
				}
				else
				{
					//Si el texto es diferente de null se guarda en frase
					if ((BOX.TEXTMatrix[i][IteradorMemoria])!="null")
					{
						frase=BOX.TEXTMatrix[i][IteradorMemoria];
					}
					//Sino no se guarda
					else
					{
						frase.clear();
				
					}
				}

				//Si el String frase esta vacio es que hemos encontrado null y no se analiza. Si no esta null se analiza
				if(frase.isNull()==false)
				{
ASYNCVUELTA:
					//Busca una palabra entera
					if (GetToken(frase,FALSE)==TOKEN_WORD)
					{
						//Bucle para conseguir la frase entera
						int j=0;
						while (frase[j].isLetter()) 
						{
							nombreSignal.push_back(frase[j]);
							j++;	
						}
						//Se guarda en la clase y se limpia la variable local
						SignalClass.NombreSignal=nombreSignal;
						nombreSignal.clear();

						//Busca en las listas de seales si aparece. Si es asi la guarda como Async o null
						for(int j=0; j<SignalClass.InputList.size(); j++)
						{
							if (SignalClass.NombreSignal==SignalClass.InputList.at(j))
							{
								if (Sincronismo==1)			{ SignalClass.SyncInput.replace(j,"ASYNC");	}
								//else						{ SignalClass.SyncInput.replace(j,"null");	}
								Encontrado=1;
							}
						}
						for(int j=0; j<SignalClass.OutputList.size(); j++)
						{
							if (SignalClass.NombreSignal==SignalClass.OutputList.at(j))
							{
								if (Sincronismo==1)			{ SignalClass.SyncOutput.replace(j,"ASYNC");}
								//else						{ SignalClass.SyncOutput.replace(j,"null");	}
								Encontrado=1;
							}
						}
						for(int j=0; j<SignalClass.InoutList.size(); j++)
						{
							if (SignalClass.NombreSignal==SignalClass.InoutList.at(j))
							{
								if (Sincronismo==1)			{ SignalClass.SyncInout.replace(j,"ASYNC");	}
								//else						{ SignalClass.SyncInout.replace(j,"null");	}
								Encontrado=1;
							}
						}
						for(int j=0; j<SignalClass.WireList.size(); j++)
						{
							if (SignalClass.NombreSignal==SignalClass.WireList.at(j))
							{
								if (Sincronismo==1)			{ SignalClass.SyncWire.replace(j,"ASYNC");	}
								//else						{ SignalClass.SyncWire.replace(j,"null");	}
								Encontrado=1;
							}
						}
						//Si no se encuentra el nombre de la seal se manda error
						if (Encontrado==0)
						{
							ErroresYWarnings("Error! Encontrado nombre de seal sin definicin previa", MSG_Err);
						}

						//Se borra el nombre
						GetToken(frase,TRUE);

						//Se busca si aparece algun simbolo despues
						//Se borra y se vuelve a buscar el siguiente nombre
						if ((GetToken(frase,FALSE) > TOKEN_COMMENT) && (GetToken(frase,FALSE) < TOKEN_NUM))
						{
							GetToken(frase,TRUE);
							goto ASYNCVUELTA;
						}
					}
				}
			}
		}
		if		((BOX.TYPE.at(IteradorMemoria) == "SyncOps")
			||	(BOX.TYPE.at(IteradorMemoria) == "Event")
			||	(BOX.TYPE.at(IteradorMemoria) == "CondSyncOps"))
		{
			Sincronismo=2;
			//Recorremos la matriz de texto en busca de toda la informacin almacenada
			for(int i=0; i<BOX.TEXTMatrix.size(); i++)
			{
				//Si la caja es tipo Event o CondSyncOps se toma la parte inferior
				if		((BOX.TYPE.at(IteradorMemoria) == "Event")
					||	(BOX.TYPE.at(IteradorMemoria) == "CondSyncOps"))
				{
					//Si el texto es diferente de null se guarda en frase
					if ((BOX.TEXTDOWNMatrix[i][IteradorMemoria])!="null")
					{
						frase=BOX.TEXTDOWNMatrix[i][IteradorMemoria];
					}
					//Sino no se guarda
					else
					{
						frase.clear();
				
					}
				}

				else
				{
					//Si el texto es diferente de null se guarda en frase
					if ((BOX.TEXTMatrix[i][IteradorMemoria])!="null")
					{
						frase=BOX.TEXTMatrix[i][IteradorMemoria];
					}
					//Sino no se guarda
					else
					{
						frase.clear();
				
					}
				}

				//Si el String frase esta vacio es que hemos encontrado null y no se analiza. Si no esta null se analiza
				if(frase.isNull()==false)
				{
SYNCVUELTA:
					//Busca una palabra entera
					if (GetToken(frase,FALSE)==TOKEN_WORD)
					{
						//Bucle para conseguir la frase entera
						int j=0;
						while (frase[j].isLetter()) 
						{
							nombreSignal.push_back(frase[j]);
							j++;	
						}
						//Se guarda en la clase y se limpia la variable local
						SignalClass.NombreSignal=nombreSignal;
						nombreSignal.clear();

						//Busca en las listas de seales si aparece. Si es asi la guarda como Async o Sync
						for(int j=0; j<SignalClass.InputList.size(); j++)
						{
							if (SignalClass.NombreSignal==SignalClass.InputList.at(j))
							{
								if (Sincronismo==2)			{ SignalClass.SyncInput.replace(j,"SYNC");	}
								//else						{ SignalClass.SyncInput.replace(j,"null");	}
								Encontrado=1;
							}
						}
						for(int j=0; j<SignalClass.OutputList.size(); j++)
						{
							if (SignalClass.NombreSignal==SignalClass.OutputList.at(j))
							{
								if (Sincronismo==2)			{ SignalClass.SyncOutput.replace(j,"SYNC");	}
								//else						{ SignalClass.SyncOutput.replace(j,"null");	}
								Encontrado=1;
							}
						}
						for(int j=0; j<SignalClass.InoutList.size(); j++)
						{
							if (SignalClass.NombreSignal==SignalClass.InoutList.at(j))
							{
								if (Sincronismo==2)			{ SignalClass.SyncInout.replace(j,"SYNC");	}
								//else						{ SignalClass.SyncInout.replace(j,"null");	}
								Encontrado=1;
							}
						}
						for(int j=0; j<SignalClass.WireList.size(); j++)
						{
							if (SignalClass.NombreSignal==SignalClass.WireList.at(j))
							{
								if (Sincronismo==2)			{ SignalClass.SyncWire.replace(j,"SYNC");	}
								//else						{ SignalClass.SyncWire.replace(j,"null");	}
								Encontrado=1;
							}
						}
						//Si no se encuentra el nombre de la seal se manda error
						if (Encontrado==0)
						{
							ErroresYWarnings("Error! Encontrado nombre de seal sin definicin previa", MSG_Err);
						}

						//Se borra el nombre
						GetToken(frase,TRUE);

						//Se busca si aparece algun simbolo despues
						//Se borra y se vuelve a buscar el siguiente nombre
						if ((GetToken(frase,FALSE) > TOKEN_COMMENT) && (GetToken(frase,FALSE) < TOKEN_NUM))
						{
							GetToken(frase,TRUE);
							goto SYNCVUELTA;
						}
					}
				}
			}
		}
	}
	textProcess->addItems(TokenProcess);
	TokenProcess.clear();
}

//Esta funcion es la encargada de simular el circuito digital en forma operativa.
//Al llegar a esta funcin ya tenemos que tener almacenado en memoria todas las seales que hay y de que tipo son
//asi como tener definido los padres, sabiendo si son de un diseo o de un test bench.
//	1-	Lo primero que hace es llamar a la funcin que reccore cada padre guardando sus estados.
//	2-	Lo siguiente es recorrer todo lo que esta entre las cabeceras y el primer estado interpretando la informacion
//		En el caso del diseo seran box Ports, Code, Initial y Event... Nos interesa lo no analizado en la Sim. Declarativa (ThreadSync y Initial)
//		En el caso del testBench hay mucha informacin valiosa que hay que analizar e interpretar
//	3-	Despues, una vez sabemos el tiempo de ciclo, vamos al primer estado (State) de cada cabecera, comprobamos la informacion
//		al inicio del ciclo, ejecutamos todo hasta encontrar el siguiente State, verificamos si algo a cambiado, y si nada ha cambiado
//		pasamos al siguiente ciclo. Si algo ha cambiado se repite hasta que nada cambie.
//		Antes de salir del ciclo se verifica lo que pida el box (=> en el ciclo que pida @Ciclo)
//		*En el caso de que sea un STATE de DISEO no hay que hacer nada
//		*En el caso de que sea un STATE de TESTBENCH hay que ver cuantos ciclos actua ese STATE (TextUp). Si esta vacio es 1 ciclo
void MainWindow::SimulacionOperativa()
{
	//Variables para ir viendo el proceso de simulacin
	QString SimOperativa;
	QStringList SimulacionOperativa;

	//Mostramos la portada que indica el inicio de la simulacin operativa
	SimulacionOperativa.append("\n//--------------SIMULACION OPERATIVA-----------------//");
	textProcess->addItems(SimulacionOperativa);
	textOut->addItems(SimulacionOperativa);
	TXTOut(SimulacionOperativa);
	SimulacionOperativa.clear();

	//Guarda los estados de cada padre
	BoxState();

	//Muestra los datos de entrada de la simulacion operativa
	MostrarDatosInicioSimOperativa();

	//Empezamos recorriendo todo lo que encontramos entre cabeceras y primer estado
	PresimulacionOperativa();

	//Se realiza la simulacion operativa
	SimulacionOperativaON();
}

//Funcion encargada de recorrer cada padre en busca de los estados que tiene y los guarda en una 
//matriz de la clase ORDENBOX
void MainWindow::BoxState()
{
	int StateNumber=0;
	const int Toco=1;											//Variable para tocar al pasar por un box
	int IteraPadre=0;											//Variables para saber los padres
	int IDPadre=0, NextPadre=0, Next0Padre=0, Next1Padre=0;		//Variables para los padres
	int IDSiguiente=0, IteraSiguiente=0;						//Variables para los box siguientes al padre
	int NextSiguiente=0, Next0Siguiente=0, Next1Siguiente=0;	//Variables para los box siguientes al padre
	int Itera0Siguiente=0, Itera1Siguiente=0;					//Variables para elegir el camino de una decision
	QString TYPE, MostrarType;

	//Variables de Simulacion declarativa, modo busqueda Ports
	int Tocado, k, ITERADOR, StateExiste=0;;
	QString State, Error;

	QString SimStates;
	QStringList SimuStates;

	//SimuStates.append("//-----------------------SIMU STATES-------------------------//");

	ORDENBOX.IDStatePadresMatrix.clear();
	ORDENBOX.ITStatePadresMatrix.clear();

	//Llamada a la funcin que inicializa la matriz
	MatrizIDStatesPadres();
	MatrizITStatesPadres();

	//Lo siguiente es recorrer todo el diagrama desde cada padre viendo que states tiene y guardando su ID en la matriz
	for(int i=0; i<ORDENBOX.IdCabeceras.size(); i++)
	{
		StateNumber=0;

		//Actualizamos valores del padre que toca
		IteraPadre=ORDENBOX.ItCabeceras.at(i);		//Iterador en memoria
		IDPadre=BOX.ID.at(IteraPadre);				//Id del padre
		NextPadre=BOX.NEXT.at(IteraPadre);			//Next del padre
		Next0Padre=BOX.NEXT0.at(IteraPadre);		//Next0 del padre 
		Next1Padre=BOX.NEXT1.at(IteraPadre);		//Next1 del padre

		//Ponemos todos los touch a 0 porque reiniciamos modo
		for(int j=0; j<=BOX.TOUCH.size(); j++)
		{
			if(BOX.TOUCH.at(j)==Toco)
			{
				BOX.TOUCH[j]=0;
			}
		}

		//Tocamos al padre
		BOX.TOUCH[IteraPadre]=Toco;

		//Si el Next del padre no es -1 seguimos
		if(NextPadre!=-1)
		{
VUELTASTATES:
			//Llamamos a la funcin para saber el iterador correspondiente al Next de el box actual	
			NextBox(NextPadre,IteraSiguiente);

			//Comprobamos si ya hemos pasado por aqui
			//Si ya lo hemos tocado no hacemos nada
			if(BOX.ID.at(IteraSiguiente)==Toco) {}
			//Si no lo hemos tocado, lo tocamos, cogemos sus datos y buscamos el siguiente
			else
			{
				//Toco el BOX
				BOX.TOUCH[IteraSiguiente]=Toco;
					
				//Consigo los datos del hijo
				IDSiguiente=BOX.ID.at(IteraSiguiente);			//Id del hijo
				NextSiguiente=BOX.NEXT.at(IteraSiguiente);		//Next del hijo
				Next0Siguiente=BOX.NEXT0.at(IteraSiguiente);	//Next0 del hijo 
				Next1Siguiente=BOX.NEXT1.at(IteraSiguiente);	//Next1 del hijo

				if (	(BOX.TYPE.at(IteraSiguiente) == "State")
					||	(BOX.TYPE.at(IteraSiguiente) == "MetaState")
					||	(BOX.TYPE.at(IteraSiguiente) == "StateAsyncOps"))
				{ 
					//Guardamos el ID del STATE
					ORDENBOX.IDStatePadresMatrix[StateNumber][i]=BOX.ID.at(IteraSiguiente);
					ORDENBOX.ITStatePadresMatrix[StateNumber][i]=IteraSiguiente;
					//SimuStates.append(QString::number(BOX.ID.at(IteraSiguiente)));
					StateNumber++;
				}
			}
			//Los datos del hijo pasan a ser los del padre
			NextPadre=NextSiguiente;
			Next0Padre=Next0Siguiente;
			Next1Padre=Next1Siguiente;
			IteraPadre=IteraSiguiente;	
		}
		//Si no estamos en una decisin o en un end simulation, NEXT siempre es distinto de -1
		//Si NEXT es -1 es que estamos en una decision y aparecen NEXT0 y NEXT1
		if (NextPadre==-1)
		{
			//Mando los next de 0 y 1 para saber cuales son sus siguientes iteraciones y comprobar si estan tocados
			NextBox(Next0Padre,Itera0Siguiente);
			NextBox(Next1Padre,Itera1Siguiente);

			//Como estamos en decision vamos por un lado y comprobamos si esta tocado y por el otro igual
			if(Next0Padre!=-1)
			{
				//Si no esta tocado actualizo NextPadre con Next0Padre y vuelvo con goto
				if(BOX.TOUCH.at(Itera0Siguiente)!=Toco)
				{
					NextPadre=Next0Padre;
					goto VUELTASTATES;
				}
			}
			if(Next1Padre!=-1)
			{
				//Si no esta tocado actualizo NextPadre con Next1Padre y vuelvo con goto
				if(BOX.TOUCH.at(Itera1Siguiente)!=Toco)
				{
					NextPadre=Next1Padre;
					goto VUELTASTATES;
				}
			}
		}
		//Si el Next es 0 puede ser o un END SIMULATION o un STATE que vuelve a otro descrito antes o no
		//La diferencia es que cuando es END SIMULATION, NEXT1 = 0
		//Cuando no lo es NEXT0 = -1 y NEXT1 = -1
		if(NextPadre==0)
		{
			//END SIMULATION
			if ((Next1Padre == 0) || (Next0Padre == 0)){}
			//STATE
			if ((Next0Padre == -1) && (Next1Padre == -1))
			{
				//Se coge el nombre del estado, se busca un estado con mismo nombre y si esta tocado se sale.
				//Si no esta tocado se sigue
				State=BOX.TEXTMatrix[0][IteraPadre];
				Tocado=0;
				for(k=0; k<=BOX.TOUCH.size(); k++)
				{
					//Si hay un estado igual SE MIRA SI ESTA TOCADO O NO
					if(BOX.TEXTMatrix[0][k]==State)
					{
						StateExiste=1;
						//Si esta tocado se seala
						if(BOX.TOUCH.at(k)==Toco)
						{
							Tocado=1;
							k=BOX.TOUCH.size();
						}
						//Sino, se guarda el iterador donde esta el estado igual
						else
						{
							ITERADOR=k;	
						}
					}
				}

				//Si esta tocado salimos
				if (Tocado==1){}
				//Sino, se saca el ID, su next y todo del estado con mismo nombre y se vuelve a VUELTA
				else
				{
					IDPadre=BOX.ID.at(ITERADOR);			//Id 
					NextPadre=BOX.NEXT.at(ITERADOR);		//Next
					Next0Padre=BOX.NEXT0.at(ITERADOR);		//Next0  
					Next1Padre=BOX.NEXT1.at(ITERADOR);		//Next1
					goto VUELTASTATES;
				}
			}	
		}
		//Si NextPadre es > 0 significa que no hemos acabado el diagrama y que hay un box siguiente.
		//Repetimos el proceso
		if(NextPadre>0)
		{	
			goto VUELTASTATES;
		}
	}
	//textProcess->addItems(SimuStates);
}

//Funcion que muestra los datos que se tienen al inicio de la simulacin operativa
void MainWindow::MostrarDatosInicioSimOperativa()
{	
	//Variables para ir viendo el proceso de simulacin
	QString SimOperativa, IdCabe, ItCabe, TypeCabe, IdStates, ItStates;
	QStringList SimulacionOperativa;
	QString MostrarTexto, NumeroPadre, Separador="\t--------";

	//Muestra el titulo de la informacin previa
	NumeroPadre="\t";
	for (int num=1; num<=ORDENBOX.ItCabeceras.size(); num++)
	{
		NumeroPadre.push_back("\tPadre N");
		NumeroPadre.push_back(QString::number(num));
	}
	SimulacionOperativa.append(NumeroPadre);
	NumeroPadre.clear();
	NumeroPadre="\t";
	for (int num=1; num<=ORDENBOX.ItCabeceras.size(); num++)
	{
		NumeroPadre.push_back(Separador);
	}
	SimulacionOperativa.append(NumeroPadre);

	//Mostramos la informacin previa para realizar la simulacin
	TypeCabe=("-TYPE Cabeceras:\t");
	IdCabe=("-ID Cabeceras:    \t");
	ItCabe=("-IT Cabeceras:    \t");
	for (int i=0; i<ORDENBOX.ItCabeceras.size(); i++)
	{
		IdCabe.push_back(QString::number(ORDENBOX.IdCabeceras.at(i)));
		ItCabe.push_back(QString::number(ORDENBOX.ItCabeceras.at(i)));
		TypeCabe.push_back(ORDENBOX.TypeCabeceras.at(i));
		IdCabe.push_back("\t");
		ItCabe.push_back("\t");
		TypeCabe.push_back("\t");
	}
	SimulacionOperativa.append(TypeCabe);
	SimulacionOperativa.append(IdCabe);
	SimulacionOperativa.append(ItCabe);
	

	//ID de los Estados
	IdStates=("-ID States:");
	SimulacionOperativa.append(IdStates);
	int number=0, Salida=0;
	//Se reccore la matriz viendo todos los posibles estados de una cabecera
	for(int fil=0; fil<100; fil++)
	{
		//Se muestra una barra
		MostrarTexto.push_back("      +ID State ");
		MostrarTexto.push_back(QString::number(fil+1));
		MostrarTexto.push_back(":\t\t");
		//Se recorren todas las cabeceras
		for (int col=0;  col<ORDENBOX.ItCabeceras.size(); col++)
		{
			//Si es diferente de 0 el valor (ID no puede ser 0) se muestra y se guarda el valor en number
			//haciendo referencia a que en esa linea hay algun estado
			if (ORDENBOX.IDStatePadresMatrix[fil][col]!=0)
			{
				MostrarTexto.push_back(QString::number(ORDENBOX.IDStatePadresMatrix[fil][col]));	
				number=fil-1;
			}
			//Si es igual a 0, en esa linea siguen siendo 0 y hemos llegado al final, se sale
			else if ((ORDENBOX.IDStatePadresMatrix[fil][col]==0) && (number!=fil-1) && (col==ORDENBOX.ItCabeceras.size()-1))
			{
				//Salimos del bucle
				fil=99;
				Salida=1;
				MostrarTexto.clear();
			}

			/*if(((ORDENBOX.IDStatePadresMatrix[ini][ji])==0) && (ORDENBOX.IDStatePadresMatrix[ini][ji-1])==0)
			{
				if(ini>number)
				{
					number=ini-1;
				}
				ini=9;
			}
			else
			{
				if (ORDENBOX.IDStatePadresMatrix[ini][ji]!=0)
				{
					MostrarTexto.push_back(QString::number(ORDENBOX.IDStatePadresMatrix[ini][ji]));	
				}
			}*/
			MostrarTexto.push_back("\t");
		}
		if(Salida==0){}
		SimulacionOperativa.append(MostrarTexto);
		MostrarTexto.clear();
	}

	//Mostramos el puesto en el iterador de los estados anteriores
	ItStates=("-IT States:");
	SimulacionOperativa.append(ItStates);
	//Recorremos tantas veces como estados hay
	for(int fil=0; fil<number+2; fil++)
	{
		MostrarTexto.push_back("      +It State ");
		MostrarTexto.push_back(QString::number(fil+1));
		MostrarTexto.push_back(":\t\t");
		for (int col=0; col<ORDENBOX.ItCabeceras.size(); col++)
		{
			MostrarTexto.push_back(QString::number(ORDENBOX.ITStatePadresMatrix[fil][col]));	
			MostrarTexto.push_back("\t");
		}
		SimulacionOperativa.append(MostrarTexto);
		MostrarTexto.clear();
	}

	textOut->addItems(SimulacionOperativa);
	TXTOut(SimulacionOperativa);
	SimulacionOperativa.clear();
	SimOperativa.clear();
}

//Funcion para recorrer todo lo que hay entre el inicio de cabeceras y el primer estado
//ESTA TODO HEHCO PENSANDO QUE SOLO TENEMOS 1 TESTBENCH Y 1 DISEO
void MainWindow::PresimulacionOperativa()
{
	int NumeroCabeceras=ORDENBOX.ItCabeceras.size();
	int IDParoDiseno, IDParoTestBench, ITParoDiseno, ITParoTestBench;

	//--------------------Variables para el TEST BENCH------------------------//
	int NextPadre, Next0Padre, Next1Padre, IteraPadre, IteraSiguiente, IDPadre;
	QString TYPEPadre, TextPadre;
	bool ok;

	QString PresimProcess;
	QStringList PresimulacionProcess;

	PresimulacionProcess.append("//------------CICLOS DE PRESIMULACION---------------//");

	//Borramos todas las seales de evento
	SignalClass.InputValor.clear();
	SignalClass.OutputValor.clear();
	SignalClass.InoutValor.clear();
	SignalClass.WireValor.clear();
	SignalClass.SignalUPEvent.clear();
	SignalClass.SignalUPEventValor.clear();
	SignalClass.SignalDOWNEvent.clear();
	SignalClass.SignalDOWNEventValor.clear();
	CYCLE=0;
	SIMULACION.SignalSim.clear();
	SIMULACION.ValoresSim.clear();
	
	//Inicializamos los valores de todas las seales a 0
	int inicializador=-1;
	for (int i=0; i<SignalClass.InputList.size(); i++)
	{
		SignalClass.InputValor.push_back(inicializador);
	}
	for (int i=0; i<SignalClass.OutputList.size(); i++)
	{
		SignalClass.OutputValor.push_back(inicializador);
	}
	for (int i=0; i<SignalClass.InoutList.size(); i++)
	{
		SignalClass.InoutValor.push_back(inicializador);
	}
	for (int i=0; i<SignalClass.WireList.size(); i++)
	{
		SignalClass.WireValor.push_back(inicializador);
	}

	//Recorremos tantas veces como cabeceras hay dependiendo si tenemos cabecera de testBench o de Diseo
	for(int i=0; i<NumeroCabeceras; i++)
	{
		//Si es TESTBENCH
		if (ORDENBOX.TypeCabeceras.at(i)==TYPETestBench)
		{
			//Mostramos en que tipo estamos
			PresimProcess.append("Padre N: ");
			PresimProcess.append(QString::number(i+1));
			PresimProcess.append(",\tTipo: ");
			PresimProcess.append(TYPETestBench);
			PresimProcess.append(",\tName: ");
			//Si hay textUp y Down, Text=null, y se muestra el TextUp
			if(BOX.TEXTMatrix[0][ORDENBOX.ItCabeceras[i]]=="null")	{	PresimProcess.append(BOX.TEXTUPMatrix[0][ORDENBOX.ItCabeceras[i]]);	}
			else													{	PresimProcess.append(BOX.TEXTMatrix[0][ORDENBOX.ItCabeceras[i]]);	}
			

			//Guardamos en que iterador esta el primer STATE, donde hay que parar
			IDParoTestBench=ORDENBOX.IDStatePadresMatrix[0][i];
			ITParoTestBench=ORDENBOX.ITStatePadresMatrix[0][i];

			PresimProcess.append(",\tPrimer State ID: ");
			PresimProcess.append(QString::number(IDParoTestBench));
			PresimProcess.append(",\tPrimer State IT: ");
			PresimProcess.append(QString::number(ITParoTestBench));

			PresimulacionProcess.append(PresimProcess);
			PresimProcess.clear();

			//Calculamos el NEXT de la cabecera
			NextPadre=BOX.NEXT.at(ORDENBOX.ItCabeceras[i]);

VUELTATESTBENCH:
			//Llamamos a la funcin para saber el iterador correspondiente al Next de el box actual (el hijo siguiente)	
			NextBox(NextPadre,IteraSiguiente);
			
			//Convertimos en Padre al primer hijo despues de header y empezamos a recorrer el TESTBENCH hasta el STATE primero
			IteraPadre=IteraSiguiente;

			//Si es diferente de el iterador de paro analiza la informacion
			if (IteraPadre!=ITParoTestBench)
			{
				//Actualizamos valores del padre que toca
				IDPadre=BOX.ID.at(IteraPadre);				//Id del padre
				TYPEPadre=BOX.TYPE.at(IteraPadre);			//Type del padre
				NextPadre=BOX.NEXT.at(IteraPadre);			//Next del padre
				Next0Padre=BOX.NEXT0.at(IteraPadre);		//Next0 del padre 
				Next1Padre=BOX.NEXT1.at(IteraPadre);		//Next1 del padre

				PresimProcess.append("  - ");
				PresimProcess.append(TYPEPadre);
				PresimProcess.append(": ");

				//Podemos encontrarnos diferentes cosas (Ports, Code, ThreadSync, Event, Default, Instance, Initial o Metastate)
				//De momento solo nos interesan los INSTANCE, que en su parte superior nos da la palabra clave empleada
				//								los THREADSYNC, que nos dan el tiempo o frecuencia del reloj
				//								los INITIAL, que nos dan los valores iniciales
				if(TYPEPadre=="Ports")
				{
					PresimProcess.append("No implantado\n");
					ErroresYWarnings("Error!: BOX PORTS no implantado en Presimulacion Operativa", MSG_Err);
				}
				if(TYPEPadre=="Code")
				{
					PresimProcess.append("No implantado\n");
					ErroresYWarnings("Warning!: BOX CODE no implantado en Presimulacion Operativa", MSG_War);
				}
				if(TYPEPadre=="ThreadSync")
				{
					//Tendremos un nombre definido en un instance, seguido de una seal 
					//de reloj (INPUT LIST), de un =, un numero digital y una magnitud
					//Ej: dut.clk = 10 ns
					TextPadre=BOX.TEXTMatrix[0][IteraPadre];
					PresimProcess.append(TextPadre);
					PresimProcess.append("\n");

					int posClock;

					//Eliminamos la primera palabra que hace referencia a un nombre del testbench
					if (GetToken(TextPadre,FALSE)==TOKEN_WORD) { GetToken(TextPadre,TRUE);	}
					
					//Eliminamos el punto que lo sigue
					if (GetToken(TextPadre,FALSE)==TOKEN_PUNTO) { GetToken(TextPadre,TRUE);	}

					//La siguiente palabra es una seal. La sacamos, la buscamos en la lista de INPUTS y guardamos su valor
					if (GetToken(TextPadre,FALSE)==TOKEN_WORD) 
					{
						int k=0;
						QString NameClock;
						while (TextPadre[k].isLetter()) 
						{
							NameClock.push_back(TextPadre[k]);
							k++;	
						}
						NameClock=NameClock.toUpper();

						
						//Buscamos en la lista de INPUTS
						for(int l=0; l<SignalClass.InputList.size(); l++)
						{
							if(SignalClass.InputList.at(l)==NameClock)		
							{	
								posClock=l;
								l=SignalClass.InputList.size()-1;
							}
						}
						//Borramos
						GetToken(TextPadre,TRUE);	
					}

					//Lo siguiente es un =. Lo borramos
					if (GetToken(TextPadre,FALSE)==TOKEN_IGUAL) { GetToken(TextPadre,TRUE);	}

					//Lo siguiente es un numero digital
					if (GetToken(TextPadre,FALSE)==TOKEN_NUM) 
					{
						int NumberCiclo, m=0;
						QString NumbCiclo;
						while (TextPadre[m].isDigit() ) 
						{	
							NumbCiclo[m]=TextPadre[m];
							m++;	
						}
						//Guardamos el numero y borramos
						NumberCiclo=NumbCiclo.toInt(&ok,10);
						TimeCycleClock=NumberCiclo;
						GetToken(TextPadre,TRUE);

						//Guardamos en la posicin de valor actual
						SignalClass.InputValor.replace(posClock,NumberCiclo);
					}

					//Solo queda saber las unidades
					if (GetToken(TextPadre,FALSE)==TOKEN_WORD) 
					{
						QString UnitClock;
						int k=0;
						while (TextPadre[k].isLetter()) 
						{
							UnitClock.push_back(TextPadre[k]);
							k++;	
						}
						UnidadReloj=UnitClock.toLower();
					}

					PresimProcess.append("       Reloj: ");
					PresimProcess.append(SignalClass.InputList.at(posClock));
					PresimProcess.append("\tCycleTime: ");
					PresimProcess.append(QString::number(SignalClass.InputValor.at(posClock)));
					PresimProcess.append(UnidadReloj);
					PresimProcess.append("\n");

				}
				if(TYPEPadre=="Event")
				{
					PresimProcess.append("No implantado\n");
					ErroresYWarnings("Error!: BOX EVENT no implantado en Presimulacion Operativa", MSG_Err);
				}
				if(TYPEPadre=="Default")
				{
					PresimProcess.append("No implantado\n");
					ErroresYWarnings("Error!: BOX DEFAULT no implantado en Presimulacion Operativa", MSG_Err);
				}
				if(TYPEPadre=="Instance")
				{
					//Me encuentro textoUp y Down. En Up tengo el nombre del Header de diseo al que corresponde en [0]
					//y en [1] el nombre que tengo que guardar
					TextPadre=BOX.TEXTUPMatrix[1][IteraPadre];
					//Se guarda en la posicin adecuada
					TESTBENCH.Nombre.push_back(TextPadre);
					PresimProcess.append(TextPadre);
					PresimProcess.append("\n");
				}
				if(TYPEPadre=="Initial")
				{
					//Define los valores iniciales de las SIGNAL INPUT
					//Recorre toda la matriz de texto del box
					for(int j=0; j<filasMatricesTexto; j++)
					{
						//Si el texto de la matriz de texto es diferente de null (valor de inicializacion) lee
						if(BOX.TEXTDOWNMatrix[j][IteraPadre]!="null")
						{
							TextPadre=BOX.TEXTDOWNMatrix[j][IteraPadre];
							PresimProcess.append(TextPadre);
							PresimProcess.append("\t");

							//Eliminamos la primera palabra que hace referencia a un nombre del testbench
							if (GetToken(TextPadre,FALSE)==TOKEN_WORD) { GetToken(TextPadre,TRUE);	}
					
							//Eliminamos el punto que lo sigue
							if (GetToken(TextPadre,FALSE)==TOKEN_PUNTO) { GetToken(TextPadre,TRUE);	}

							int posSignal;
							//La siguiente palabra es una seal. La sacamos, la buscamos en la lista de INPUTS y guardamos su valor
							if (GetToken(TextPadre,FALSE)==TOKEN_WORD) 
							{
								int k=0;
								QString NamesSignal;
								while (TextPadre[k].isLetter()) 
								{
									NamesSignal.push_back(TextPadre[k]);
									k++;	
								}
								NamesSignal=NamesSignal.toUpper();

								//Buscamos en la lista de INPUTS
								for(int l=0; l<SignalClass.InputList.size(); l++)
								{
									if(SignalClass.InputList.at(l)==NamesSignal)		
									{	
										posSignal=l;
										l=SignalClass.InputList.size()-1;
									}
								}
								//Borramos
								GetToken(TextPadre,TRUE);	
							}

							//Eliminamos el <= que lo sigue
							if (GetToken(TextPadre,FALSE)==TOKEN_MENORIGUAL) { GetToken(TextPadre,TRUE);	}

							//Lo siguiente es una expresion.
							//Si es un numero se lanza al analizador de expresiones
							int Number=-1;
							if (GetToken(TextPadre,FALSE)==TOKEN_NUM) 
							{
								ExprToken(TextPadre);
								//Guardamos el numero y borramos
								Number=NumDec;
								GetToken(TextPadre,TRUE);

								//Guardamos en la posicin de valor actual
								SignalClass.InputValor.replace(posSignal,Number);
							}
							//Sino es una seal
							else
							{
								int findSignal=0;
								ExprToken(TextPadre); //Recibimos la variable SignalNameTOKEN
								//Buscamos en las listas de seales
								for(int l=0; l<SignalClass.InputList.size(); l++)
								{
									if(SignalClass.InputList.at(l)==NameSignalTOKEN)		
									{	
										//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
										findSignal=1;
										Number=SignalClass.InputValor.at(l);
									}
								}
								for(int l=0; l<SignalClass.OutputList.size(); l++)
								{
									if(SignalClass.OutputList.at(l)==NameSignalTOKEN)		
									{	
										//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
										findSignal=1;
										Number=SignalClass.OutputValor.at(l);
									}
								}
								for(int l=0; l<SignalClass.InoutList.size(); l++)
								{
									if(SignalClass.InoutList.at(l)==NameSignalTOKEN)		
									{	
										//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
										findSignal=1;
										Number=SignalClass.InoutValor.at(l);
									}
								}
								for(int l=0; l<SignalClass.WireList.size(); l++)
								{
									if(SignalClass.WireList.at(l)==NameSignalTOKEN)		
									{	
										//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
										findSignal=1;
										Number=SignalClass.WireValor.at(l);
									}
								}

								if(findSignal==1)
								{
									//Guardamos en la posicin de valor actual
									SignalClass.InputValor.replace(posSignal,Number);
								}
							}
						}
					}
					PresimProcess.append("\n");
				}
				if(TYPEPadre=="Metastate")
				{
					PresimProcess.append("No implantado\n");
					ErroresYWarnings("Error!: BOX METASTATE no implantado en Presimulacion Operativa", MSG_Err);
				}
				goto VUELTATESTBENCH;
			}

			//Si no lo es muestra el proceso, las seales y sale
			PresimulacionProcess.append(PresimProcess);
			PresimProcess.clear();
		}
		//Si es DISEO
		if (ORDENBOX.TypeCabeceras.at(i)==TYPEDiseno)
		{
			//Mostramos en que tipo estamos
			PresimProcess.append("Padre N: ");
			PresimProcess.append(QString::number(i+1));
			PresimProcess.append(",\tTipo: ");
			PresimProcess.append(TYPEDiseno);
			PresimProcess.append(",        \tName: ");
			//Si hay textUp y Down, Text=null, y se muestra el TextUp
			if(BOX.TEXTMatrix[0][ORDENBOX.ItCabeceras[i]]=="null")	{	PresimProcess.append(BOX.TEXTUPMatrix[0][ORDENBOX.ItCabeceras[i]]);	}
			else													{	PresimProcess.append(BOX.TEXTMatrix[0][ORDENBOX.ItCabeceras[i]]);	}
			
			//Guardamos en que iterador esta el primer STATE, donde hay que parar
			IDParoDiseno=ORDENBOX.IDStatePadresMatrix[0][i];
			ITParoDiseno=ORDENBOX.ITStatePadresMatrix[0][i];

			PresimProcess.append(",\tPrimer State ID: ");
			PresimProcess.append(QString::number(IDParoDiseno));
			PresimProcess.append(",\tPrimer State IT: ");
			PresimProcess.append(QString::number(ITParoDiseno));

			PresimulacionProcess.append(PresimProcess);
			PresimProcess.clear();

			//Calculamos el NEXT de la cabecera
			NextPadre=BOX.NEXT.at(ORDENBOX.ItCabeceras[i]);

VUELTADISENO:
			//Llamamos a la funcin para saber el iterador correspondiente al Next de el box actual (el hijo siguiente)	
			NextBox(NextPadre,IteraSiguiente);
			
			//Convertimos en Padre al primer hijo despues de header y empezamos a recorrer el TESTBENCH hasta el STATE primero
			IteraPadre=IteraSiguiente;

			//Si es diferente de el iterador de paro analiza la informacion
			if (IteraPadre!=ITParoDiseno)
			{
				//Actualizamos valores del padre que toca
				IDPadre=BOX.ID.at(IteraPadre);				//Id del padre
				TYPEPadre=BOX.TYPE.at(IteraPadre);			//Type del padre
				NextPadre=BOX.NEXT.at(IteraPadre);			//Next del padre
				Next0Padre=BOX.NEXT0.at(IteraPadre);		//Next0 del padre 
				Next1Padre=BOX.NEXT1.at(IteraPadre);		//Next1 del padre

				PresimProcess.append("  - ");
				PresimProcess.append(TYPEPadre);
				PresimProcess.append(": ");

				//Podemos encontrarnos diferentes cosas (Ports, Code, ThreadSync, Event, Default, Instance, Initial o Metastate)
				//De momento solo nos interesan los INSTANCE, que en su parte superior nos da la palabra clave empleada
				//								los THREADSYNC, que nos dan el tiempo o frecuencia del reloj
				//								los INITIAL, que nos dan los valores iniciales
				if(TYPEPadre=="Ports")
				{
					PresimProcess.append("Ya analizado en Sim.Declarativa\n");
				}
				if(TYPEPadre=="Code")
				{
					PresimProcess.append("Ya analizado en Sim.Declarativa\n");
				}
				if(TYPEPadre=="ThreadSync")
				{
					//Tendremos una seal de reloj (INPUT LIST)
					TextPadre=BOX.TEXTMatrix[0][IteraPadre];
					PresimProcess.append(TextPadre);
					PresimProcess.append("\n");

					int posClock;

					//La siguiente palabra es una seal. La sacamos, la buscamos en la lista de INPUTS y guardamos su valor
					if (GetToken(TextPadre,FALSE)==TOKEN_WORD) 
					{
						int k=0;
						QString NameClock;
						while (TextPadre[k].isLetter()) 
						{
							NameClock.push_back(TextPadre[k]);
							k++;	
						}
						NameClock=NameClock.toUpper();

						
						//Buscamos en la lista de INPUTS
						for(int l=0; l<SignalClass.InputList.size(); l++)
						{
							if(SignalClass.InputList.at(l)==NameClock)		
							{	
								posClock=l;
								l=SignalClass.InputList.size()-1;
							}
						}
						//Borramos
						GetToken(TextPadre,TRUE);	
					}
					PresimProcess.append("       Reloj: ");
					PresimProcess.append(SignalClass.InputList.at(posClock));
					PresimProcess.append("\n");
				}
				if(TYPEPadre=="Event")
				{
					//VAN A APARECER TEXTOS UP Y DOWN. EN EL UP TE INDICAN CUANDO SE EJECUTA EL EVENTO
					//Y EN EL DOWN QUE SE HACE CUANDO OCURRE
					//Para guardar toda esa informacin vamos a emplear las variables siguientes de SignalClass.:
					//		- SignalClass.SignalUPEvent
					//		- SignalClass.SignalUPEventValor
					//		- SignalClass.SignalDOWNEvent
					//		- SignalClass.SignalDOWNEventValor
					//Con esto, sabremos que cuando tenemos una seal en SignalUPEvent, si su valor es el de SignalUPEventValor
					//Tenemos que poner su SignalDOWNEvent de la misma posicin de iterador en su valor de SignalDOWNEventValor

					//PRIMERO HAY QUE RECORRER LA MATRIZ DE TEXTO DOWN Y SE ALMACENA TODO
					//Recorre toda la matriz de texto DOWN del box
					for(int j=0; j<filasMatricesTexto; j++)
					{
						//Si el texto de la matriz de texto es diferente de null (valor de inicializacion) lee
						if(BOX.TEXTDOWNMatrix[j][IteraPadre]!="null")
						{
							TextPadre=BOX.TEXTDOWNMatrix[j][IteraPadre];
							PresimProcess.append(TextPadre);
							PresimProcess.append("\t");

							int posSignal;
							//La siguiente palabra es una seal. La sacamos, la buscamos en la lista de INPUTS y guardamos su valor
							if (GetToken(TextPadre,FALSE)==TOKEN_WORD) 
							{
								int k=0;
								QString NamesSignal;
								while (TextPadre[k].isLetter()) 
								{
									NamesSignal.push_back(TextPadre[k]);
									k++;	
								}
								NamesSignal=NamesSignal.toUpper();

								//Buscamos en la lista de OUTPUTS, WIRES Y INOUTS que son las que puedo modificar
								for(int l=0; l<SignalClass.OutputList.size(); l++)
								{
									if(SignalClass.OutputList.at(l)==NamesSignal)		
									{	
										posSignal=l;
										l=SignalClass.OutputList.size()-1;
										//Guardamos en la lista de seales que aparecen en eventos el nombre
										SignalClass.SignalDOWNEvent.push_back(NamesSignal);
									}
								}
								for(int l=0; l<SignalClass.InoutList.size(); l++)
								{
									if(SignalClass.InoutList.at(l)==NamesSignal)		
									{	
										posSignal=l;
										l=SignalClass.InoutList.size()-1;
										//Guardamos en la lista de seales que aparecen en eventos el nombre
										SignalClass.SignalDOWNEvent.push_back(NamesSignal);
									}
								}
								for(int l=0; l<SignalClass.WireList.size(); l++)
								{
									if(SignalClass.WireList.at(l)==NamesSignal)		
									{	
										posSignal=l;
										l=SignalClass.WireList.size()-1;
										//Guardamos en la lista de seales que aparecen en eventos el nombre
										SignalClass.SignalDOWNEvent.push_back(NamesSignal);
									}
								}
								//Borramos
								GetToken(TextPadre,TRUE);	
							}

							//Eliminamos el <= que lo sigue
							if (GetToken(TextPadre,FALSE)==TOKEN_MENORIGUAL) { GetToken(TextPadre,TRUE);	}

							//Lo siguiente es una expresion.
							//Si es un numero se lanza al analizador de expresiones
							int Number=-1;
							if (GetToken(TextPadre,FALSE)==TOKEN_NUM) 
							{
								ExprToken(TextPadre);
								//Guardamos el numero y borramos
								Number=NumDec;
								GetToken(TextPadre,TRUE);

								//Guardamos el valor que activa la seal de evento
								SignalClass.SignalDOWNEventValor.push_back(Number);
							}
							//Sino es una seal
							else
							{
								int findSignal=0;
								ExprToken(TextPadre); //Recibimos la variable SignalNameTOKEN
								//Buscamos en las listas de seales
								for(int l=0; l<SignalClass.InputList.size(); l++)
								{
									if(SignalClass.InputList.at(l)==NameSignalTOKEN)		
									{	
										//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
										findSignal=1;
										Number=SignalClass.InputValor.at(l);
									}
								}
								for(int l=0; l<SignalClass.OutputList.size(); l++)
								{
									if(SignalClass.OutputList.at(l)==NameSignalTOKEN)		
									{	
										//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
										findSignal=1;
										Number=SignalClass.OutputValor.at(l);
									}
								}
								for(int l=0; l<SignalClass.InoutList.size(); l++)
								{
									if(SignalClass.InoutList.at(l)==NameSignalTOKEN)		
									{	
										//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
										findSignal=1;
										Number=SignalClass.InoutValor.at(l);
									}
								}
								for(int l=0; l<SignalClass.WireList.size(); l++)
								{
									if(SignalClass.WireList.at(l)==NameSignalTOKEN)		
									{	
										//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
										findSignal=1;
										Number=SignalClass.WireValor.at(l);
									}
								}

								if(findSignal==1)
								{
									//Guardamos el valor que activa la seal de evento
									SignalClass.SignalDOWNEventValor.push_back(Number);
								}
							}
						}
					}
					//DESPUES SE RECORRE LA MATRIZ DE TEXTO UP
					//Recorre toda la matriz de texto UP del box
					for(int p=0; p<filasMatricesTextoUp; p++)
					{
						//Si el texto de la matriz de texto es diferente de null (valor de inicializacion) lee
						if(BOX.TEXTUPMatrix[p][IteraPadre]!="null")
						{
							TextPadre=BOX.TEXTUPMatrix[p][IteraPadre];
							PresimProcess.append(TextPadre);
							PresimProcess.append("\t");

							int posSignal;
							//La siguiente palabra es una seal. La sacamos, la buscamos en la lista de INPUTS y guardamos su valor
							if (GetToken(TextPadre,FALSE)==TOKEN_WORD) 
							{
								int k=0;
								QString NamesSignal;
								while (TextPadre[k].isLetter()) 
								{
									NamesSignal.push_back(TextPadre[k]);
									k++;	
								}
								NamesSignal=NamesSignal.toUpper();

								//Buscamos en la lista de INPUTS que son las que ejecutan eventos
								for(int l=0; l<SignalClass.InputList.size(); l++)
								{
									if(SignalClass.InputList.at(l)==NamesSignal)		
									{	
										posSignal=l;
										l=SignalClass.InputList.size()-1;
										//Guardamos en la lista de seales que aparecen en eventos el nombre
										SignalClass.SignalUPEvent.push_back(NamesSignal);
									}
								}
								//Borramos
								GetToken(TextPadre,TRUE);	
							}

							//Eliminamos el == que lo sigue
							if (GetToken(TextPadre,FALSE)==TOKEN_DOSIGUAL) { GetToken(TextPadre,TRUE);	}

							//Lo siguiente es una expresion.
							//Si es un numero se lanza al analizador de expresiones
							int Number=-1;
							if (GetToken(TextPadre,FALSE)==TOKEN_NUM) 
							{
								ExprToken(TextPadre);
								//Guardamos el numero y borramos
								Number=NumDec;
								GetToken(TextPadre,TRUE);

								//Guardamos el valor que activa la seal de evento
								SignalClass.SignalUPEventValor.push_back(Number);
							}
							//Sino es una seal
							else
							{
								int findSignal=0;
								ExprToken(TextPadre); //Recibimos la variable SignalNameTOKEN
								//Buscamos en las listas de seales
								for(int l=0; l<SignalClass.InputList.size(); l++)
								{
									if(SignalClass.InputList.at(l)==NameSignalTOKEN)		
									{	
										//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
										findSignal=1;
										Number=SignalClass.InputValor.at(l);
									}
								}
								for(int l=0; l<SignalClass.OutputList.size(); l++)
								{
									if(SignalClass.OutputList.at(l)==NameSignalTOKEN)		
									{	
										//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
										findSignal=1;
										Number=SignalClass.OutputValor.at(l);
									}
								}
								for(int l=0; l<SignalClass.InoutList.size(); l++)
								{
									if(SignalClass.InoutList.at(l)==NameSignalTOKEN)		
									{	
										//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
										findSignal=1;
										Number=SignalClass.InoutValor.at(l);
									}
								}
								for(int l=0; l<SignalClass.WireList.size(); l++)
								{
									if(SignalClass.WireList.at(l)==NameSignalTOKEN)		
									{	
										//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
										findSignal=1;
										Number=SignalClass.WireValor.at(l);
									}
								}

								if(findSignal==1)
								{
									//Guardamos el valor que activa la seal de evento
									SignalClass.SignalUPEventValor.push_back(Number);
								}
							}
						}
					}
						
					//ANTES DE SALIR DEL ANALISIS DE ESTE EVENTO, PARA QUE QUEDE TODO CORRECTO HAY QUE COPIAR 
					//LO DE TEXTUP TANTAS VECES COMO TEXTDOWN HAYA PARA QUE TODO TEXTDOWN TENGA EN SU MISMA POSICION EN
					//TEXTUP SU CONDICION DE EJECUCION
					int TamanoSignalUp=SignalClass.SignalUPEvent.size();
					int TamanoSignalDown=SignalClass.SignalDOWNEvent.size();

					//Si el tamao es diferente
					if(TamanoSignalUp!=TamanoSignalDown)
					{
						//EJEMPLO: suponiendo que estamos en un 2 event, que indica que cuando ok=1 tiene que pasar todo
						//lo de debajo. Tenemos que copiar ok en [3] y [4] tal cual (con reset ya lo habriamos hecho igual aqui 
						//pero en la pasada anterior
						//Posicion:			 [0]		[1]		[2]		[3]		[4]		[5]
						//SignalUp:			reset	   reset	 ok
						//SignalUpValor:	  0			 0		 1
						//SignalDown:		done		go		done	go		inA
						//SignalDownValor:	  1			 0		 0		 1		 0
						
						int Diferencia=TamanoSignalDown-TamanoSignalUp;
						QString NameSignalUp=SignalClass.SignalUPEvent.at(TamanoSignalUp-1);
						int ValorSignalUp=SignalClass.SignalUPEventValor.at(TamanoSignalUp-1);

						//Para el ejemplo de arriba, diferencia seria 2, asique habria que hacer un for de 2 pasadas
						for(int n=0; n<Diferencia; n++)
						{
							SignalClass.SignalUPEvent.push_back(NameSignalUp);
							SignalClass.SignalUPEventValor.push_back(ValorSignalUp);
						}
					}
					PresimProcess.append("\n");
				}
				if(TYPEPadre=="Default")
				{
					//Define los valores iniciales de las SIGNAL INPUT
					//Recorre toda la matriz de texto del box
					for(int j=0; j<filasMatricesTexto; j++)
					{
						//Si el texto de la matriz de texto es diferente de null (valor de inicializacion) lee
						if(BOX.TEXTDOWNMatrix[j][IteraPadre]!="null")
						{
							TextPadre=BOX.TEXTDOWNMatrix[j][IteraPadre];
							PresimProcess.append(TextPadre);
							PresimProcess.append("\t");

							int posSignal;

							//La siguiente palabra es una seal. La sacamos, la buscamos en la lista de SIGNALS y guardamos su valor
							if (GetToken(TextPadre,FALSE)==TOKEN_WORD) 
							{
								int k=0;
								QString NamesSignal;
								while (TextPadre[k].isLetter()) 
								{
									NamesSignal.push_back(TextPadre[k]);
									k++;	
								}
								NamesSignal=NamesSignal.toUpper();

								if (SignalClass.InputList.size()!=0)
								{
									//Buscamos en la lista de INPUTS
									for(int l=0; l<SignalClass.InputList.size(); l++)
									{
										if(SignalClass.InputList.at(l)==NamesSignal)		
										{	
											posSignal=l;
											l=SignalClass.InputList.size()-1;
											SignalClass.SignalDefaultName.push_back(NamesSignal);
										}
									}
								}
								if (SignalClass.OutputList.size()!=0)
								{
									//Buscamos en la lista de OUTPUTS
									for(int l=0; l<SignalClass.OutputList.size(); l++)
									{
										if(SignalClass.OutputList.at(l)==NamesSignal)		
										{	
											posSignal=l;
											l=SignalClass.OutputList.size()-1;
											SignalClass.SignalDefaultName.push_back(NamesSignal);
										}
									}
								}
								if (SignalClass.InoutList.size()!=0)
								{
									//Buscamos en la lista de INOUTS
									for(int l=0; l<SignalClass.InoutList.size(); l++)
									{
										if(SignalClass.InoutList.at(l)==NamesSignal)		
										{	
											posSignal=l;
											l=SignalClass.InoutList.size()-1;
											SignalClass.SignalDefaultName.push_back(NamesSignal);
										}
									}
								}
								if (SignalClass.WireList.size()!=0)
								{
									//Buscamos en la lista de WIRE
									for(int l=0; l<SignalClass.WireList.size(); l++)
									{
										if(SignalClass.WireList.at(l)==NamesSignal)		
										{	
											posSignal=l;
											l=SignalClass.WireList.size()-1;
											SignalClass.SignalDefaultName.push_back(NamesSignal);

										}
									}
								}
								//Borramos
								GetToken(TextPadre,TRUE);	
							}

							//Eliminamos el <= que lo sigue
							if (GetToken(TextPadre,FALSE)==TOKEN_MENORIGUAL) { GetToken(TextPadre,TRUE);	}

							//Lo siguiente es una expresion.
							int Number=-1;
							ExprToken(TextPadre);

							//Guardamos el numero y borramos
							GetToken(TextPadre,TRUE);
									
							//Si encontramos logica
							if(HayLogica==1)
							{
								//Dependiendo la comparacion Verdadera o falsa guardo 1 o 0
								if(ComparacionExpresiones==true)	{	Number=1;	}
								else								{	Number=0;	}
							}
							//Sino hay logica
							else
							{
								//Si hay comparacion igual que con la logica
								if(HayComparacion==1)
								{
									if(ComparacionExpresiones==true)	{	Number=1;	}
									else								{	Number=0;	}
								}
								//Si solo es una expresion guardo el numero que me devuelve
								else
								{
									Number=NumDec;
								}
							}

							//Guardamos el numero de la seal por defecto
							SignalClass.SignalDefaultValor.push_back(Number);
							SignalClass.OutputValor.replace(posSignal,Number);
						}
					}
					//		//La siguiente palabra es una seal. La sacamos, la buscamos en la lista de OUTPUTS y guardamos su valor
					//		if (GetToken(TextPadre,FALSE)==TOKEN_WORD) 
					//		{
					//			int k=0;
					//			QString NamesSignal;
					//			while (TextPadre[k].isLetter()) 
					//			{
					//				NamesSignal.push_back(TextPadre[k]);
					//				k++;	
					//			}
					//			NamesSignal=NamesSignal.toUpper();

					//			//Buscamos en la lista de OUTPUTS
					//			for(int l=0; l<SignalClass.OutputList.size(); l++)
					//			{
					//				if(SignalClass.OutputList.at(l)==NamesSignal)		
					//				{	
					//					posSignal=l;
					//					l=SignalClass.OutputList.size()-1;
					//				}
					//			}
					//			//Borramos
					//			GetToken(TextPadre,TRUE);	
					//		}

					//		//Eliminamos el <= que lo sigue
					//		if (GetToken(TextPadre,FALSE)==TOKEN_MENORIGUAL) { GetToken(TextPadre,TRUE);	}

					//		//Lo siguiente es una expresion.
					//		//Si es un numero se lanza al analizador de expresiones
					//		int Number=-1;
					//		if (GetToken(TextPadre,FALSE)==TOKEN_NUM) 
					//		{
					//			ExprToken(TextPadre);
					//			//Guardamos el numero y borramos
					//			Number=NumDec;
					//			GetToken(TextPadre,TRUE);

					//			//Guardamos en la posicin de valor actual
					//			SignalClass.OutputValor.replace(posSignal,Number);
					//		}
					//		//Sino es una seal
					//		else
					//		{
					//			int findSignal=0;
					//			ExprToken(TextPadre); //Recibimos la variable SignalNameTOKEN
					//			//Buscamos en las listas de seales
					//			for(int l=0; l<SignalClass.InputList.size(); l++)
					//			{
					//				if(SignalClass.InputList.at(l)==NameSignalTOKEN)		
					//				{	
					//					//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
					//					findSignal=1;
					//					Number=SignalClass.InputValor.at(l);
					//				}
					//			}
					//			for(int l=0; l<SignalClass.OutputList.size(); l++)
					//			{
					//				if(SignalClass.OutputList.at(l)==NameSignalTOKEN)		
					//				{	
					//					//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
					//					findSignal=1;
					//					Number=SignalClass.OutputValor.at(l);
					//				}
					//			}
					//			for(int l=0; l<SignalClass.InoutList.size(); l++)
					//			{
					//				if(SignalClass.InoutList.at(l)==NameSignalTOKEN)		
					//				{	
					//					//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
					//					findSignal=1;
					//					Number=SignalClass.InoutValor.at(l);
					//				}
					//			}
					//			for(int l=0; l<SignalClass.WireList.size(); l++)
					//			{
					//				if(SignalClass.WireList.at(l)==NameSignalTOKEN)		
					//				{	
					//					//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
					//					findSignal=1;
					//					Number=SignalClass.WireValor.at(l);
					//				}
					//			}

					//			if(findSignal==1)
					//			{
					//				//Guardamos en la posicin de valor actual
					//				SignalClass.OutputValor.replace(posSignal,Number);
					//			}
					//		}
					//	}
					//}
					PresimProcess.append("\n");
				}
				if(TYPEPadre=="Instance")
				{
					PresimProcess.append("No implantado\n");
					ErroresYWarnings("Error!: BOX INSTANCE no implantado en Presimulacion Operativa", MSG_Err);
				}
				if(TYPEPadre=="Initial")
				{
					PresimProcess.append("No implantado\n");
					ErroresYWarnings("Error!: BOX INITIAL no implantado en Presimulacion Operativa", MSG_Err);
				}
				if(TYPEPadre=="Metastate")
				{
					PresimProcess.append("No implantado\n");
					ErroresYWarnings("Error!: BOX METASTATE no implantado en Presimulacion Operativa", MSG_Err);
				}
				goto VUELTADISENO;
			}

			//Si no lo es muestra el proceso, las seales y sale
			PresimulacionProcess.append(PresimProcess);
			PresimProcess.clear();
		}
	}

	//Se muestran los eventos que hay en la simulacin
	MostrarEventos();

	//Antes de comprobar la presimulacin ejecuto el analizador de eventos para el ciclo inicial
	AnalizadorEventos();

	//Se inicializa la matriz para guardar los datos
	int NumSignals = SignalClass.InputList.size()+SignalClass.OutputList.size()+SignalClass.InoutList.size()+SignalClass.WireList.size();
	MatrizDatosSimulacionFinal(NumSignals);
	
	//En la presimulacion guardo el nombre de las seales
	for(int i=0; i<SignalClass.InputList.size(); i++)
	{
		SIMULACION.SignalSim.push_back(SignalClass.InputList.at(i));
	}
	for(int i=0; i<SignalClass.OutputList.size(); i++)
	{
		SIMULACION.SignalSim.push_back(SignalClass.OutputList.at(i));
	}
	for(int i=0; i<SignalClass.InoutList.size(); i++)
	{
		SIMULACION.SignalSim.push_back(SignalClass.InoutList.at(i));
	}
	for(int i=0; i<SignalClass.WireList.size(); i++)
	{
		SIMULACION.SignalSim.push_back(SignalClass.WireList.at(i));
	}

	//Guardo las seales de este ciclo
	GuardarSimulacion(CYCLE);

	//AUMENTO EL CICLO. Toda esta presimulacin corresponde al CICLO 0 de reloj
	CYCLE=CYCLE+1;

	PresimulacionProcess.append(PresimProcess);
	textProcess->addItems(PresimulacionProcess);
}

//Funcion que se encarga de ir recorriendo los esquemas una vez realizada la presimulacion
//para simular operativamente el circuito.
void MainWindow::SimulacionOperativaON()
{
	int STATESTEST=0, STATESDISENO=0, testeo=0;
	int NumeroCabeceras=ORDENBOX.ItCabeceras.size();
	int IDInitDiseno, IDInitTestBench, ITInitDiseno, ITInitTestBench;
	int IDParoDiseno, IDParoTestBench, ITParoDiseno, ITParoTestBench;

	//--------------------Variables para el TEST BENCH------------------------//
	int NextPadre, Next0Padre, Next1Padre, IteraPadre, IteraSiguiente, IDPadre;
	QString TYPEInit, TYPEParo, TextInit, TYPEPadre, TextPadre;
	int PRIMERAPASADA=1;
	bool ok;
	int CICLOREPETICIONSTATE=1;			//Variable para saber cuantas veces repetimos el ciclo
	int CICLOACTUAL=0;					//Variable para saber en que ciclo me encuentro

	int FINAL=0;		//Variable que al encontrar un METASTATE con endsimulation se activa
	int Vuelta=0;

	QString SimOpProcess;
	QStringList SimulacionOpProcess;

	SimulacionOpProcess.append("\n//------------CICLOS DE SIMULACION---------------//");

VUELTASIMULACIONOPERATIVA:

	SimOpProcess.append("CICLO SE SIMULACION NUEVO: ");
	SimOpProcess.append(QString::number(CYCLE));
	SimOpProcess.append("\n");

	//DEPENDIENDO EL NUMERO DE ESQUEMAS QUE TENGAMOS TENEMOS QUE RECORRER ASI DE VECES ENTRE ESTADOS
	//Recorremos tantas veces como cabeceras hay dependiendo si tenemos cabecera de testBench o de Diseo
	for(int i=0; i<NumeroCabeceras; i++)
	{
//VUELTACAMBIOS:

		//Si es TESTBENCH
		if (ORDENBOX.TypeCabeceras.at(i)==TYPETestBench)
		{

			testeo=1;
			SimOpProcess.append(" + TEST BENCH:\n");
			//Vamos aumentando cada vez que hemos acabado de analizar todo entre 1 State y otro
			IDInitTestBench=ORDENBOX.IDStatePadresMatrix[STATESTEST][i];
			ITInitTestBench=ORDENBOX.ITStatePadresMatrix[STATESTEST][i];
			IDParoTestBench=ORDENBOX.IDStatePadresMatrix[STATESTEST+1][i];
			ITParoTestBench=ORDENBOX.ITStatePadresMatrix[STATESTEST+1][i];

			SimOpProcess.append("\tState inicio ID: ");
			SimOpProcess.append(QString::number(IDInitTestBench));
			SimOpProcess.append(",\tState inicio IT: ");
			SimOpProcess.append(QString::number(ITInitTestBench));
			TYPEInit=BOX.TYPE[ITInitTestBench];
			SimOpProcess.append(",\tSTATE TYPE: ");
			SimOpProcess.append(TYPEInit);
			SimOpProcess.append("\t     ->");
			SimOpProcess.append("\tState paro ID: ");
			SimOpProcess.append(QString::number(IDParoTestBench));
			SimOpProcess.append(",\t\tState paro IT: ");
			SimOpProcess.append(QString::number(ITParoTestBench));
			TYPEParo=BOX.TYPE[ITParoTestBench];
			SimOpProcess.append(",\t\tSTATE TYPE: ");
			SimOpProcess.append(TYPEParo);
			
			//Una vez tengo claro en que STATE estoy y a cual tengo que ir cuando acabe, compruebo en que TYPE estoy
			SimulacionOpProcess.append(SimOpProcess);
			SimOpProcess.clear();

			//Obtenemos la informacion de TYPE y Texts
			QString TextTBUpPadre, TextTBDownPadre, TextTBPadre;

			TYPEPadre=BOX.TYPE.at(ITInitTestBench);					//Type del padre
			TextTBDownPadre=BOX.TEXTDOWNMatrix[0][ITInitTestBench];	//TextDown del padre
			TextTBPadre=BOX.TEXTMatrix[0][ITInitTestBench];			//Text del padre

			SimOpProcess.append("  - ");
			SimOpProcess.append(TYPEPadre);
			SimOpProcess.append(": \n");

			//Podemos encontrarnos diferentes TYPE BOX
			//De momento solo nos interesan los StateAsyncOps, que en su parte superior nos dan los ciclos de ejecuccion de lo de abajo
			//								los State, que nos dan estados simples
			//								los MetaState, que nos indicara un fin
			if(TYPEPadre=="StateSyncOps")
			{
				//Leemos lo que hay en textUp para saber el numero de ciclos
				TextTBUpPadre=BOX.TEXTUPMatrix[0][ITInitTestBench];		//TextUp del padre
				SimOpProcess.append("\tTextUp: ");
				SimOpProcess.append(TextTBUpPadre);
				SimOpProcess.append("\n");
				
				//Si es la primera vez que entro en este STATE tengo que sacar su tiempo de ciclo activo
				if (PRIMERAPASADA==1)
				{
					CICLOACTUAL=0;
					if (TextTBUpPadre.isNull())
					{
						CICLOREPETICIONSTATE=1;
					}
					else
					{
						for(int m=0; m<40; m++)
						{
							//NOS ENCONTRAMOS UN NOMBRE CON O SIN DIGITOS SEGUIDO DE <NUM>. Necesitamos NUM
							if(GetToken(TextTBUpPadre,FALSE)==TOKEN_MENOR)
							{
								//Se borra el menor <
								GetToken(TextTBUpPadre,TRUE);
								/*SimOpProcess.append(TextTBUpPadre);
								SimOpProcess.append("\n");*/
								//Se capta el numero de ciclos
								int l=0;
								QString NumDecString;
								while (TextTBUpPadre[l].isDigit() ) 
								{	
									NumDecString[l]=TextTBUpPadre[l];
									l++;	
								}
								//Guardamos el numero y borramos
								CICLOREPETICIONSTATE=NumDecString.toInt(&ok,10);
								GetToken(TextTBUpPadre, TRUE);
							}
							else
							{
								GetToken(TextTBUpPadre,TRUE);
								/*SimOpProcess.append(TextTBUpPadre);
								SimOpProcess.append("\n");*/
							}
							if (GetToken(TextTBUpPadre,FALSE)==TOKEN_MAYOR)
							{
								m=10;
							}
						}
					}
				}

				//Interpreto lo que me encuentro y actuo. Puede haber:
				//
				//		-	=>		:	Verificar lo que haya
				//		-	=>  @N	:	Verificar en el ciclo indicado
				//		-			:	Actuar
				//		-		@N	:	Actuar en el ciclo indicado
				for (int n=0; n<filasMatricesTexto; n++)
				{
					QString TextDownTB=BOX.TEXTDOWNMatrix[n][ITInitTestBench];
					int CicloVerificacion=0, CicloActuacion=0;
					bool ok1;
					
					//Comprobamos si el texto es null
					if (TextDownTB=="null")
					{
						n=filasMatricesTexto-1;
					}
					//Si hay texto
					else
					{
						//Si tenemos una verificacion solo lo hacemos en la primera pasada y guardamos en la matriz de verificaciones
						if (GetToken(TextDownTB,FALSE) == TOKEN_VERIFICACION)
						{
							//Mostramos el texto
							SimOpProcess.append("\t+ Texto DOWN:\t");
							SimOpProcess.append(TextDownTB);

							//Borramos la verificacion =>
							GetToken(TextDownTB,TRUE);

							SimOpProcess.append("    \t->\tVerificacion ");
			
							//Si tenemos @
							if (GetToken(TextDownTB,FALSE) == TOKEN_ARROBA)
							{
								if (PRIMERAPASADA==1)
								{
									//Borramos y conseguimos el N de ciclo de la verificacion
									GetToken(TextDownTB,TRUE);
									int p=0;
									QString NumDecStringDOWN;
									while (TextDownTB[p].isDigit() ) 
									{	
										NumDecStringDOWN[p]=TextDownTB[p];
										p++;	
									}
									CicloVerificacion=NumDecStringDOWN.toInt(&ok1,10);
									GetToken(TextDownTB, TRUE);
									SimOpProcess.append("en ciclo: ");
									SimOpProcess.append(QString::number(CicloVerificacion));
									SimOpProcess.append(".      CYCLE:");
									SimOpProcess.append(QString::number(CYCLE));
									SimOpProcess.append(".      Suma:");
									SimOpProcess.append(QString::number(CYCLE + CicloVerificacion - 1));

									VERIFICACIONES.Ciclo.push_back((CYCLE + CicloVerificacion - 1));

									//La primera palabra sera la del simulador test bench definida en Instance (Ej.: dut)
									//La borramos y el punto de despues
									if (GetToken(TextDownTB,FALSE) == TOKEN_WORD)		{ GetToken(TextDownTB,TRUE); }
									if (GetToken(TextDownTB,FALSE) == TOKEN_PUNTO)		{ GetToken(TextDownTB,TRUE); }

									int posSignal;

									//La siguiente palabra es una seal. La sacamos, la buscamos en la lista de SIGNALS y guardamos su valor
									if (GetToken(TextDownTB,FALSE)==TOKEN_WORD) 
									{
										int k=0;
										QString NamesSignal;
										while (TextDownTB[k].isLetter()) 
										{
											NamesSignal.push_back(TextDownTB[k]);
											k++;	
										}
										NamesSignal=NamesSignal.toUpper();

										if (SignalClass.InputList.size()!=0)
										{
											//Buscamos en la lista de INPUTS
											for(int l=0; l<SignalClass.InputList.size(); l++)
											{
												if(SignalClass.InputList.at(l)==NamesSignal)		
												{	
													posSignal=l;
													l=SignalClass.InputList.size()-1;
													VERIFICACIONES.NameSignal.push_back(NamesSignal);
												}
											}
										}
										if (SignalClass.OutputList.size()!=0)
										{
											//Buscamos en la lista de OUTPUTS
											for(int l=0; l<SignalClass.OutputList.size(); l++)
											{
												if(SignalClass.OutputList.at(l)==NamesSignal)		
												{	
													posSignal=l;
													l=SignalClass.OutputList.size()-1;
													VERIFICACIONES.NameSignal.push_back(NamesSignal);
												}
											}
										}
										if (SignalClass.InoutList.size()!=0)
										{
											//Buscamos en la lista de INOUTS
											for(int l=0; l<SignalClass.InoutList.size(); l++)
											{
												if(SignalClass.InoutList.at(l)==NamesSignal)		
												{	
													posSignal=l;
													l=SignalClass.InoutList.size()-1;
													VERIFICACIONES.NameSignal.push_back(NamesSignal);
												}
											}
										}
										if (SignalClass.WireList.size()!=0)
										{
											//Buscamos en la lista de WIRE
											for(int l=0; l<SignalClass.WireList.size(); l++)
											{
												if(SignalClass.WireList.at(l)==NamesSignal)		
												{	
													posSignal=l;
													l=SignalClass.WireList.size()-1;
													VERIFICACIONES.NameSignal.push_back(NamesSignal);
												}
											}
										}
										//Borramos
										GetToken(TextDownTB,TRUE);	
									}

									//Eliminamos el == que lo sigue
									if (GetToken(TextDownTB,FALSE)==TOKEN_DOSIGUAL) { GetToken(TextDownTB,TRUE);	}

									//Lo siguiente es una expresion.
									int Number=-1;
									ExprToken(TextDownTB);

									//Guardamos el numero y borramos
									GetToken(TextDownTB,TRUE);
									
									//Si encontramos logica
									if(HayLogica==1)
									{
										//Dependiendo la comparacion Verdadera o falsa guardo 1 o 0
										if(ComparacionExpresiones==true)	{	Number=1;	}
										else								{	Number=0;	}
									}
									//Sino hay logica
									else
									{
										//Si hay comparacion igual que con la logica
										if(HayComparacion==1)
										{
											if(ComparacionExpresiones==true)	{	Number=1;	}
											else								{	Number=0;	}
										}
										//Si solo es una expresion guardo el numero que me devuelve
										else
										{
											Number=NumDec;
										}
									}

									//Guardamos en la posicin de valor actual
									VERIFICACIONES.Valor.push_back(Number);
									
									////Si es un numero se lanza al analizador de expresiones
									//int Number=-1;
									//if (GetToken(TextDownTB,FALSE)==TOKEN_NUM) 
									//{
									//	ExprToken(TextDownTB);
									//	//Guardamos el numero y borramos
									//	Number=NumDec;
									//	GetToken(TextDownTB,TRUE);

									//	//Guardamos en la posicin de valor actual
									//	VERIFICACIONES.Valor.push_back(Number);
									//}
									////Sino, es una seal
									//else
									//{
									//	int findSignal=0;
									//	ExprToken(TextDownTB); //Recibimos la variable SignalNameTOKEN
									//	Number=NumDec;
									//	//Buscamos en las listas de seales
									//	for(int l=0; l<SignalClass.InputList.size(); l++)
									//	{
									//		if(SignalClass.InputList.at(l)==NameSignalTOKEN)		
									//		{	
									//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
									//			findSignal=1;
									//			//Number=SignalClass.InputValor.at(l);
									//		}
									//	}
									//	for(int l=0; l<SignalClass.OutputList.size(); l++)
									//	{
									//		if(SignalClass.OutputList.at(l)==NameSignalTOKEN)		
									//		{	
									//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
									//			findSignal=1;
									//			//Number=SignalClass.OutputValor.at(l);
									//		}
									//	}
									//	for(int l=0; l<SignalClass.InoutList.size(); l++)
									//	{
									//		if(SignalClass.InoutList.at(l)==NameSignalTOKEN)		
									//		{	
									//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
									//			findSignal=1;
									//			//Number=SignalClass.InoutValor.at(l);
									//		}
									//	}
									//	for(int l=0; l<SignalClass.WireList.size(); l++)
									//	{
									//		if(SignalClass.WireList.at(l)==NameSignalTOKEN)		
									//		{	
									//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
									//			findSignal=1;
									//			//Number=SignalClass.WireValor.at(l);
									//		}
									//	}

									//	//Si se ha encontrado la seal
									//	if(findSignal==1)
									//	{
									//		//Guardamos en la posicin de valor actual
									//		VERIFICACIONES.Valor.push_back(Number);
									//	}
									
									//Mostramos
									SimulacionOpProcess.append(SimOpProcess);
									SimOpProcess.clear();
								}
							}
							//Sino hay @, se verifica siempre
							else
							{
								SimOpProcess.append("siempre.");
								SimOpProcess.append(" CYCLE:");
								SimOpProcess.append(QString::number(CYCLE));

								/*for(int itVer=0; itVer<CICLOREPETICIONSTATE; itVer++)
								{*/
									VERIFICACIONES.Ciclo.push_back(CYCLE);
								//}

								//Borramos =>
								GetToken(TextDownTB,TRUE);

								//La primera palabra sera la del simulador test bench definida en Instance (Ej.: dut)
								//La borramos y el punto de despues
								if (GetToken(TextDownTB,FALSE) == TOKEN_WORD)		{ GetToken(TextDownTB,TRUE); }
								if (GetToken(TextDownTB,FALSE) == TOKEN_PUNTO)		{ GetToken(TextDownTB,TRUE); }

								int posSignal;

								//La siguiente palabra es una seal. La sacamos, la buscamos en la lista de SIGNALS y guardamos su valor
								if (GetToken(TextDownTB,FALSE)==TOKEN_WORD) 
								{
									int k=0;
									QString NamesSignal;
									while (TextDownTB[k].isLetter()) 
									{
										NamesSignal.push_back(TextDownTB[k]);
										k++;	
									}
									NamesSignal=NamesSignal.toUpper();

									if (SignalClass.InputList.size()!=0)
									{
										//Buscamos en la lista de INPUTS
										for(int l=0; l<SignalClass.InputList.size(); l++)
										{
											if(SignalClass.InputList.at(l)==NamesSignal)		
											{	
												posSignal=l;
												l=SignalClass.InputList.size()-1;
												//for(int itVer=0; itVer<CICLOREPETICIONSTATE; itVer++)
												//{
												VERIFICACIONES.NameSignal.push_back(NamesSignal);
												//}
											}
										}
									}
									if (SignalClass.OutputList.size()!=0)
									{
										//Buscamos en la lista de OUTPUTS
										for(int l=0; l<SignalClass.OutputList.size(); l++)
										{
											if(SignalClass.OutputList.at(l)==NamesSignal)		
											{	
												posSignal=l;
												l=SignalClass.OutputList.size()-1;
												//for(int itVer=0; itVer<CICLOREPETICIONSTATE; itVer++)
												//{
													VERIFICACIONES.NameSignal.push_back(NamesSignal);
												//}
											}
										}
									}
									if (SignalClass.InoutList.size()!=0)
									{
										//Buscamos en la lista de INOUTS
										for(int l=0; l<SignalClass.InoutList.size(); l++)
										{
											if(SignalClass.InoutList.at(l)==NamesSignal)		
											{	
												posSignal=l;
												l=SignalClass.InoutList.size()-1;
												//for(int itVer=0; itVer<CICLOREPETICIONSTATE; itVer++)
												//{
													VERIFICACIONES.NameSignal.push_back(NamesSignal);
												//}
											}
										}
									}
									if (SignalClass.WireList.size()!=0)
									{
										//Buscamos en la lista de WIRE
										for(int l=0; l<SignalClass.WireList.size(); l++)
										{
											if(SignalClass.WireList.at(l)==NamesSignal)		
											{	
												posSignal=l;
												l=SignalClass.WireList.size()-1;
												//for(int itVer=0; itVer<CICLOREPETICIONSTATE; itVer++)
												//{
													VERIFICACIONES.NameSignal.push_back(NamesSignal);
												//}
											}
										}
									}
									//Borramos
									GetToken(TextDownTB,TRUE);	
								}

								//Eliminamos el == que lo sigue
								if (GetToken(TextDownTB,FALSE)==TOKEN_DOSIGUAL) { GetToken(TextDownTB,TRUE);	}

								//Lo siguiente es una expresion.
								int Number=-1;
								ExprToken(TextDownTB);

								//Guardamos el numero y borramos
								GetToken(TextDownTB,TRUE);
								//Si encontramos logica
								if(HayLogica==1)
								{
									//Dependiendo la comparacion Verdadera o falsa guardo 1 o 0
									if(ComparacionExpresiones==true)	{	Number=1;	}
									else								{	Number=0;	}
								}
								//Sino hay logica
								else
								{
									//Si hay comparacion igual que con la logica
									if(HayComparacion==1)
									{
										if(ComparacionExpresiones==true)	{	Number=1;	}
										else								{	Number=0;	}
									}
									//Si solo es una expresion guardo el numero que me devuelve
									else
									{
										Number=NumDec;
									}
								////Lo siguiente es una expresion.
								////Si es un numero se lanza al analizador de expresiones
								//int Number=-1;
								//if (GetToken(TextDownTB,FALSE)==TOKEN_NUM) 
								//{
								//	ExprToken(TextDownTB);
								//	//Guardamos el numero y borramos
								//	Number=NumDec;
								//	GetToken(TextDownTB,TRUE);

								//	//Guardamos en la posicin de valor actual
								//	//for(int itVer=0; itVer<CICLOREPETICIONSTATE; itVer++)
								//	//{
								//		VERIFICACIONES.Valor.push_back(Number);
								//	//}
								//}
								////Sino, es una seal
								//else
								//{
								//	int findSignal=0;
								//	ExprToken(TextDownTB); //Recibimos la variable SignalNameTOKEN
								//	Number=NumDec;
								//	//Buscamos en las listas de seales
								//	for(int l=0; l<SignalClass.InputList.size(); l++)
								//	{
								//		if(SignalClass.InputList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			//Number=SignalClass.InputValor.at(l);
								//		}
								//	}
								//	for(int l=0; l<SignalClass.OutputList.size(); l++)
								//	{
								//		if(SignalClass.OutputList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			Number=SignalClass.OutputValor.at(l);
								//		}
								//	}
								//	for(int l=0; l<SignalClass.InoutList.size(); l++)
								//	{
								//		if(SignalClass.InoutList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			Number=SignalClass.InoutValor.at(l);
								//		}
								//	}
								//	for(int l=0; l<SignalClass.WireList.size(); l++)
								//	{
								//		if(SignalClass.WireList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			Number=SignalClass.WireValor.at(l);
								//		}
								//	}

								//	//Si se ha encontrado la seal
								//	if(findSignal==1)
								//	{
								//		//Guardamos en la posicin de valor actual
								//		//for(int itVer=0; itVer<CICLOREPETICIONSTATE; itVer++)
								//		//{
								//			VERIFICACIONES.Valor.push_back(Number);
								//		//}
								//	}
								}
								VERIFICACIONES.Valor.push_back(Number);
								//Mostramos
								SimulacionOpProcess.append(SimOpProcess);
								SimOpProcess.clear();
							}
						}
						//Sino hay verificacion ACTUO
						else
						{
							QString TextDownTBInit=TextDownTB;
							//Si Tenemos @ conseguimos el N Ciclo y solo actuamos si estamos en el
							if (GetToken(TextDownTB,FALSE) == TOKEN_ARROBA)
							{
								GetToken(TextDownTB,TRUE);
								int p=0;
								QString NumDecStringDOWN;
								while (TextDownTB[p].isDigit() ) 
								{	
									NumDecStringDOWN[p]=TextDownTB[p];
									p++;	
								}
								CicloActuacion=NumDecStringDOWN.toInt(&ok1,10);
								GetToken(TextDownTB, TRUE);
								
								//Si el ciclo de actuacion es igual al actual se muestra
								if (CicloActuacion == CICLOACTUAL)
								{
									//Mostramos
									SimOpProcess.append("\t+ Texto DOWN:\t");
									SimOpProcess.append(TextDownTBInit);
									SimOpProcess.append("      \t->\tActuar ");
									SimOpProcess.append("en ciclo: ");
									SimOpProcess.append(QString::number(CicloActuacion));
									SimOpProcess.append("\tCiclo Actual Interno (de 0 a ");
									SimOpProcess.append(QString::number(CICLOREPETICIONSTATE));
									SimOpProcess.append("-1): ");
									SimOpProcess.append(QString::number(CICLOACTUAL));

									//La primera palabra sera la del simulador test bench definida en Instance (Ej.: dut)
									//La borramos y el punto de despues
									if (GetToken(TextDownTB,FALSE) == TOKEN_WORD)		{ GetToken(TextDownTB,TRUE); }
									if (GetToken(TextDownTB,FALSE) == TOKEN_PUNTO)		{ GetToken(TextDownTB,TRUE); }

									int posSignal;

									//La siguiente palabra es una seal. La sacamos, la buscamos en la lista de SIGNALS y guardamos su valor
									if (GetToken(TextDownTB,FALSE)==TOKEN_WORD) 
									{
										int k=0;
										QString NamesSignal;
										while (TextDownTB[k].isLetter()) 
										{
											NamesSignal.push_back(TextDownTB[k]);
											k++;	
										}
										NamesSignal=NamesSignal.toUpper();

										if (SignalClass.InputList.size()!=0)
										{
											//Buscamos en la lista de INPUTS
											for(int l=0; l<SignalClass.InputList.size(); l++)
											{
												if(SignalClass.InputList.at(l)==NamesSignal)		
												{	
													posSignal=l;
													l=SignalClass.InputList.size()-1;
												}
											}
										}
										//if (SignalClass.OutputList.size()!=0)
										//{
										//	//Buscamos en la lista de OUTPUTS
										//	for(int l=0; l<SignalClass.OutputList.size(); l++)
										//	{
										//		if(SignalClass.OutputList.at(l)==NamesSignal)		
										//		{	
										//			posSignal=l;
										//			l=SignalClass.OutputList.size()-1;
										//		}
										//	}
										//}
										//if (SignalClass.InoutList.size()!=0)
										//{
										//	//Buscamos en la lista de INOUTS
										//	for(int l=0; l<SignalClass.InoutList.size(); l++)
										//	{
										//		if(SignalClass.InoutList.at(l)==NamesSignal)		
										//		{	
										//			posSignal=l;
										//			l=SignalClass.InoutList.size()-1;
										//		}
										//	}
										//}
										//if (SignalClass.WireList.size()!=0)
										//{
										//	//Buscamos en la lista de WIRE
										//	for(int l=0; l<SignalClass.WireList.size(); l++)
										//	{
										//		if(SignalClass.WireList.at(l)==NamesSignal)		
										//		{	
										//			posSignal=l;
										//			l=SignalClass.WireList.size()-1;
										//		}
										//	}
										//}
										//Borramos
										GetToken(TextDownTB,TRUE);	
									}

									//Eliminamos el <= que lo sigue
									if (GetToken(TextDownTB,FALSE)==TOKEN_MENORIGUAL) { GetToken(TextDownTB,TRUE);	}

									//Lo siguiente es una expresion.
									int Number=-1;
									ExprToken(TextDownTB);

									//Guardamos el numero y borramos
									GetToken(TextDownTB,TRUE);
									//Si encontramos logica
									if(HayLogica==1)
									{
										//Dependiendo la comparacion Verdadera o falsa guardo 1 o 0
										if(ComparacionExpresiones==true)	{	Number=1;	}
										else								{	Number=0;	}
									}
									//Sino hay logica
									else
									{
										//Si hay comparacion igual que con la logica
										if(HayComparacion==1)
										{
											if(ComparacionExpresiones==true)	{	Number=1;	}
											else								{	Number=0;	}
										}
										//Si solo es una expresion guardo el numero que me devuelve
										else
										{
											Number=NumDec;
										}
									}



									////Si es un numero se lanza al analizador de expresiones
									//int Number=-1;
									//if (GetToken(TextDownTB,FALSE)==TOKEN_NUM) 
									//{
									//	//SimOpProcess.append("\tMando: ");
									//	//SimOpProcess.append(TextDownTB);
									//	ExprToken(TextDownTB);
									//	//Guardamos el numero y borramos
									//	Number=NumDec;
									//	//SimOpProcess.append(", Recibo: ");
									//	//SimOpProcess.append(QString::number(Number));
									//	GetToken(TextDownTB,TRUE);

									//	//Guardamos en la posicin de valor actual
									//	SignalClass.InputValor.replace(posSignal,Number);
									//}
									////Sino, es una seal
									//else
									//{
									//	int findSignal=0;
									//	//SimOpProcess.append("\tMando: ");
									//	//SimOpProcess.append(TextDownTB);
									//	ExprToken(TextDownTB); //Recibimos la variable SignalNameTOKEN
									//	Number=NumDec;
									//	//SimOpProcess.append(", Recibo: ");
									//	//SimOpProcess.append(NameSignalTOKEN);
									//	//Buscamos en las listas de seales
									//	for(int l=0; l<SignalClass.InputList.size(); l++)
									//	{
									//		if(SignalClass.InputList.at(l)==NameSignalTOKEN)		
									//		{	
									//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
									//			findSignal=1;
									//			Number=SignalClass.InputValor.at(l);
									//		}
									//	}
									//	for(int l=0; l<SignalClass.OutputList.size(); l++)
									//	{
									//		if(SignalClass.OutputList.at(l)==NameSignalTOKEN)		
									//		{	
									//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
									//			findSignal=1;
									//			Number=SignalClass.OutputValor.at(l);
									//		}
									//	}
									//	for(int l=0; l<SignalClass.InoutList.size(); l++)
									//	{
									//		if(SignalClass.InoutList.at(l)==NameSignalTOKEN)		
									//		{	
									//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
									//			findSignal=1;
									//			Number=SignalClass.InoutValor.at(l);
									//		}
									//	}
									//	for(int l=0; l<SignalClass.WireList.size(); l++)
									//	{
									//		if(SignalClass.WireList.at(l)==NameSignalTOKEN)		
									//		{	
									//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
									//			findSignal=1;
									//			Number=SignalClass.WireValor.at(l);
									//		}
									//	}

									//	if(findSignal==1)
									//	{
									//		//SimOpProcess.append("\t\tRecibido. Cambio: ");
									//		//SimOpProcess.append(NameSignalTOKEN);
									//		//SimOpProcess.append("\tPosicion: ");
									//		//SimOpProcess.append(QString::number(posSignal));
									//		//SimOpProcess.append("\tValor: ");
									//		//SimOpProcess.append(QString::number(Number));

											//Guardamos en la posicin de valor actual
									SignalClass.InputValor.replace(posSignal,Number);
									/*	}
									}*/

									//Mostramos
									SimulacionOpProcess.append(SimOpProcess);
									SimOpProcess.clear();
								}
							}
							//Sino actuamos siempre
							else
							{
								SimOpProcess.append("\t+ Texto DOWN:\t");
								SimOpProcess.append(TextDownTB);
								SimOpProcess.append("      \t->\tActuar ");
								SimOpProcess.append("siempre");

								//La primera palabra sera la del simulador test bench definida en Instance (Ej.: dut)
								//La borramos y el punto de despues
								if (GetToken(TextDownTB,FALSE) == TOKEN_WORD)		{ GetToken(TextDownTB,TRUE); }
								if (GetToken(TextDownTB,FALSE) == TOKEN_PUNTO)		{ GetToken(TextDownTB,TRUE); }

								int posSignal;

								//La siguiente palabra es una seal. La sacamos, la buscamos en la lista de SIGNALS y guardamos su valor
								if (GetToken(TextDownTB,FALSE)==TOKEN_WORD) 
								{
									int k=0;
									QString NamesSignal;
									while (TextDownTB[k].isLetter()) 
									{
										NamesSignal.push_back(TextDownTB[k]);
										k++;	
									}
									NamesSignal=NamesSignal.toUpper();

									if (SignalClass.InputList.size()!=0)
									{
										//Buscamos en la lista de INPUTS
										for(int l=0; l<SignalClass.InputList.size(); l++)
										{
											if(SignalClass.InputList.at(l)==NamesSignal)		
											{	
												posSignal=l;
												l=SignalClass.InputList.size()-1;
											}
										}
									}
									//Borramos
									GetToken(TextDownTB,TRUE);	
								}

								//Eliminamos el <= que lo sigue
								if (GetToken(TextDownTB,FALSE)==TOKEN_MENORIGUAL) { GetToken(TextDownTB,TRUE);	}

								//Lo siguiente es una expresion.
								int Number=-1;
								ExprToken(TextDownTB);

								//Guardamos el numero y borramos
								GetToken(TextDownTB,TRUE);
								//Si encontramos logica
								if(HayLogica==1)
								{
									//Dependiendo la comparacion Verdadera o falsa guardo 1 o 0
									if(ComparacionExpresiones==true)	{	Number=1;	}
									else								{	Number=0;	}
								}
								//Sino hay logica
								else
								{
									//Si hay comparacion igual que con la logica
									if(HayComparacion==1)
									{
										if(ComparacionExpresiones==true)	{	Number=1;	}
										else								{	Number=0;	}
									}
									//Si solo es una expresion guardo el numero que me devuelve
									else
									{
										Number=NumDec;
									}
								}



								////Lo siguiente es una expresion.
								////Si es un numero se lanza al analizador de expresiones
								//int Number=-1;
								//if (GetToken(TextDownTB,FALSE)==TOKEN_NUM) 
								//{
								//	//SimOpProcess.append("\tMando: ");
								//	//SimOpProcess.append(TextDownTB);
								//	ExprToken(TextDownTB);
								//	//Guardamos el numero y borramos
								//	Number=NumDec;
								//	//SimOpProcess.append(", Recibo: ");
								//	//SimOpProcess.append(QString::number(Number));
								//	GetToken(TextDownTB,TRUE);

								//	//Guardamos en la posicin de valor actual
								//	SignalClass.InputValor.replace(posSignal,Number);
								//}
								////Sino, es una seal
								//else
								//{
								//	int findSignal=0;
								//	//SimOpProcess.append("\tMando: ");
								//	//SimOpProcess.append(TextDownTB);
								//	ExprToken(TextDownTB); //Recibimos la variable SignalNameTOKEN
								//	Number=NumDec;
								//	//SimOpProcess.append(", Recibo: ");
								//	//SimOpProcess.append(NameSignalTOKEN);
								//	//Buscamos en las listas de seales
								//	for(int l=0; l<SignalClass.InputList.size(); l++)
								//	{
								//		if(SignalClass.InputList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			Number=SignalClass.InputValor.at(l);
								//		}
								//	}
								//	for(int l=0; l<SignalClass.OutputList.size(); l++)
								//	{
								//		if(SignalClass.OutputList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			Number=SignalClass.OutputValor.at(l);
								//		}
								//	}
								//	for(int l=0; l<SignalClass.InoutList.size(); l++)
								//	{
								//		if(SignalClass.InoutList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			Number=SignalClass.InoutValor.at(l);
								//		}
								//	}
								//	for(int l=0; l<SignalClass.WireList.size(); l++)
								//	{
								//		if(SignalClass.WireList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			Number=SignalClass.WireValor.at(l);
								//		}
								//	}

								//	if(findSignal==1)
								//	{

										//Guardamos en la posicin de valor actual
								SignalClass.InputValor.replace(posSignal,Number);
								/*	}
								}*/

								//Mostramos
								SimulacionOpProcess.append(SimOpProcess);
								SimOpProcess.clear();
							}
						}

						//Guardo las seales de este ciclo aqui por ser ASYNC
//						GuardarSimulacion(CYCLE);
					}
				}
				//Desactivo la variable de primera pasada, que es cuando guardo el numero de ciclos de este state
				PRIMERAPASADA=0;

			}
			if(TYPEPadre=="StateAsyncOps")
			{
				//Leemos lo que hay en textUp para saber el numero de ciclos
				TextTBUpPadre=BOX.TEXTUPMatrix[0][ITInitTestBench];		//TextUp del padre
				SimOpProcess.append("\tTextUp: ");
				SimOpProcess.append(TextTBUpPadre);
				SimOpProcess.append("\n");
				
				//Si es la primera vez que entro en este STATE tengo que sacar su tiempo de ciclo activo
				if (PRIMERAPASADA==1)
				{
					CICLOACTUAL=0;
					if (TextTBUpPadre.isNull())
					{
						CICLOREPETICIONSTATE=1;
					}
					else
					{
						for(int m=0; m<40; m++)
						{
							//NOS ENCONTRAMOS UN NOMBRE CON O SIN DIGITOS SEGUIDO DE <NUM>. Necesitamos NUM
							if(GetToken(TextTBUpPadre,FALSE)==TOKEN_MENOR)
							{
								//Se borra el menor <
								GetToken(TextTBUpPadre,TRUE);
								/*SimOpProcess.append(TextTBUpPadre);
								SimOpProcess.append("\n");*/
								//Se capta el numero de ciclos
								int l=0;
								QString NumDecString;
								while (TextTBUpPadre[l].isDigit() ) 
								{	
									NumDecString[l]=TextTBUpPadre[l];
									l++;	
								}
								//Guardamos el numero y borramos
								CICLOREPETICIONSTATE=NumDecString.toInt(&ok,10);
								GetToken(TextTBUpPadre, TRUE);
							}
							else
							{
								GetToken(TextTBUpPadre,TRUE);
								/*SimOpProcess.append(TextTBUpPadre);
								SimOpProcess.append("\n");*/
							}
							if (GetToken(TextTBUpPadre,FALSE)==TOKEN_MAYOR)
							{
								m=10;
							}
						}
					}
				}

				//Interpreto lo que me encuentro y actuo. Puede haber:
				//
				//		-	=>		:	Verificar lo que haya
				//		-	=>  @N	:	Verificar en el ciclo indicado
				//		-			:	Actuar
				//		-		@N	:	Actuar en el ciclo indicado
				for (int n=0; n<filasMatricesTexto; n++)
				{
					QString TextDownTB=BOX.TEXTDOWNMatrix[n][ITInitTestBench];
					int CicloVerificacion=0, CicloActuacion=0;
					bool ok1;
					
					//Comprobamos si el texto es null
					if (TextDownTB=="null")
					{
						n=filasMatricesTexto-1;
					}
					//Si hay texto
					else
					{
						//Si tenemos una verificacion solo lo hacemos en la primera pasada y guardamos en la matriz de verificaciones
						if (GetToken(TextDownTB,FALSE) == TOKEN_VERIFICACION)
						{
							//Mostramos el texto
							SimOpProcess.append("\t+ Texto DOWN:\t");
							SimOpProcess.append(TextDownTB);

							//Borramos la verificacion =>
							GetToken(TextDownTB,TRUE);

							SimOpProcess.append("    \t->\tVerificacion ");
			
							//Si tenemos @
							if (GetToken(TextDownTB,FALSE) == TOKEN_ARROBA)
							{
								if (PRIMERAPASADA==1)
								{
									//Borramos y conseguimos el N de ciclo de la verificacion
									GetToken(TextDownTB,TRUE);
									int p=0;
									QString NumDecStringDOWN;
									while (TextDownTB[p].isDigit() ) 
									{	
										NumDecStringDOWN[p]=TextDownTB[p];
										p++;	
									}
									CicloVerificacion=NumDecStringDOWN.toInt(&ok1,10);
									GetToken(TextDownTB, TRUE);
									SimOpProcess.append("en ciclo: ");
									SimOpProcess.append(QString::number(CicloVerificacion));
									SimOpProcess.append(".      CYCLE:");
									SimOpProcess.append(QString::number(CYCLE));
									SimOpProcess.append(".      Suma:");
									SimOpProcess.append(QString::number(CYCLE + CicloVerificacion - 1));

									VERIFICACIONES.Ciclo.push_back((CYCLE + CicloVerificacion - 1));

									//La primera palabra sera la del simulador test bench definida en Instance (Ej.: dut)
									//La borramos y el punto de despues
									if (GetToken(TextDownTB,FALSE) == TOKEN_WORD)		{ GetToken(TextDownTB,TRUE); }
									if (GetToken(TextDownTB,FALSE) == TOKEN_PUNTO)		{ GetToken(TextDownTB,TRUE); }

									int posSignal;

									//La siguiente palabra es una seal. La sacamos, la buscamos en la lista de SIGNALS y guardamos su valor
									if (GetToken(TextDownTB,FALSE)==TOKEN_WORD) 
									{
										int k=0;
										QString NamesSignal;
										while (TextDownTB[k].isLetter()) 
										{
											NamesSignal.push_back(TextDownTB[k]);
											k++;	
										}
										NamesSignal=NamesSignal.toUpper();

										if (SignalClass.InputList.size()!=0)
										{
											//Buscamos en la lista de INPUTS
											for(int l=0; l<SignalClass.InputList.size(); l++)
											{
												if(SignalClass.InputList.at(l)==NamesSignal)		
												{	
													posSignal=l;
													l=SignalClass.InputList.size()-1;
													VERIFICACIONES.NameSignal.push_back(NamesSignal);
												}
											}
										}
										if (SignalClass.OutputList.size()!=0)
										{
											//Buscamos en la lista de OUTPUTS
											for(int l=0; l<SignalClass.OutputList.size(); l++)
											{
												if(SignalClass.OutputList.at(l)==NamesSignal)		
												{	
													posSignal=l;
													l=SignalClass.OutputList.size()-1;
													VERIFICACIONES.NameSignal.push_back(NamesSignal);
												}
											}
										}
										if (SignalClass.InoutList.size()!=0)
										{
											//Buscamos en la lista de INOUTS
											for(int l=0; l<SignalClass.InoutList.size(); l++)
											{
												if(SignalClass.InoutList.at(l)==NamesSignal)		
												{	
													posSignal=l;
													l=SignalClass.InoutList.size()-1;
													VERIFICACIONES.NameSignal.push_back(NamesSignal);
												}
											}
										}
										if (SignalClass.WireList.size()!=0)
										{
											//Buscamos en la lista de WIRE
											for(int l=0; l<SignalClass.WireList.size(); l++)
											{
												if(SignalClass.WireList.at(l)==NamesSignal)		
												{	
													posSignal=l;
													l=SignalClass.WireList.size()-1;
													VERIFICACIONES.NameSignal.push_back(NamesSignal);
												}
											}
										}
										//Borramos
										GetToken(TextDownTB,TRUE);	
									}

									//Eliminamos el == que lo sigue
									if (GetToken(TextDownTB,FALSE)==TOKEN_DOSIGUAL) { GetToken(TextDownTB,TRUE);	}

									//Lo siguiente es una expresion.
									int Number=-1;
									ExprToken(TextDownTB);

									//Guardamos el numero y borramos
									GetToken(TextDownTB,TRUE);
									
									//Si encontramos logica
									if(HayLogica==1)
									{
										//Dependiendo la comparacion Verdadera o falsa guardo 1 o 0
										if(ComparacionExpresiones==true)	{	Number=1;	}
										else								{	Number=0;	}
									}
									//Sino hay logica
									else
									{
										//Si hay comparacion igual que con la logica
										if(HayComparacion==1)
										{
											if(ComparacionExpresiones==true)	{	Number=1;	}
											else								{	Number=0;	}
										}
										//Si solo es una expresion guardo el numero que me devuelve
										else
										{
											Number=NumDec;
										}
									}

									//Guardamos en la posicin de valor actual
									VERIFICACIONES.Valor.push_back(Number);
									
									////Si es un numero se lanza al analizador de expresiones
									//int Number=-1;
									//if (GetToken(TextDownTB,FALSE)==TOKEN_NUM) 
									//{
									//	ExprToken(TextDownTB);
									//	//Guardamos el numero y borramos
									//	Number=NumDec;
									//	GetToken(TextDownTB,TRUE);

									//	//Guardamos en la posicin de valor actual
									//	VERIFICACIONES.Valor.push_back(Number);
									//}
									////Sino, es una seal
									//else
									//{
									//	int findSignal=0;
									//	ExprToken(TextDownTB); //Recibimos la variable SignalNameTOKEN
									//	Number=NumDec;
									//	//Buscamos en las listas de seales
									//	for(int l=0; l<SignalClass.InputList.size(); l++)
									//	{
									//		if(SignalClass.InputList.at(l)==NameSignalTOKEN)		
									//		{	
									//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
									//			findSignal=1;
									//			//Number=SignalClass.InputValor.at(l);
									//		}
									//	}
									//	for(int l=0; l<SignalClass.OutputList.size(); l++)
									//	{
									//		if(SignalClass.OutputList.at(l)==NameSignalTOKEN)		
									//		{	
									//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
									//			findSignal=1;
									//			//Number=SignalClass.OutputValor.at(l);
									//		}
									//	}
									//	for(int l=0; l<SignalClass.InoutList.size(); l++)
									//	{
									//		if(SignalClass.InoutList.at(l)==NameSignalTOKEN)		
									//		{	
									//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
									//			findSignal=1;
									//			//Number=SignalClass.InoutValor.at(l);
									//		}
									//	}
									//	for(int l=0; l<SignalClass.WireList.size(); l++)
									//	{
									//		if(SignalClass.WireList.at(l)==NameSignalTOKEN)		
									//		{	
									//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
									//			findSignal=1;
									//			//Number=SignalClass.WireValor.at(l);
									//		}
									//	}

									//	//Si se ha encontrado la seal
									//	if(findSignal==1)
									//	{
									//		//Guardamos en la posicin de valor actual
									//		VERIFICACIONES.Valor.push_back(Number);
									//	}
									
									//Mostramos
									SimulacionOpProcess.append(SimOpProcess);
									SimOpProcess.clear();
								}
							}
							//Sino hay @, se verifica siempre
							else
							{
								SimOpProcess.append("siempre.");
								SimOpProcess.append(" CYCLE:");
								SimOpProcess.append(QString::number(CYCLE));

								/*for(int itVer=0; itVer<CICLOREPETICIONSTATE; itVer++)
								{*/
									VERIFICACIONES.Ciclo.push_back(CYCLE);
								//}

								//Borramos =>
								GetToken(TextDownTB,TRUE);

								//La primera palabra sera la del simulador test bench definida en Instance (Ej.: dut)
								//La borramos y el punto de despues
								if (GetToken(TextDownTB,FALSE) == TOKEN_WORD)		{ GetToken(TextDownTB,TRUE); }
								if (GetToken(TextDownTB,FALSE) == TOKEN_PUNTO)		{ GetToken(TextDownTB,TRUE); }

								int posSignal;

								//La siguiente palabra es una seal. La sacamos, la buscamos en la lista de SIGNALS y guardamos su valor
								if (GetToken(TextDownTB,FALSE)==TOKEN_WORD) 
								{
									int k=0;
									QString NamesSignal;
									while (TextDownTB[k].isLetter()) 
									{
										NamesSignal.push_back(TextDownTB[k]);
										k++;	
									}
									NamesSignal=NamesSignal.toUpper();

									if (SignalClass.InputList.size()!=0)
									{
										//Buscamos en la lista de INPUTS
										for(int l=0; l<SignalClass.InputList.size(); l++)
										{
											if(SignalClass.InputList.at(l)==NamesSignal)		
											{	
												posSignal=l;
												l=SignalClass.InputList.size()-1;
												//for(int itVer=0; itVer<CICLOREPETICIONSTATE; itVer++)
												//{
												VERIFICACIONES.NameSignal.push_back(NamesSignal);
												//}
											}
										}
									}
									if (SignalClass.OutputList.size()!=0)
									{
										//Buscamos en la lista de OUTPUTS
										for(int l=0; l<SignalClass.OutputList.size(); l++)
										{
											if(SignalClass.OutputList.at(l)==NamesSignal)		
											{	
												posSignal=l;
												l=SignalClass.OutputList.size()-1;
												//for(int itVer=0; itVer<CICLOREPETICIONSTATE; itVer++)
												//{
													VERIFICACIONES.NameSignal.push_back(NamesSignal);
												//}
											}
										}
									}
									if (SignalClass.InoutList.size()!=0)
									{
										//Buscamos en la lista de INOUTS
										for(int l=0; l<SignalClass.InoutList.size(); l++)
										{
											if(SignalClass.InoutList.at(l)==NamesSignal)		
											{	
												posSignal=l;
												l=SignalClass.InoutList.size()-1;
												//for(int itVer=0; itVer<CICLOREPETICIONSTATE; itVer++)
												//{
													VERIFICACIONES.NameSignal.push_back(NamesSignal);
												//}
											}
										}
									}
									if (SignalClass.WireList.size()!=0)
									{
										//Buscamos en la lista de WIRE
										for(int l=0; l<SignalClass.WireList.size(); l++)
										{
											if(SignalClass.WireList.at(l)==NamesSignal)		
											{	
												posSignal=l;
												l=SignalClass.WireList.size()-1;
												//for(int itVer=0; itVer<CICLOREPETICIONSTATE; itVer++)
												//{
													VERIFICACIONES.NameSignal.push_back(NamesSignal);
												//}
											}
										}
									}
									//Borramos
									GetToken(TextDownTB,TRUE);	
								}

								//Eliminamos el == que lo sigue
								if (GetToken(TextDownTB,FALSE)==TOKEN_DOSIGUAL) { GetToken(TextDownTB,TRUE);	}

								//Lo siguiente es una expresion.
								int Number=-1;
								ExprToken(TextDownTB);

								//Guardamos el numero y borramos
								GetToken(TextDownTB,TRUE);
								//Si encontramos logica
								if(HayLogica==1)
								{
									//Dependiendo la comparacion Verdadera o falsa guardo 1 o 0
									if(ComparacionExpresiones==true)	{	Number=1;	}
									else								{	Number=0;	}
								}
								//Sino hay logica
								else
								{
									//Si hay comparacion igual que con la logica
									if(HayComparacion==1)
									{
										if(ComparacionExpresiones==true)	{	Number=1;	}
										else								{	Number=0;	}
									}
									//Si solo es una expresion guardo el numero que me devuelve
									else
									{
										Number=NumDec;
									}
								////Lo siguiente es una expresion.
								////Si es un numero se lanza al analizador de expresiones
								//int Number=-1;
								//if (GetToken(TextDownTB,FALSE)==TOKEN_NUM) 
								//{
								//	ExprToken(TextDownTB);
								//	//Guardamos el numero y borramos
								//	Number=NumDec;
								//	GetToken(TextDownTB,TRUE);

								//	//Guardamos en la posicin de valor actual
								//	//for(int itVer=0; itVer<CICLOREPETICIONSTATE; itVer++)
								//	//{
								//		VERIFICACIONES.Valor.push_back(Number);
								//	//}
								//}
								////Sino, es una seal
								//else
								//{
								//	int findSignal=0;
								//	ExprToken(TextDownTB); //Recibimos la variable SignalNameTOKEN
								//	Number=NumDec;
								//	//Buscamos en las listas de seales
								//	for(int l=0; l<SignalClass.InputList.size(); l++)
								//	{
								//		if(SignalClass.InputList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			//Number=SignalClass.InputValor.at(l);
								//		}
								//	}
								//	for(int l=0; l<SignalClass.OutputList.size(); l++)
								//	{
								//		if(SignalClass.OutputList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			Number=SignalClass.OutputValor.at(l);
								//		}
								//	}
								//	for(int l=0; l<SignalClass.InoutList.size(); l++)
								//	{
								//		if(SignalClass.InoutList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			Number=SignalClass.InoutValor.at(l);
								//		}
								//	}
								//	for(int l=0; l<SignalClass.WireList.size(); l++)
								//	{
								//		if(SignalClass.WireList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			Number=SignalClass.WireValor.at(l);
								//		}
								//	}

								//	//Si se ha encontrado la seal
								//	if(findSignal==1)
								//	{
								//		//Guardamos en la posicin de valor actual
								//		//for(int itVer=0; itVer<CICLOREPETICIONSTATE; itVer++)
								//		//{
								//			VERIFICACIONES.Valor.push_back(Number);
								//		//}
								//	}
								}
								VERIFICACIONES.Valor.push_back(Number);
								//Mostramos
								SimulacionOpProcess.append(SimOpProcess);
								SimOpProcess.clear();
							}
						}
						//Sino hay verificacion ACTUO
						else
						{
							QString TextDownTBInit=TextDownTB;
							//Si Tenemos @ conseguimos el N Ciclo y solo actuamos si estamos en el
							if (GetToken(TextDownTB,FALSE) == TOKEN_ARROBA)
							{
								GetToken(TextDownTB,TRUE);
								int p=0;
								QString NumDecStringDOWN;
								while (TextDownTB[p].isDigit() ) 
								{	
									NumDecStringDOWN[p]=TextDownTB[p];
									p++;	
								}
								CicloActuacion=NumDecStringDOWN.toInt(&ok1,10);
								GetToken(TextDownTB, TRUE);
								
								//Si el ciclo de actuacion es igual al actual se muestra
								if (CicloActuacion == CICLOACTUAL)
								{
									//Mostramos
									SimOpProcess.append("\t+ Texto DOWN:\t");
									SimOpProcess.append(TextDownTBInit);
									SimOpProcess.append("      \t->\tActuar ");
									SimOpProcess.append("en ciclo: ");
									SimOpProcess.append(QString::number(CicloActuacion));
									SimOpProcess.append("\tCiclo Actual Interno (de 0 a ");
									SimOpProcess.append(QString::number(CICLOREPETICIONSTATE));
									SimOpProcess.append("-1): ");
									SimOpProcess.append(QString::number(CICLOACTUAL));

									//La primera palabra sera la del simulador test bench definida en Instance (Ej.: dut)
									//La borramos y el punto de despues
									if (GetToken(TextDownTB,FALSE) == TOKEN_WORD)		{ GetToken(TextDownTB,TRUE); }
									if (GetToken(TextDownTB,FALSE) == TOKEN_PUNTO)		{ GetToken(TextDownTB,TRUE); }

									int posSignal;

									//La siguiente palabra es una seal. La sacamos, la buscamos en la lista de SIGNALS y guardamos su valor
									if (GetToken(TextDownTB,FALSE)==TOKEN_WORD) 
									{
										int k=0;
										QString NamesSignal;
										while (TextDownTB[k].isLetter()) 
										{
											NamesSignal.push_back(TextDownTB[k]);
											k++;	
										}
										NamesSignal=NamesSignal.toUpper();

										if (SignalClass.InputList.size()!=0)
										{
											//Buscamos en la lista de INPUTS
											for(int l=0; l<SignalClass.InputList.size(); l++)
											{
												if(SignalClass.InputList.at(l)==NamesSignal)		
												{	
													posSignal=l;
													l=SignalClass.InputList.size()-1;
												}
											}
										}
										//if (SignalClass.OutputList.size()!=0)
										//{
										//	//Buscamos en la lista de OUTPUTS
										//	for(int l=0; l<SignalClass.OutputList.size(); l++)
										//	{
										//		if(SignalClass.OutputList.at(l)==NamesSignal)		
										//		{	
										//			posSignal=l;
										//			l=SignalClass.OutputList.size()-1;
										//		}
										//	}
										//}
										//if (SignalClass.InoutList.size()!=0)
										//{
										//	//Buscamos en la lista de INOUTS
										//	for(int l=0; l<SignalClass.InoutList.size(); l++)
										//	{
										//		if(SignalClass.InoutList.at(l)==NamesSignal)		
										//		{	
										//			posSignal=l;
										//			l=SignalClass.InoutList.size()-1;
										//		}
										//	}
										//}
										//if (SignalClass.WireList.size()!=0)
										//{
										//	//Buscamos en la lista de WIRE
										//	for(int l=0; l<SignalClass.WireList.size(); l++)
										//	{
										//		if(SignalClass.WireList.at(l)==NamesSignal)		
										//		{	
										//			posSignal=l;
										//			l=SignalClass.WireList.size()-1;
										//		}
										//	}
										//}
										//Borramos
										GetToken(TextDownTB,TRUE);	
									}

									//Eliminamos el <= que lo sigue
									if (GetToken(TextDownTB,FALSE)==TOKEN_MENORIGUAL) { GetToken(TextDownTB,TRUE);	}

									//Lo siguiente es una expresion.
									int Number=-1;
									ExprToken(TextDownTB);

									//Guardamos el numero y borramos
									GetToken(TextDownTB,TRUE);
									//Si encontramos logica
									if(HayLogica==1)
									{
										//Dependiendo la comparacion Verdadera o falsa guardo 1 o 0
										if(ComparacionExpresiones==true)	{	Number=1;	}
										else								{	Number=0;	}
									}
									//Sino hay logica
									else
									{
										//Si hay comparacion igual que con la logica
										if(HayComparacion==1)
										{
											if(ComparacionExpresiones==true)	{	Number=1;	}
											else								{	Number=0;	}
										}
										//Si solo es una expresion guardo el numero que me devuelve
										else
										{
											Number=NumDec;
										}
									}



									////Si es un numero se lanza al analizador de expresiones
									//int Number=-1;
									//if (GetToken(TextDownTB,FALSE)==TOKEN_NUM) 
									//{
									//	//SimOpProcess.append("\tMando: ");
									//	//SimOpProcess.append(TextDownTB);
									//	ExprToken(TextDownTB);
									//	//Guardamos el numero y borramos
									//	Number=NumDec;
									//	//SimOpProcess.append(", Recibo: ");
									//	//SimOpProcess.append(QString::number(Number));
									//	GetToken(TextDownTB,TRUE);

									//	//Guardamos en la posicin de valor actual
									//	SignalClass.InputValor.replace(posSignal,Number);
									//}
									////Sino, es una seal
									//else
									//{
									//	int findSignal=0;
									//	//SimOpProcess.append("\tMando: ");
									//	//SimOpProcess.append(TextDownTB);
									//	ExprToken(TextDownTB); //Recibimos la variable SignalNameTOKEN
									//	Number=NumDec;
									//	//SimOpProcess.append(", Recibo: ");
									//	//SimOpProcess.append(NameSignalTOKEN);
									//	//Buscamos en las listas de seales
									//	for(int l=0; l<SignalClass.InputList.size(); l++)
									//	{
									//		if(SignalClass.InputList.at(l)==NameSignalTOKEN)		
									//		{	
									//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
									//			findSignal=1;
									//			Number=SignalClass.InputValor.at(l);
									//		}
									//	}
									//	for(int l=0; l<SignalClass.OutputList.size(); l++)
									//	{
									//		if(SignalClass.OutputList.at(l)==NameSignalTOKEN)		
									//		{	
									//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
									//			findSignal=1;
									//			Number=SignalClass.OutputValor.at(l);
									//		}
									//	}
									//	for(int l=0; l<SignalClass.InoutList.size(); l++)
									//	{
									//		if(SignalClass.InoutList.at(l)==NameSignalTOKEN)		
									//		{	
									//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
									//			findSignal=1;
									//			Number=SignalClass.InoutValor.at(l);
									//		}
									//	}
									//	for(int l=0; l<SignalClass.WireList.size(); l++)
									//	{
									//		if(SignalClass.WireList.at(l)==NameSignalTOKEN)		
									//		{	
									//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
									//			findSignal=1;
									//			Number=SignalClass.WireValor.at(l);
									//		}
									//	}

									//	if(findSignal==1)
									//	{
									//		//SimOpProcess.append("\t\tRecibido. Cambio: ");
									//		//SimOpProcess.append(NameSignalTOKEN);
									//		//SimOpProcess.append("\tPosicion: ");
									//		//SimOpProcess.append(QString::number(posSignal));
									//		//SimOpProcess.append("\tValor: ");
									//		//SimOpProcess.append(QString::number(Number));

											//Guardamos en la posicin de valor actual
									SignalClass.InputValor.replace(posSignal,Number);
									/*	}
									}*/

									//Mostramos
									SimulacionOpProcess.append(SimOpProcess);
									SimOpProcess.clear();
								}
							}
							//Sino actuamos siempre
							else
							{
								SimOpProcess.append("\t+ Texto DOWN:\t");
								SimOpProcess.append(TextDownTB);
								SimOpProcess.append("      \t->\tActuar ");
								SimOpProcess.append("siempre");

								//La primera palabra sera la del simulador test bench definida en Instance (Ej.: dut)
								//La borramos y el punto de despues
								if (GetToken(TextDownTB,FALSE) == TOKEN_WORD)		{ GetToken(TextDownTB,TRUE); }
								if (GetToken(TextDownTB,FALSE) == TOKEN_PUNTO)		{ GetToken(TextDownTB,TRUE); }

								int posSignal;

								//La siguiente palabra es una seal. La sacamos, la buscamos en la lista de SIGNALS y guardamos su valor
								if (GetToken(TextDownTB,FALSE)==TOKEN_WORD) 
								{
									int k=0;
									QString NamesSignal;
									while (TextDownTB[k].isLetter()) 
									{
										NamesSignal.push_back(TextDownTB[k]);
										k++;	
									}
									NamesSignal=NamesSignal.toUpper();

									if (SignalClass.InputList.size()!=0)
									{
										//Buscamos en la lista de INPUTS
										for(int l=0; l<SignalClass.InputList.size(); l++)
										{
											if(SignalClass.InputList.at(l)==NamesSignal)		
											{	
												posSignal=l;
												l=SignalClass.InputList.size()-1;
											}
										}
									}
									//Borramos
									GetToken(TextDownTB,TRUE);	
								}

								//Eliminamos el <= que lo sigue
								if (GetToken(TextDownTB,FALSE)==TOKEN_MENORIGUAL) { GetToken(TextDownTB,TRUE);	}

								//Lo siguiente es una expresion.
								int Number=-1;
								ExprToken(TextDownTB);

								//Guardamos el numero y borramos
								GetToken(TextDownTB,TRUE);
								//Si encontramos logica
								if(HayLogica==1)
								{
									//Dependiendo la comparacion Verdadera o falsa guardo 1 o 0
									if(ComparacionExpresiones==true)	{	Number=1;	}
									else								{	Number=0;	}
								}
								//Sino hay logica
								else
								{
									//Si hay comparacion igual que con la logica
									if(HayComparacion==1)
									{
										if(ComparacionExpresiones==true)	{	Number=1;	}
										else								{	Number=0;	}
									}
									//Si solo es una expresion guardo el numero que me devuelve
									else
									{
										Number=NumDec;
									}
								}



								////Lo siguiente es una expresion.
								////Si es un numero se lanza al analizador de expresiones
								//int Number=-1;
								//if (GetToken(TextDownTB,FALSE)==TOKEN_NUM) 
								//{
								//	//SimOpProcess.append("\tMando: ");
								//	//SimOpProcess.append(TextDownTB);
								//	ExprToken(TextDownTB);
								//	//Guardamos el numero y borramos
								//	Number=NumDec;
								//	//SimOpProcess.append(", Recibo: ");
								//	//SimOpProcess.append(QString::number(Number));
								//	GetToken(TextDownTB,TRUE);

								//	//Guardamos en la posicin de valor actual
								//	SignalClass.InputValor.replace(posSignal,Number);
								//}
								////Sino, es una seal
								//else
								//{
								//	int findSignal=0;
								//	//SimOpProcess.append("\tMando: ");
								//	//SimOpProcess.append(TextDownTB);
								//	ExprToken(TextDownTB); //Recibimos la variable SignalNameTOKEN
								//	Number=NumDec;
								//	//SimOpProcess.append(", Recibo: ");
								//	//SimOpProcess.append(NameSignalTOKEN);
								//	//Buscamos en las listas de seales
								//	for(int l=0; l<SignalClass.InputList.size(); l++)
								//	{
								//		if(SignalClass.InputList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			Number=SignalClass.InputValor.at(l);
								//		}
								//	}
								//	for(int l=0; l<SignalClass.OutputList.size(); l++)
								//	{
								//		if(SignalClass.OutputList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			Number=SignalClass.OutputValor.at(l);
								//		}
								//	}
								//	for(int l=0; l<SignalClass.InoutList.size(); l++)
								//	{
								//		if(SignalClass.InoutList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			Number=SignalClass.InoutValor.at(l);
								//		}
								//	}
								//	for(int l=0; l<SignalClass.WireList.size(); l++)
								//	{
								//		if(SignalClass.WireList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			Number=SignalClass.WireValor.at(l);
								//		}
								//	}

								//	if(findSignal==1)
								//	{

										//Guardamos en la posicin de valor actual
								SignalClass.InputValor.replace(posSignal,Number);
								/*	}
								}*/

								//Mostramos
								SimulacionOpProcess.append(SimOpProcess);
								SimOpProcess.clear();
							}
						}

						//Guardo las seales de este ciclo aqui por ser ASYNC
//						GuardarSimulacion(CYCLE);
					}
				}
				//Desactivo la variable de primera pasada, que es cuando guardo el numero de ciclos de este state
				PRIMERAPASADA=0;
			}
			if(TYPEPadre=="State")
			{
				//Leemos lo que hay en textUp para saber el numero de ciclos
				TextTBPadre=BOX.TEXTMatrix[0][ITInitTestBench];		//TextUp del padre
				SimOpProcess.append("\tText: ");
				SimOpProcess.append(TextTBPadre);
				SimOpProcess.append("\n");
				
				//Si es la primera vez que entro en este STATE tengo que sacar su tiempo de ciclo activo
				if (PRIMERAPASADA==1)
				{
					CICLOACTUAL=0;
					if (TextTBPadre.isNull())
					{
						CICLOREPETICIONSTATE=1;
					}
					else
					{
						for(int m=0; m<40; m++)
						{
							//NOS ENCONTRAMOS UN NOMBRE CON O SIN DIGITOS SEGUIDO DE <NUM>. Necesitamos NUM
							if(GetToken(TextTBPadre,FALSE)==TOKEN_MENOR)
							{
								//Se borra el menor <
								GetToken(TextTBPadre,TRUE);
								/*SimOpProcess.append(TextTBUpPadre);
								SimOpProcess.append("\n");*/
								//Se capta el numero de ciclos
								int l=0;
								QString NumDecString;
								while (TextTBPadre[l].isDigit() ) 
								{	
									NumDecString[l]=TextTBPadre[l];
									l++;	
								}
								//Guardamos el numero y borramos
								CICLOREPETICIONSTATE=NumDecString.toInt(&ok,10);
								GetToken(TextTBPadre, TRUE);
							}
							else
							{
								GetToken(TextTBPadre,TRUE);
								/*SimOpProcess.append(TextTBUpPadre);
								SimOpProcess.append("\n");*/
							}
							if (GetToken(TextTBPadre,FALSE)==TOKEN_MAYOR)
							{
								m=10;
							}
						}
					}
					//Desactivo la variable de primera pasada, que es cuando guardo el numero de ciclos de este state
					PRIMERAPASADA=0;
				}
			}
			if(TYPEPadre=="MetaState")
			{
				//Leemos lo que hay en textUp para saber el numero de ciclos
				TextTBPadre=BOX.TEXTMatrix[0][ITInitTestBench];		//TextUp del padre
				SimOpProcess.append("\tText: ");
				SimOpProcess.append(TextTBPadre);
				SimOpProcess.append("\n");
				CICLOACTUAL=1;
				if(BOX.TEXTMatrix[0][ITInitTestBench]=="end simulation")
				{
					//Activamos la seal de fin de simulacion
					FINAL=1;
				}
			}

			if (FINAL!=1)
			{
				//Una vez analizado todos los casos aumentamos el ciclo. Si coincide con el maximo no tocamos STATESTEST. Sino disminuimos 1
				CICLOACTUAL=CICLOACTUAL+1;
				SimOpProcess.append("\n\t\tCICLO DE STATE: ");
				SimOpProcess.append(QString::number(CICLOACTUAL));
				SimOpProcess.append(" DE ");
				SimOpProcess.append(QString::number(CICLOREPETICIONSTATE));
				SimOpProcess.append(" TOTALES\n");
			}
			//Si el ciclo actual es menor que el ciclo de veces que tiene que repetirse un state no se modifica la variable
			if (CICLOACTUAL<CICLOREPETICIONSTATE)
			{
				STATESTEST=STATESTEST;
			}
			//Si es igual o mayor aumento
			else
			{
				STATESTEST=STATESTEST+1;

				//Pongo la variable de primera pasada activa para el proximo state
				PRIMERAPASADA=1;
			}


			//Guardo TODAS las seales de este ciclo
			GuardarSimulacion(CYCLE);

			//Ponemos igual todos los valores actuales para que al entrar despues a diseo se mantengan
			SignalClass.InputValorDesp=SignalClass.InputValor;
			SignalClass.OutputValorDesp=SignalClass.OutputValor;
			SignalClass.InoutValorDesp=SignalClass.InoutValor;
			SignalClass.WireValorDesp=SignalClass.WireValor;

			SimulacionOpProcess.append(SimOpProcess);
			SimOpProcess.clear();
		}




		//Si es DISEO
		if (ORDENBOX.TypeCabeceras.at(i)==TYPEDiseno)
		{
			//Al entrar en diseo guardamos todo lo obtenido en el ciclo anterior para el inicio de este
			SignalClass.InputValor=SignalClass.InputValorDesp;
			SignalClass.OutputValor=SignalClass.OutputValorDesp;
			SignalClass.InoutValor=SignalClass.InoutValorDesp;
			SignalClass.WireValor=SignalClass.WireValorDesp;

			testeo=0;
			//Si esta activada la seal de final no hace nada
			if (FINAL!=1)
			{
				//Comprobamos primero si hay algun box DEFAULT. Si lo hay, vemos si su Next == ID del estado inicio
				for (int x=0; x<BOX.ID.size(); x++)
				{
					if (BOX.TYPE.at(x)=="Default")
					{
						if (BOX.NEXT.at(x)==ORDENBOX.IDStatePadresMatrix[STATESDISENO][i])
						{
							SimOpProcess.append("VALORES POR DEFECTO\n");
							//Ponemos las seales por defecto a sus valores por defecto
							ValoresPorDefecto();
						}
					}
				}
				SimOpProcess.append(" + DISEO:\n");
				//Inicialmente STATES es 1 porque 0 seria el de la cabecera
				//Vamos aumentando cada vez que hemos acabado de analizar todo entre 1 State y otro
				IDInitDiseno=ORDENBOX.IDStatePadresMatrix[STATESDISENO][i];
				ITInitDiseno=ORDENBOX.ITStatePadresMatrix[STATESDISENO][i];
				IDParoDiseno=ORDENBOX.IDStatePadresMatrix[STATESDISENO+1][i];
				ITParoDiseno=ORDENBOX.ITStatePadresMatrix[STATESDISENO+1][i];
	
				SimOpProcess.append("\tState inicio ID: ");
				SimOpProcess.append(QString::number(IDInitDiseno));
				SimOpProcess.append(",\tState inicio IT: ");
				SimOpProcess.append(QString::number(ITInitDiseno));
				TYPEInit=BOX.TYPE.at(ITInitDiseno);
				SimOpProcess.append(",\tSTATE TYPE: ");
				SimOpProcess.append(TYPEInit);
				SimOpProcess.append("\t     ->");
	
				//El ID de un box no puede ser 0. Si el del siguiente es un 0 significa que es un state
				//que vuelve a otro state. En ese caso se comprueba con que text del resto de state coincide 
				//y se guarda en IDParo e ITParo los del STATE con el mismo nombre
				if(IDParoDiseno==0)
				{
					QString TextoStateRepetido=BOX.TEXTMatrix[0][ITInitDiseno];
					for(int k=0; k<=BOX.TOUCH.size(); k++)
					{
						//Si hay un estado igual se pone su IT en ITParoDiseno y su ID en IDParoDiseno
						if((BOX.TEXTMatrix[0][k]==TextoStateRepetido) && (BOX.ID[k]!=IDInitDiseno))
						{
							ITParoDiseno=k;
							IDParoDiseno=BOX.ID.at(k);
							/*SimulacionOpProcess.append("State repetido:");
							SimulacionOpProcess.append(QString::number(IDParoDiseno));*/
							
							//Se busca donde esta el State con ese ID en ORDENBOX.IDStatePadresMatrix y su posicion es STATESDISENO + 1
							for(int l=0; l<3; l++)
							{
								if(IDParoDiseno == ORDENBOX.IDStatePadresMatrix[l][i])
								{
									/*SimulacionOpProcess.append("Encontrado:");
									SimulacionOpProcess.append(QString::number(ORDENBOX.IDStatePadresMatrix[l][i]));
									SimulacionOpProcess.append("con l:");
									SimulacionOpProcess.append(QString::number(l));*/
									STATESDISENO=l;
									l=ORDENBOX.IDStatePadresMatrix.size()-1;
								}
							}
						}
					}
				}
				else
				{
					STATESDISENO=STATESDISENO+1;
				}
	
				SimOpProcess.append("\tState paro sin decisiones ID: ");
				SimOpProcess.append(QString::number(IDParoDiseno));
				SimOpProcess.append(",\tState paro sin decisiones IT: ");
				SimOpProcess.append(QString::number(ITParoDiseno));
				TYPEParo=BOX.TYPE.at(ITParoDiseno);
				SimOpProcess.append(",\tSTATE TYPE: ");
				SimOpProcess.append(TYPEParo);
	
				SimulacionOpProcess.append(SimOpProcess);
				SimOpProcess.clear();
VUELTACAMBIOS:
				//Guardamos las variables del ciclo anterior (Desp) en la variable con la que comprobaremos al final y que nos dice
				//Si cambian los valores en el ciclo
				SignalClass.InputValorAnt=SignalClass.InputValor;
				SignalClass.OutputValorAnt=SignalClass.OutputValor;
				SignalClass.InoutValorAnt=SignalClass.InoutValor;
				SignalClass.WireValorAnt=SignalClass.WireValor;

				//Guardamos la matriz con los valores actuales para comprobar al final si es igual o no
				//SIMULACION.MatrizAntes=SIMULACION.ValoresSim;

				//Una vez tengo claro en que STATE estoy y a cual tengo que ir cuando acabe, recorro lo que hay entre medias
				//Calculamos el NEXT del ID actual
				NextPadre=BOX.NEXT.at(ITInitDiseno);
				
VUELTADISENOOPERATIVA:
				
				//Llamamos a la funcin para saber el iterador correspondiente al Next de el box actual (el hijo siguiente)	
				//NextBox(NextPadre,IteraSiguiente);
				for(int x=0; x<BOX.ID.size(); x++)
				{
					if(NextPadre==BOX.ID.at(x))
					{
						IteraSiguiente=x;
					}
				}			
	
				//Convertimos en Padre al primer hijo despues de header y empezamos a recorrer el TESTBENCH hasta el STATE primero
				IteraPadre=IteraSiguiente;
	
				//Si es diferente de el iterador de paro analiza la informacion
				if ((IteraPadre!=ITParoDiseno) && (IteraPadre!=ITInitDiseno))
				{
					////Actualizamos valores del padre que toca
					IDPadre=BOX.ID.at(IteraPadre);				//Id del padre
					TYPEPadre=BOX.TYPE.at(IteraPadre);			//Type del padre
					NextPadre=BOX.NEXT.at(IteraPadre);			//Next del padre
					Next0Padre=BOX.NEXT0.at(IteraPadre);		//Next0 del padre 
					Next1Padre=BOX.NEXT1.at(IteraPadre);		//Next1 del padre
	
					SimOpProcess.append("  - ");
					SimOpProcess.append(TYPEPadre);
					SimOpProcess.append(": ");
	
					//Podemos encontrarnos diferentes cosas (SyncOps, AsyncOps...)
					//De momento solo nos interesan los DECISION, que indican el camino a seguir
					//								los SyncOps y AsyncOps que nos indican las actividades a realizar
					//								los CondAsyncOps y CondSyncOps que es una mezcla de ambos
					if(TYPEPadre=="SyncOps")
					{
						//Define los valores iniciales de las SIGNAL INPUT
						//Recorre toda la matriz de texto del box
						for(int j=0; j<filasMatricesTexto; j++)
						{
							//Si el texto de la matriz de texto es diferente de null (valor de inicializacion) lee
							if(BOX.TEXTMatrix[j][IteraPadre]!="null")
							{
								TextPadre=BOX.TEXTMatrix[j][IteraPadre];
								SimOpProcess.append(TextPadre);
								SimOpProcess.append("\t->\t");

								int findSignalA=0;
								int posSignal;
								//La siguiente palabra es una seal. La sacamos, la buscamos en las listas y guardamos su valor
								if (GetToken(TextPadre,FALSE)==TOKEN_WORD) 
								{
									int k=0;
									QString NamesSignal;
									while (TextPadre[k].isLetter()) 
									{
										NamesSignal.push_back(TextPadre[k]);
										k++;	
									}
									NamesSignal=NamesSignal.toUpper();
									if (SignalClass.InputList.size()!=0)
									{
										//Buscamos en la lista de INPUTS
										for(int l=0; l<SignalClass.InputList.size(); l++)
										{
											if(SignalClass.InputList.at(l)==NamesSignal)		
											{	
												findSignalA=1;
												posSignal=l;
												l=SignalClass.InputList.size()-1;
											}
										}
									}
									if (SignalClass.OutputList.size()!=0)
									{
										//Buscamos en la lista de OUTPUTS
										for(int l=0; l<SignalClass.OutputList.size(); l++)
										{
											if(SignalClass.OutputList.at(l)==NamesSignal)		
											{	
												findSignalA=2;
												posSignal=l;
												l=SignalClass.OutputList.size()-1;
											}
										}
									}
									if (SignalClass.InoutList.size()!=0)
									{
										//Buscamos en la lista de INOUTS
										for(int l=0; l<SignalClass.InoutList.size(); l++)
										{
											if(SignalClass.InoutList.at(l)==NamesSignal)		
											{	
												findSignalA=3;
												posSignal=l;
												l=SignalClass.InoutList.size()-1;
											}
										}
									}
									if (SignalClass.WireList.size()!=0)
									{
										//Buscamos en la lista de WIRES
										for(int l=0; l<SignalClass.WireList.size(); l++)
										{
											if(SignalClass.WireList.at(l)==NamesSignal)		
											{	
												findSignalA=4;
												posSignal=l;
												l=SignalClass.WireList.size()-1;
											}
										}
									}
									//Borramos
									GetToken(TextPadre,TRUE);	
									SimOpProcess.append("Nombre seal: ");
									SimOpProcess.append(NamesSignal);
								}

								//Eliminamos el <= que lo sigue
								if (GetToken(TextPadre,FALSE)==TOKEN_MENORIGUAL) { GetToken(TextPadre,TRUE);	}

								//Lo siguiente es una expresion.
								int Number=-1;
								ExprToken(TextPadre);

								//Guardamos el numero y borramos
								GetToken(TextPadre,TRUE);
								//Si encontramos logica
								if(HayLogica==1)
								{
									//Dependiendo la comparacion Verdadera o falsa guardo 1 o 0
									if(ComparacionExpresiones==true)	{	Number=1;	}
									else								{	Number=0;	}
								}
								//Sino hay logica
								else
								{
									//Si hay comparacion igual que con la logica
									if(HayComparacion==1)
									{
										if(ComparacionExpresiones==true)	{	Number=1;	}
										else								{	Number=0;	}
									}
									//Si solo es una expresion guardo el numero que me devuelve
									else
									{
										Number=NumDec;
									}
								}

								SimOpProcess.append("\tValor: ");
								SimOpProcess.append(QString::number(Number));
								SimOpProcess.append("\t");
								////Lo siguiente es una expresion.
								////Si es un numero se lanza al analizador de expresiones
								//int Number=-1;
								//if (GetToken(TextPadre,FALSE)==TOKEN_NUM) 
								//{
								//	ExprToken(TextPadre);
								//	//Guardamos el numero y borramos
								//	Number=NumDec;
								//	GetToken(TextPadre,TRUE);

								/*if ((findSignalA==1) || (findSignalA==2) || (findSignalA==3) ||(findSignalA==4))
								{*/
								//Guardamos en la posicin de valor en el siguiente ciclo (porque son seales syncronas)
								if (findSignalA==1)	{	SignalClass.InputValorDesp.replace(posSignal,Number);	}
								if (findSignalA==2)	{	SignalClass.OutputValorDesp.replace(posSignal,Number);	}
								if (findSignalA==3)	{	SignalClass.InoutValorDesp.replace(posSignal,Number);	}
								if (findSignalA==4)	{	SignalClass.WireValorDesp.replace(posSignal,Number);	}
								//}
								//}
								////Sino es una seal
								//else
								//{
								//	int findSignal=0;
								//	ExprToken(TextPadre); //Recibimos la variable SignalNameTOKEN y NumDec
								//	Number=NumDec;
								//	//Buscamos en las listas de seales
								//	for(int l=0; l<SignalClass.InputList.size(); l++)
								//	{
								//		if(SignalClass.InputList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			//Number=SignalClass.InputValor.at(l);
								//		}
								//	}
								//	for(int l=0; l<SignalClass.OutputList.size(); l++)
								//	{
								//		if(SignalClass.OutputList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			//Number=SignalClass.OutputValor.at(l);
								//		}
								//	}
								//	for(int l=0; l<SignalClass.InoutList.size(); l++)
								//	{
								//		if(SignalClass.InoutList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			//Number=SignalClass.InoutValor.at(l);
								//		}
								//	}
								//	for(int l=0; l<SignalClass.WireList.size(); l++)
								//	{
								//		if(SignalClass.WireList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			//Number=SignalClass.WireValor.at(l);
								//		}
								//	}

								//	if (findSignal==1)
								//	{
								//		//Guardamos en la posicin de valor actual
								//		if (findSignalA==1)	{	SignalClass.InputValor.replace(posSignal,Number);	}
								//		if (findSignalA==2)	{	SignalClass.OutputValor.replace(posSignal,Number);	}
								//		if (findSignalA==3)	{	SignalClass.InoutValor.replace(posSignal,Number);	}
								//		if (findSignalA==4)	{	SignalClass.WireValor.replace(posSignal,Number);	}
								//	}
								//}
							}
						}
						SimOpProcess.append("\n");
					}
					else if(TYPEPadre=="AsyncOps")
					{
						//Define los valores iniciales de las SIGNAL INPUT
						//Recorre toda la matriz de texto del box
						for(int j=0; j<filasMatricesTexto; j++)
						{
							//Si el texto de la matriz de texto es diferente de null (valor de inicializacion) lee
							if(BOX.TEXTMatrix[j][IteraPadre]!="null")
							{
								TextPadre=BOX.TEXTMatrix[j][IteraPadre];
								SimOpProcess.append(TextPadre);
								SimOpProcess.append("\t->\t");

								int posSignal;
								int findSignalA=0;
								//La siguiente palabra es una seal. La sacamos, la buscamos en las listas y guardamos su valor
								if (GetToken(TextPadre,FALSE)==TOKEN_WORD) 
								{
									int k=0;
									QString NamesSignal;
									while (TextPadre[k].isLetter()) 
									{
										NamesSignal.push_back(TextPadre[k]);
										k++;	
									}
									NamesSignal=NamesSignal.toUpper();
									if (SignalClass.InputList.size()!=0)
									{
										//Buscamos en la lista de INPUTS
										for(int l=0; l<SignalClass.InputList.size(); l++)
										{
											if(SignalClass.InputList.at(l)==NamesSignal)		
											{	
												findSignalA=1;
												posSignal=l;
												l=SignalClass.InputList.size()-1;
											}
										}
									}
									if (SignalClass.OutputList.size()!=0)
									{
										//Buscamos en la lista de OUTPUTS
										for(int l=0; l<SignalClass.OutputList.size(); l++)
										{
											if(SignalClass.OutputList.at(l)==NamesSignal)		
											{	
												findSignalA=2;
												posSignal=l;
												l=SignalClass.OutputList.size()-1;
											}
										}
									}
									if (SignalClass.InoutList.size()!=0)
									{
										//Buscamos en la lista de INOUTS
										for(int l=0; l<SignalClass.InoutList.size(); l++)
										{
											if(SignalClass.InoutList.at(l)==NamesSignal)		
											{	
												findSignalA=3;
												posSignal=l;
												l=SignalClass.InoutList.size()-1;
											}
										}
									}
									if (SignalClass.WireList.size()!=0)
									{
										//Buscamos en la lista de WIRES
										for(int l=0; l<SignalClass.WireList.size(); l++)
										{
											if(SignalClass.WireList.at(l)==NamesSignal)		
											{	
												findSignalA=4;
												posSignal=l;
												l=SignalClass.WireList.size()-1;
											}
										}
									}
									//Borramos
									GetToken(TextPadre,TRUE);	
									SimOpProcess.append("Nombre seal: ");
									SimOpProcess.append(NamesSignal);
								}

								

								//Eliminamos el <= que lo sigue
								if (GetToken(TextPadre,FALSE)==TOKEN_MENORIGUAL) { GetToken(TextPadre,TRUE);	}

								//Lo siguiente es una expresion.
								int Number=-1;
								ExprToken(TextPadre);

								//Guardamos el numero y borramos
								GetToken(TextPadre,TRUE);
								//Si encontramos logica
								if(HayLogica==1)
								{
									//Dependiendo la comparacion Verdadera o falsa guardo 1 o 0
									if(ComparacionExpresiones==true)	{	Number=1;	}
									else								{	Number=0;	}
								}
								//Sino hay logica
								else
								{
									//Si hay comparacion igual que con la logica
									if(HayComparacion==1)
									{
										if(ComparacionExpresiones==true)	{	Number=1;	}
										else								{	Number=0;	}
									}
									//Si solo es una expresion guardo el numero que me devuelve
									else
									{
										Number=NumDec;
									}
								}
								SimOpProcess.append("\tValor: ");
								SimOpProcess.append(QString::number(Number));
								SimOpProcess.append("\t");
								////Lo siguiente es una expresion.
								////Si es un numero se lanza al analizador de expresiones
								//int Number=-1;
								//if (GetToken(TextPadre,FALSE)==TOKEN_NUM) 
								//{
								//	ExprToken(TextPadre);
								//	//Guardamos el numero y borramos
								//	Number=NumDec;
								//	GetToken(TextPadre,TRUE);

								/*if ((findSignalA==1) || (findSignalA==2) || (findSignalA==3) ||(findSignalA==4))
								{*/
									//Guardamos en la posicin de valor actual
								//Guardamos en la variable de valor actual por ser asincronas
								if (findSignalA==1)	{	SignalClass.InputValor.replace(posSignal,Number);	}
								if (findSignalA==2)	{	SignalClass.OutputValor.replace(posSignal,Number);	}
								if (findSignalA==3)	{	SignalClass.InoutValor.replace(posSignal,Number);	}
								if (findSignalA==4)	{	SignalClass.WireValor.replace(posSignal,Number);	}

								if (findSignalA==1)	{	SignalClass.InputValorDesp.replace(posSignal,Number);	}
								if (findSignalA==2)	{	SignalClass.OutputValorDesp.replace(posSignal,Number);	}
								if (findSignalA==3)	{	SignalClass.InoutValorDesp.replace(posSignal,Number);	}
								if (findSignalA==4)	{	SignalClass.WireValorDesp.replace(posSignal,Number);	}
								//}
								//}
								////Sino es una seal
								//else
								//{
								//	int findSignal=0;
								//	ExprToken(TextPadre); //Recibimos la variable SignalNameTOKEN
								//	Number=NumDec;
								//	//Buscamos en las listas de seales
								//	for(int l=0; l<SignalClass.InputList.size(); l++)
								//	{
								//		if(SignalClass.InputList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			//Number=SignalClass.InputValor.at(l);
								//		}
								//	}
								//	for(int l=0; l<SignalClass.OutputList.size(); l++)
								//	{
								//		if(SignalClass.OutputList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			//Number=SignalClass.OutputValor.at(l);
								//		}
								//	}
								//	for(int l=0; l<SignalClass.InoutList.size(); l++)
								//	{
								//		if(SignalClass.InoutList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			//Number=SignalClass.InoutValor.at(l);
								//		}
								//	}
								//	for(int l=0; l<SignalClass.WireList.size(); l++)
								//	{
								//		if(SignalClass.WireList.at(l)==NameSignalTOKEN)		
								//		{	
								//			//Si coincide el nombre de la seal con las que ya hay se guarda su valor en una variable
								//			findSignal=1;
								//			//Number=SignalClass.WireValor.at(l);
								//		}
								//	}

								//	if (findSignal==1)
								//	{
								//		//Guardamos en la posicin de valor actual
								//		if (findSignalA==1)	
								//		{	
								//			SignalClass.InputValor.replace(posSignal,Number);	
								//		}
								//		if (findSignalA==2)	
								//		{	
								//			SignalClass.OutputValor.replace(posSignal,Number);	
								//		}
								//		if (findSignalA==3)	
								//		{	
								//			SignalClass.InoutValor.replace(posSignal,Number);	
								//		}
								//		if (findSignalA==4)	
								//		{	
								//			SignalClass.WireValor.replace(posSignal,Number);	
								//		}
								//	}
								//}
							}
						}
						SimOpProcess.append("\n");

						//Al ser asincrono ejecuto ya
						//Guardo las seales de este ciclo
//						GuardarSimulacion(CYCLE);
					}
					else if(TYPEPadre=="Decision")
					{
						//Box decision. Esta formado por una condicion que puede ser verdadera o falsa.
						//Si es verdadera el box siguiente es el del ID = Next1
						//Si es falsa el box siguiente es el del ID = NEXT0
						TextPadre=BOX.TEXTMatrix[0][IteraPadre];
						SimOpProcess.append(TextPadre);
						SimOpProcess.append("\t");
	
						//Se manda la expresion
						ExprPreviaToken(TextPadre);
						
						SimOpProcess.append("\t->\tResultado Decision: ");
						
						//PODEMOS RECIBIR 3 COSAS:
						//	-	Logica= si hay logica && o || la variable HayLogica estara a 1 y la variable ComparacionExpresiones indica si es True o False
						//	-	Comparacion= si hay comparacion HayComparacion==1 y la variable ComparacionExpresiones indica si es True o False
						//	-	Signal Valor= tenemos una seal con su valor actual guardado en NumDec
						
						//Si hay logica
						if(HayLogica==1)
						{
							//Vemos si la logica es verdadera o falsa
							if(ComparacionExpresiones==true)	
							{ 
								SimOpProcess.append("Logica Verdadera(Camino 1). Next ID: ");
								NextPadre=Next1Padre; 
								SimOpProcess.append(QString::number(Next1Padre));
								SimOpProcess.append("\n");
							}
							else								
							{
								SimOpProcess.append("Logica Falsa (Camino 0). Next ID: ");
								NextPadre=Next0Padre;	
								SimOpProcess.append(QString::number(Next0Padre));
								SimOpProcess.append("\n");
							}
						}
						else
						{
							//Si no hay logica puede haber comparacion
							if(HayComparacion==1)
							{
								//Vemos si la comparacion es verdadera o falsa
								if(ComparacionExpresiones==true)	
								{ 
									SimOpProcess.append("Comparacion Verdadera (Camino 1). Next ID: ");
									SimOpProcess.append(QString::number(Next1Padre));
									NextPadre=Next1Padre; 
									SimOpProcess.append("\n");
								}
								else
								{ 
									SimOpProcess.append("Comparacion Falsa (Camino 0). Next ID: ");
									SimOpProcess.append(QString::number(Next0Padre));
									SimOpProcess.append("\n");
									NextPadre=Next0Padre;	
								}
							}
							else
							{
								//Si no hay ni logica ni comparacion comprobamos si el NumDec (QUE SERA EL VALOR DE UNA SIGNAL) es 1 o 0
								if(NumDec==1)
								{
									SimOpProcess.append("Valor Signal 1 (Camino 1). Next ID: ");
									SimOpProcess.append(QString::number(Next1Padre));
									SimOpProcess.append("\n");
									NextPadre=Next1Padre;
								}
								else if(NumDec==0)
								{
									SimOpProcess.append("Valor Signal 0 (Camino 0). Next ID: ");
									NextPadre=Next0Padre;
									SimOpProcess.append(QString::number(Next0Padre));
									SimOpProcess.append("\n");
								}
								else
								{
									SimOpProcess.append("DECISION MAL INTERPRETADA EN EXPRESIONES. POR DEFECTO DECISION = 1\n");
								}
							}
						}
					}
					else if(TYPEPadre=="CondSyncOps")
					{
						//Box decision. Esta formado por una condicion que puede ser verdadera o falsa.
						//El box siguiente siempre es Next
						//Si es verdadera se ejecuta el box
						//Si es falsa no se ejecuta el box
						int EJECUTOBOX=-1;
						TextPadre=BOX.TEXTUPMatrix[0][IteraPadre];
						SimOpProcess.append(TextPadre);
						SimOpProcess.append("\t");
	
						//Se manda la expresion
						ExprPreviaToken(TextPadre);
						
						SimOpProcess.append("\t->\tResultado Decision: ");
						SimOpProcess.append(QString::number(NumDec));
						
						//PODEMOS RECIBIR 3 COSAS:
						//	-	Logica= si hay logica && o || la variable HayLogica estara a 1 y la variable ComparacionExpresiones indica si es True o False
						//	-	Comparacion= si hay comparacion HayComparacion==1 y la variable ComparacionExpresiones indica si es True o False
						//	-	Signal Valor= tenemos una seal con su valor actual guardado en NumDec
						
						//Si hay logica
						if(HayLogica==1)
						{
							//Vemos si la logica es verdadera o falsa
							if(ComparacionExpresiones==true)	
							{ 
								SimOpProcess.append(". Logica Verdadera. EJECUTO\n");
								EJECUTOBOX=1;
							}
							else								
							{
								SimOpProcess.append(". Logica Falsa. NO EJECUTO\n");	
								EJECUTOBOX=0;
							}
						}
						else
						{
							//Si no hay logica puede haber comparacion
							if(HayComparacion==1)
							{
								//Vemos si la comparacion es verdadera o falsa
								if(ComparacionExpresiones==true)	
								{ 
									SimOpProcess.append(". Comparacion Verdadera. EJECUTO\n");
									EJECUTOBOX=1;
								}
								else
								{ 
									SimOpProcess.append(". Comparacion Falsa. NO EJECUTO\n");
									EJECUTOBOX=0;
								}
							}
							else
							{
								//Si no hay ni logica ni comparacion comprobamos si el NumDec (QUE SERA EL VALOR DE UNA SIGNAL) es 1 o 0
								if(NumDec==1)
								{
									SimOpProcess.append(". Valor Signal 1. EJECUTO\n");
									EJECUTOBOX=1;
								}
								else if(NumDec==0)
								{
									SimOpProcess.append(". Valor Signal 0. NO EJECUTO\n");
									EJECUTOBOX=0;
								}
								else
								{
									SimOpProcess.append("DECISION MAL INTERPRETADA EN EXPRESIONES. POR DEFECTO DECISION = 0\n");
									ErroresYWarnings("Error! Decision mal interpretada en CondSyncOps del diseo",MSG_Err);
								}
							}
						}
						
						//Dependiendo de si la condicion es verdadera o no ejecuto el box o no
						if (EJECUTOBOX==1)
						{
							//Define los valores iniciales de las SIGNAL INPUT
							//Recorre toda la matriz de texto del box
							for(int j=0; j<filasMatricesTexto; j++)
							{
								//Si el texto de la matriz de texto es diferente de null (valor de inicializacion) lee
								if(BOX.TEXTDOWNMatrix[j][IteraPadre]!="null")
								{
									TextPadre=BOX.TEXTDOWNMatrix[j][IteraPadre];
									SimOpProcess.append(TextPadre);
									SimOpProcess.append("\t");

									int findSignalA=0;
									int posSignal;
									//La siguiente palabra es una seal. La sacamos, la buscamos en las listas y guardamos su valor
									if (GetToken(TextPadre,FALSE)==TOKEN_WORD) 
									{
										int k=0;
										QString NamesSignal;
										while (TextPadre[k].isLetter()) 
										{
											NamesSignal.push_back(TextPadre[k]);
											k++;	
										}
										NamesSignal=NamesSignal.toUpper();
										if (SignalClass.InputList.size()!=0)
										{
											//Buscamos en la lista de INPUTS
											for(int l=0; l<SignalClass.InputList.size(); l++)
											{
												if(SignalClass.InputList.at(l)==NamesSignal)		
												{	
													findSignalA=1;
													posSignal=l;
													l=SignalClass.InputList.size()-1;
												}
											}
										}
										if (SignalClass.OutputList.size()!=0)
										{
											//Buscamos en la lista de OUTPUTS
											for(int l=0; l<SignalClass.OutputList.size(); l++)
											{
												if(SignalClass.OutputList.at(l)==NamesSignal)		
												{	
													findSignalA=2;
													posSignal=l;
													l=SignalClass.OutputList.size()-1;
												}
											}
										}
										if (SignalClass.InoutList.size()!=0)
										{
											//Buscamos en la lista de INOUTS
											for(int l=0; l<SignalClass.InoutList.size(); l++)
											{
												if(SignalClass.InoutList.at(l)==NamesSignal)		
												{	
													findSignalA=3;
													posSignal=l;
													l=SignalClass.InoutList.size()-1;
												}
											}
										}
										if (SignalClass.WireList.size()!=0)
										{
											//Buscamos en la lista de WIRES
											for(int l=0; l<SignalClass.WireList.size(); l++)
											{
												if(SignalClass.WireList.at(l)==NamesSignal)		
												{	
													findSignalA=4;
													posSignal=l;
													l=SignalClass.WireList.size()-1;
												}
											}
										}
										//Borramos
										GetToken(TextPadre,TRUE);	
									}

									//Eliminamos el <= que lo sigue
									if (GetToken(TextPadre,FALSE)==TOKEN_MENORIGUAL) { GetToken(TextPadre,TRUE);	}

									//Lo siguiente es una expresion.
									int Number=-1;
									ExprToken(TextPadre);

									//Guardamos el numero y borramos
									GetToken(TextPadre,TRUE);
									//Si encontramos logica
									if(HayLogica==1)
									{
										//Dependiendo la comparacion Verdadera o falsa guardo 1 o 0
										if(ComparacionExpresiones==true)	{	Number=1;	}
										else								{	Number=0;	}
									}
									//Sino hay logica
									else
									{
										//Si hay comparacion igual que con la logica
										if(HayComparacion==1)
										{
											if(ComparacionExpresiones==true)	{	Number=1;	}
											else								{	Number=0;	}
										}
										//Si solo es una expresion guardo el numero que me devuelve
										else
										{
											Number=NumDec;
										}
									}

							
										if (findSignalA==1)	{	SignalClass.InputValorDesp.replace(posSignal,Number);	}
										if (findSignalA==2)	{	SignalClass.OutputValorDesp.replace(posSignal,Number);	}
										if (findSignalA==3)	{	SignalClass.InoutValorDesp.replace(posSignal,Number);	}
										if (findSignalA==4)	{	SignalClass.WireValorDesp.replace(posSignal,Number);	}
													
								}
							}
							SimOpProcess.append("\n");
						}
					}
					else if(TYPEPadre=="CondAsyncOps")
					{
						//Box decision. Esta formado por una condicion que puede ser verdadera o falsa.
						//El box siguiente siempre es Next
						//Si es verdadera se ejecuta el box
						//Si es falsa no se ejecuta el box
						int EJECUTOBOX=-1;
						TextPadre=BOX.TEXTUPMatrix[0][IteraPadre];
						SimOpProcess.append(TextPadre);
						SimOpProcess.append("\t");
	
						//Se manda la expresion
						ExprPreviaToken(TextPadre);
						
						SimOpProcess.append("\t->\tResultado Decision: ");
						
						//PODEMOS RECIBIR 3 COSAS:
						//	-	Logica= si hay logica && o || la variable HayLogica estara a 1 y la variable ComparacionExpresiones indica si es True o False
						//	-	Comparacion= si hay comparacion HayComparacion==1 y la variable ComparacionExpresiones indica si es True o False
						//	-	Signal Valor= tenemos una seal con su valor actual guardado en NumDec
						
						//Si hay logica
						if(HayLogica==1)
						{
							//Vemos si la logica es verdadera o falsa
							if(ComparacionExpresiones==true)	
							{ 
								SimOpProcess.append("Logica Verdadera. Ejecuto\n");
								EJECUTOBOX=1;
							}
							else								
							{
								SimOpProcess.append("Logica Falsa. NO EJECUTO\n");	
								EJECUTOBOX=0;
							}
						}
						else
						{
							//Si no hay logica puede haber comparacion
							if(HayComparacion==1)
							{
								//Vemos si la comparacion es verdadera o falsa
								if(ComparacionExpresiones==true)	
								{ 
									SimOpProcess.append("Comparacion Verdadera. EJECUTO\n");
									EJECUTOBOX=1;
								}
								else
								{ 
									SimOpProcess.append("Comparacion Falsa. NO EJECUTO\n");
									EJECUTOBOX=0;
								}
							}
							else
							{
								//Si no hay ni logica ni comparacion comprobamos si el NumDec (QUE SERA EL VALOR DE UNA SIGNAL) es 1 o 0
								if(NumDec==1)
								{
									SimOpProcess.append("Valor Signal 1. EJECUTO\n");
									EJECUTOBOX=1;
								}
								else if(NumDec==0)
								{
									SimOpProcess.append("Valor Signal 0. NO EJECUTO\n");
									EJECUTOBOX=0;
								}
								else
								{
									SimOpProcess.append("DECISION MAL INTERPRETADA EN EXPRESIONES. POR DEFECTO DECISION = 0\n");
									ErroresYWarnings("Error! Decision mal interpretada en CondAsyncOps del diseo",MSG_Err);
								}
							}
						}
						
						//Dependiendo de si la condicion es verdadera o no ejecuto el box o no
						if (EJECUTOBOX==1)
						{
							//Define los valores iniciales de las SIGNAL INPUT
							//Recorre toda la matriz de texto del box
							for(int j=0; j<filasMatricesTexto; j++)
							{
								//Si el texto de la matriz de texto es diferente de null (valor de inicializacion) lee
								if(BOX.TEXTDOWNMatrix[j][IteraPadre]!="null")
								{
									TextPadre=BOX.TEXTDOWNMatrix[j][IteraPadre];
									SimOpProcess.append(TextPadre);
									SimOpProcess.append("\t");

									int findSignalA=0;
									int posSignal;
									//La siguiente palabra es una seal. La sacamos, la buscamos en las listas y guardamos su valor
									if (GetToken(TextPadre,FALSE)==TOKEN_WORD) 
									{
										int k=0;
										QString NamesSignal;
										while (TextPadre[k].isLetter()) 
										{
											NamesSignal.push_back(TextPadre[k]);
											k++;	
										}
										NamesSignal=NamesSignal.toUpper();
										if (SignalClass.InputList.size()!=0)
										{
											//Buscamos en la lista de INPUTS
											for(int l=0; l<SignalClass.InputList.size(); l++)
											{
												if(SignalClass.InputList.at(l)==NamesSignal)		
												{	
													findSignalA=1;
													posSignal=l;
													l=SignalClass.InputList.size()-1;
												}
											}
										}
										if (SignalClass.OutputList.size()!=0)
										{
											//Buscamos en la lista de OUTPUTS
											for(int l=0; l<SignalClass.OutputList.size(); l++)
											{
												if(SignalClass.OutputList.at(l)==NamesSignal)		
												{	
													findSignalA=2;
													posSignal=l;
													l=SignalClass.OutputList.size()-1;
												}
											}
										}
										if (SignalClass.InoutList.size()!=0)
										{
											//Buscamos en la lista de INOUTS
											for(int l=0; l<SignalClass.InoutList.size(); l++)
											{
												if(SignalClass.InoutList.at(l)==NamesSignal)		
												{	
													findSignalA=3;
													posSignal=l;
													l=SignalClass.InoutList.size()-1;
												}
											}
										}
										if (SignalClass.WireList.size()!=0)
										{
											//Buscamos en la lista de WIRES
											for(int l=0; l<SignalClass.WireList.size(); l++)
											{
												if(SignalClass.WireList.at(l)==NamesSignal)		
												{	
													findSignalA=4;
													posSignal=l;
													l=SignalClass.WireList.size()-1;
												}
											}
										}
										//Borramos
										GetToken(TextPadre,TRUE);	
									}

									//Eliminamos el <= que lo sigue
									if (GetToken(TextPadre,FALSE)==TOKEN_MENORIGUAL) { GetToken(TextPadre,TRUE);	}

									//Lo siguiente es una expresion.
									int Number=-1;
									ExprToken(TextPadre);

									//Guardamos el numero y borramos
									GetToken(TextPadre,TRUE);
									//Si encontramos logica
									if(HayLogica==1)
									{
										//Dependiendo la comparacion Verdadera o falsa guardo 1 o 0
										if(ComparacionExpresiones==true)	{	Number=1;	}
										else								{	Number=0;	}
									}
									//Sino hay logica
									else
									{
										//Si hay comparacion igual que con la logica
										if(HayComparacion==1)
										{
											if(ComparacionExpresiones==true)	{	Number=1;	}
											else								{	Number=0;	}
										}
										//Si solo es una expresion guardo el numero que me devuelve
										else
										{
											Number=NumDec;
										}
									}

									if (findSignalA==1)	{	SignalClass.InputValor.replace(posSignal,Number);	}
									if (findSignalA==2)	{	SignalClass.OutputValor.replace(posSignal,Number);	}
									if (findSignalA==3)	{	SignalClass.InoutValor.replace(posSignal,Number);	}
									if (findSignalA==4)	{	SignalClass.WireValor.replace(posSignal,Number);	}

									if (findSignalA==1)	{	SignalClass.InputValorDesp.replace(posSignal,Number);	}
									if (findSignalA==2)	{	SignalClass.OutputValorDesp.replace(posSignal,Number);	}
									if (findSignalA==3)	{	SignalClass.InoutValorDesp.replace(posSignal,Number);	}
									if (findSignalA==4)	{	SignalClass.WireValorDesp.replace(posSignal,Number);	}
									
									
								}
							}
							SimOpProcess.append("\n");

							//Al ser asincrono ejecuto ya
							//Guardo las seales de este ciclo
//							GuardarSimulacion(CYCLE);
						}
					}
					else if(TYPEPadre=="Switch")
					{
						SimOpProcess.append("No implantado\n");
						ErroresYWarnings("Error!: BOX SWITCH no implantado en Simulacion Operativa", MSG_Err);
					}
					else if(TYPEPadre=="Code")
					{
						SimOpProcess.append("No implantado\n");
						ErroresYWarnings("Warning!: BOX CODE no implantado en Simulacion Operativa", MSG_War);
					}
					else if(TYPEPadre=="Instance")
					{
						SimOpProcess.append("No implantado\n");
						ErroresYWarnings("Error!: BOX INSTANCE no implantado en Simulacion Operativa", MSG_Err);
					}
					else if(TYPEPadre=="Metastate")
					{
						SimOpProcess.append("No implantado\n");
						ErroresYWarnings("Error!: BOX METASTATE no implantado en Simulacion Operativa", MSG_Err);
					}
					goto VUELTADISENOOPERATIVA;
					SimulacionOpProcess.append(SimOpProcess);
					SimOpProcess.clear();
				}
				//Si encontramos que el iterador coincide con el de un state
				else
				{
					//Si es el mismo que el de inicio significa que hemos llegado a una decision y nos devuelve al state inicial
					if (IteraPadre==ITInitDiseno)
					{
						STATESDISENO=STATESDISENO-1;
						SimulacionOpProcess.append(SimOpProcess);
						SimOpProcess.clear();
					}
					else
					{
	
						//Calculamos el Next del Siguiente. Si es ==0 significa que estamos en un STATE que va a otro
						int IteradorVueltaState=0, IDVueltaState=0;
						NextBox(NextPadre,IteradorVueltaState);
						IDVueltaState=BOX.ID[IteradorVueltaState];
						/*SimulacionOpProcess.append(QString::number(IteradorVueltaState));*/
						/*SimulacionOpProcess.append(QString::number(IDVueltaState));*/
	
						//Si NextPadre == IDVueltaState significa que estamos en un state que nos devuelve a un state con el mismo nombre
						if (NextPadre==IDVueltaState)
						{
							QString StateVuelta=BOX.TEXTMatrix[0][IteraPadre];
							for(int k=0; k<=BOX.TOUCH.size(); k++)
							{
								//Si hay un estado igual se pone su IT en ITParoDiseno y su ID en IDParoDiseno
								if((BOX.TEXTMatrix[0][k]==StateVuelta) && (BOX.ID[k]!=IDInitDiseno))
								{
									ITParoDiseno=k;
									IDParoDiseno=BOX.ID.at(k);
							
									//Se busca donde esta el State con ese ID en ORDENBOX.IDStatePadresMatrix y su posicion es STATESDISENO + 1
									for(int l=0; l<3; l++)
									{
										if(IDParoDiseno == ORDENBOX.IDStatePadresMatrix[l][i])
										{
											//SimulacionOpProcess.append("Encontrado:");
											//SimulacionOpProcess.append(QString::number(ORDENBOX.IDStatePadresMatrix[l][i]));
											//SimulacionOpProcess.append("con l:");
											//SimulacionOpProcess.append(QString::number(l));
											STATESDISENO=l;
											l=ORDENBOX.IDStatePadresMatrix.size()-1;
										}
									}
								}
							}
						}
					}
				}
			}
			//Guardo las seales ASYNCRONAS de este ciclo
//			GuardarASYNC(CYCLE);
		}
	}

	//Guardo las seales de este ciclo
	//GuardarSimulacion(CYCLE);

	//Compruebo que la informacion anterior a la ejecucion del ciclo es igual a la actual.
	//Si cambia (la primera vez siempre cambia) reejecuto. Sino no.
	int CAMBIO=-1;
	//Compruebo las variables antes del ciclo (SignalClass.In-Ou--ValorAnt) y actual que es donde cambio las asyncronas (SignalClass.In-Ou--Valor)
	//ComprobarCambiosEnSimulacion(CAMBIO);

	if (SignalClass.InputValorAnt!=SignalClass.InputValor)		{	CAMBIO=1;	}
	if (SignalClass.OutputValorAnt!=SignalClass.OutputValor)	{	CAMBIO=1;	}
	if (SignalClass.InoutValorAnt!=SignalClass.InoutValor)		{	CAMBIO=1;	}
	if (SignalClass.WireValorAnt!=SignalClass.WireValor)		{	CAMBIO=1;	}

	//Si cambio no se ha puesto a 1 lo pongo a 0
	if(CAMBIO==-1)	{	CAMBIO=0;	}

	//Si cambia en el ciclo vuelvo teniendo en cuenta q
	if (CAMBIO==1)
	{
		SimOpProcess.append("\t\tCAMBIA EN EL CICLO: ");
		SimOpProcess.append(QString::number(CYCLE));
		SimOpProcess.append("\n\n");
		Vuelta++;
		NextPadre=BOX.NEXT.at(ITInitDiseno);

		//Por si nos encontramos un bucle infinito lo repetimos 100o veces
		repeticiones++;
		if (repeticiones < 1000)
		{
			STATESDISENO=STATESDISENO+1;
			goto VUELTACAMBIOS;
		}
		else
		{
			QString ErrorComprobacion;
			ErrorComprobacion.append("Error! Bucle infinito en el diseo. CICLO: ");
			ErrorComprobacion.append(QString::number(CYCLE));
			ErroresYWarnings(ErrorComprobacion,MSG_Err);
		}
	}
	//No cambia
	else if (CAMBIO==0)
	{
		SimOpProcess.append("\t\tNO CAMBIA EN EL CICLO: ");
		SimOpProcess.append(QString::number(CYCLE));
		SimOpProcess.append("\n");
		Vuelta=0;
		repeticiones=0;
	}
	else
	{
		QString ErrorComprobacion;
		ErrorComprobacion.append("Error! Fallo en la comprobacion del cambio de valores en simulacion. CICLO: ");
		ErrorComprobacion.append(QString::number(CYCLE));
		ErroresYWarnings(ErrorComprobacion,MSG_Err);
	}

	//SI HEMOS ACABADO DE RECORRER EL TEST BENCH, LA SEAL FINAL SE HABRA ACTIVADO Y SE PUEDE SALIR. SINO
	//HAY QUE VOLVER A ANALIZAR LOS SIGUIENTES BOXES ENTRE STATES
	if(FINAL!=1)
	{
		//Si estamos en diseo
		if (testeo!=1)
		{
			//Si no hemos llegado al final, en este ciclo guardo los valores actuales (solo seales syncronas)
			//y en el siguiente guardo los valores despues, con syncronas y no
			GuardarActual(CYCLE);

			//Guardo TODAS las seales con las que empieza el ciclo siguiente, que son todas las calculadas en este ciclo,
			//Tanto las ASYNCRONAS, que ya han obtenido su valor en este ciclo (y que no se han modificado entre 2 simulaciones del mismo ciclo)
			//como las SYNCRONAS que hemos obtenido en este ciclo el valor que toman en el futuro (al principio del ciclo siguiente)
			CYCLE++;
			GuardarDespues(CYCLE);

			SignalClass.InputValor=SignalClass.InputValorDesp;
			SignalClass.OutputValor=SignalClass.OutputValorDesp;
			SignalClass.InoutValor=SignalClass.InoutValorDesp;
			SignalClass.WireValor=SignalClass.WireValorDesp;
		}
		
		goto VUELTASIMULACIONOPERATIVA;
	}

	textProcess->addItems(SimulacionOpProcess);
	SimulacionOpProcess.clear();
}

//Funcion para mostrar todos los eventos de la simulacion
void MainWindow::MostrarEventos()
{
	QStringList ListaEventos;
	QString Eventos;

	//Se muestran todas las seales al finalizar la simulacin declarativa
	ListaEventos.append("\n//----------------------------- Lista de eventos -------------------------------//");
	Eventos.append(" - Signal activacion:\t");
	for (int l=0; l<SignalClass.SignalUPEvent.size(); l++)
	{
		Eventos.push_back(SignalClass.SignalUPEvent.at(l));
		Eventos.push_back("\t");
	}
	ListaEventos.append(Eventos);
	Eventos.clear();
	
	Eventos.append(" - Valor activacion:\t");
	for (int l=0; l<SignalClass.SignalUPEventValor.size(); l++)
	{	
		Eventos.push_back(QString::number(SignalClass.SignalUPEventValor.at(l)));
		Eventos.push_back("\t");
	}
	ListaEventos.append(Eventos);
	Eventos.clear();

	Eventos.append(" - Signal cambio:\t");
	for (int l=0; l<SignalClass.SignalDOWNEvent.size(); l++)
	{
		Eventos.push_back(SignalClass.SignalDOWNEvent.at(l));
		Eventos.push_back("\t");
	}
	ListaEventos.append(Eventos);
	Eventos.clear();
	
	Eventos.append(" - Valor cambio:\t");
	for (int l=0; l<SignalClass.SignalDOWNEventValor.size(); l++)
	{	
		Eventos.push_back(QString::number(SignalClass.SignalDOWNEventValor.at(l)));
		Eventos.push_back("\t");
	}
	Eventos.push_back("\n");
	ListaEventos.append(Eventos);
	textOut->addItems(ListaEventos);
	TXTOut(ListaEventos);
	ListaEventos.clear();
	Eventos.clear();
}

//Funcion encargada de guardar los valores por defecto
void MainWindow::ValoresPorDefecto()
{

	//Recorre las seales almacenadas como ejecutadoras de eventos
	for(int i=0; i<SignalClass.SignalDefaultName.size(); i++)
	{
		QString SignalDefault=SignalClass.SignalDefaultName.at(i);
		int SignalDefaultValor=SignalClass.SignalDefaultValor.at(i);

		//Buscamos la seal en las listas y comprobamos si el valor actual coincide con el del evento
		for(int l=0; l<SignalClass.InputList.size(); l++)
		{
			if(SignalClass.InputList.at(l)==SignalDefault)		
			{	
				//Si coincide el nombre de la seal con las que ya hay se comprueba pone su valor por defecto
				SignalClass.InputValorDesp.replace(l,SignalDefaultValor);
			}
		}
		for(int l=0; l<SignalClass.OutputList.size(); l++)
		{
			if(SignalClass.OutputList.at(l)==SignalDefault)		
			{	
				//Si coincide el nombre de la seal con las que ya hay se comprueba pone su valor por defecto
				SignalClass.OutputValorDesp.replace(l,SignalDefaultValor);
			}
		}
		for(int l=0; l<SignalClass.InoutList.size(); l++)
		{
			if(SignalClass.InoutList.at(l)==SignalDefault)		
			{	
				//Si coincide el nombre de la seal con las que ya hay se comprueba pone su valor por defecto
				SignalClass.InoutValorDesp.replace(l,SignalDefaultValor);
			}
		}
		for(int l=0; l<SignalClass.WireList.size(); l++)
		{
			if(SignalClass.WireList.at(l)==SignalDefault)		
			{	
				//Si coincide el nombre de la seal con las que ya hay se comprueba pone su valor por defecto
				SignalClass.WireValorDesp.replace(l,SignalDefaultValor);
			}
		}
	}
}

//Esta funcion se encarga de comprobar si alguna condicion (SignalClass.SignalUpEvent) que ejecuta eventos esta activa
//por ejemplo que la seal reset == 1, y si es asi ejecuta los cambios marcados en SignalClass.SignalDownEvent modificando
//el valor de las seales en SignalClass.InputValor, OutputValor...
void MainWindow::AnalizadorEventos()
{
	QStringList AnalizadorEventos;
	QString SignalEventON;
	int SignalEventONValor=0;
	int Number;
	int findSignal=0;

	AnalizadorEventos.append("\n//--------------Analizador Eventos--------------//");
	textProcess->addItems(AnalizadorEventos);
	AnalizadorEventos.clear();

	//Recorre las seales almacenadas como ejecutadoras de eventos
	for(int i=0; i<SignalClass.SignalUPEvent.size(); i++)
	{
		//Se guardan los valores de las seales ejecutadoras
		SignalEventON=SignalClass.SignalUPEvent.at(i);
		SignalEventONValor=SignalClass.SignalUPEventValor.at(i);
		/*AnalizadorEventos.append(SignalEventON);
		AnalizadorEventos.append(QString::number(SignalEventONValor));*/

		//Buscamos la seal en las listas y comprobamos si el valor actual coincide con el del evento
		for(int l=0; l<SignalClass.InputList.size(); l++)
		{
			if(SignalClass.InputList.at(l)==SignalEventON)		
			{	
				//Si coincide el nombre de la seal con las que ya hay se comprueba su valor actual y si coincide con el 
				//valor que la activa en el evento
				Number=SignalClass.InputValor.at(l);
				findSignal=1;
			}
		}
		for(int l=0; l<SignalClass.OutputList.size(); l++)
		{
			if(SignalClass.OutputList.at(l)==SignalEventON)		
			{	
				//Si coincide el nombre de la seal con las que ya hay se comprueba su valor actual y si coincide con el 
				//valor que la activa en el evento
				findSignal=1;
				Number=SignalClass.OutputValor.at(l);
			}
		}
		for(int l=0; l<SignalClass.InoutList.size(); l++)
		{
			if(SignalClass.InoutList.at(l)==SignalEventON)		
			{	
				//Si coincide el nombre de la seal con las que ya hay se comprueba su valor actual y si coincide con el 
				//valor que la activa en el evento
				findSignal=1;
				Number=SignalClass.InoutValor.at(l);
			}
		}
		for(int l=0; l<SignalClass.WireList.size(); l++)
		{
			if(SignalClass.WireList.at(l)==SignalEventON)		
			{	
				//Si coincide el nombre de la seal con las que ya hay se comprueba su valor actual y si coincide con el 
				//valor que la activa en el evento
				findSignal=1;
				Number=SignalClass.WireValor.at(l);
			}
		}

		//Si ha encontrado la seal
		if(findSignal==1)
		{
			AnalizadorEventos.append("Evento encontrado");
			//Se comprueba si el valor actual de la seal coincide con el de la que la activa
			if(Number==SignalEventONValor)
			{
				//Hay que cambiar los valores de las seales
				//Se guardan los valores de las seales al ser ejecutadas
				QString SignalEJECT=SignalClass.SignalDOWNEvent.at(i);
				int SignalEJECTValor=SignalClass.SignalDOWNEventValor.at(i);
				//Buscamos la seal en las listas y si la encontramos cambiamos su valor
				for(int l=0; l<SignalClass.InputList.size(); l++)
				{
					if(SignalClass.InputList.at(l)==SignalEJECT)		
					{	
						//Si coincide el nombre de la seal con las que ya hay se cambia su valor
						SignalClass.InputValor.replace(l,SignalEJECTValor);
					}
				}
				for(int l=0; l<SignalClass.OutputList.size(); l++)
				{
					if(SignalClass.OutputList.at(l)==SignalEJECT)		
					{	
						//Si coincide el nombre de la seal con las que ya hay se cambia su valor
						SignalClass.OutputValor.replace(l,SignalEJECTValor);
					}
				}
				for(int l=0; l<SignalClass.InoutList.size(); l++)
				{
					if(SignalClass.InoutList.at(l)==SignalEJECT)		
					{	
						//Si coincide el nombre de la seal con las que ya hay se cambia su valor
						SignalClass.InoutValor.replace(l,SignalEJECTValor);
					}
				}
				for(int l=0; l<SignalClass.WireList.size(); l++)
				{
					if(SignalClass.WireList.at(l)==SignalEJECT)		
					{	
						//Si coincide el nombre de la seal con las que ya hay se cambia su valor
						SignalClass.WireValor.replace(l,SignalEJECTValor);
					}
				}
			}
		}
	}
	AnalizadorEventos.push_back("\t");
	textProcess->addItems(AnalizadorEventos);
	AnalizadorEventos.clear();
}

//Esta funcion se encarga de guardar al final de cada ciclo todos los valores de la simulacin en class DatosSimulacion
//Dejandolo de la siguiente manera (EJEMPLO):
//
//						   CYCLE
//SIMULACION.SignalSim				reset		go		inA		inB		inC	...
//SIMULACION.ValoresSim		[0]		  1			0		 1		 0		 1	...
//							[1]		  0			1		 0		 1		 0	...
//							[2]		  0			1		 1		 0		 0	...
//							[3]		  1			0		 0		 0		 0	...
//							...		 ...	   ...		...		...		...	...
void MainWindow::GuardarSimulacion(int CYCLE)
{
	QString Ciclos;
	QStringList CyclesProcess;
	
	CyclesProcess.append("//------------SAVE CYCLE-----------//");
	Ciclos.append("- Ciclo: ");
	Ciclos.append(QString::number(CYCLE));
	CyclesProcess.append(Ciclos);
	Ciclos.clear();

	////Solo guardamos las asncronas
	//if (tipo == 0)
	//{
	//	Ciclos.append("ASYNCRONAS:\n");
	//	int i, Input=0;
	//	if(SignalClass.InputValor.size()!=0)
	//	{
	//		for(i=0; i<SignalClass.InputValor.size(); i++)
	//		{
	//			Ciclos.append(QString::number(SIMULACION.ValoresSim[CYCLE][i]));
	//			if (SignalClass.SyncInput.at(i)=="ASYNC")
	//			{
	//				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.InputValor[i];

	//				Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.InputList[i]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.InputValor[i]));
	//			}
	//			CyclesProcess.append(Ciclos);
	//			Ciclos.clear();
	//		}
	//		Input=i;
	//	}
	//	if(SignalClass.OutputValor.size()!=0)
	//	{
	//		int j=0;
	//		for(i=Input; i<Input+SignalClass.OutputValor.size(); i++)
	//		{
	//			Ciclos.append(QString::number(SIMULACION.ValoresSim[CYCLE][i]));
	//			Ciclos.append(QString::number(SignalClass.OutputValor[j]));
	//			if (SignalClass.SyncOutput.at(j)=="ASYNC")
	//			{
	//				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.OutputValor[j];
	//				//SIMULACION.ValoresSim[CYCLE][i]=SignalClass.OutputValor[j]);

	//				Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.OutputList[j]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.OutputValor[j]));
	//				

	//				
	//			}
	//			j++;
	//			CyclesProcess.append(Ciclos);
	//			Ciclos.clear();
	//			
	//		}
	//		Input=i;
	//	}
	//	if(SignalClass.InoutValor.size()!=0)
	//	{
	//		int j=0;
	//		for(i=Input; i<Input+SignalClass.InoutValor.size(); i++)
	//		{
	//			Ciclos.append(QString::number(SIMULACION.ValoresSim[CYCLE][i]));
	//			if (SignalClass.SyncInout.at(j)=="ASYNC")
	//			{
	//	//			SIMULACION.ValoresSim[CYCLE][i]=SignalClass.InoutValor[j];
	//	//			
	//				Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.InoutList[j]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.InoutValor[j]));

	//				
	//			}
	//			j++;
	//			CyclesProcess.append(Ciclos);
	//			Ciclos.clear();
	//		}
	//		Input=i;
	//	}
	//	if(SignalClass.WireValor.size()!=0)
	//	{
	//		int j=0;
	//		for(i=Input; i<Input+SignalClass.WireValor.size(); i++)
	//		{
	//			Ciclos.append(QString::number(SIMULACION.ValoresSim[CYCLE][i]));
	//			if (SignalClass.SyncWire.at(j)=="ASYNC")
	//			{
	//	//			SIMULACION.ValoresSim[CYCLE][i]=SignalClass.WireValor[j];
	//				
	//				//Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.WireList[j]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.WireValor[j]));

	//				//j++;
	//			}
	//			j++;
	//			CyclesProcess.append(Ciclos);
	//			Ciclos.clear();
	//		}
	//	}
	//}
	////Solo guardamos las sncronas
	//else if (tipo == 1)
	//{
	//	Ciclos.append("SYNCRONAS:\n");
	//	int i, Input;
	//	if(SignalClass.InputValor.size()!=0)
	//	{
	//		for(i=0; i<SignalClass.InputValor.size(); i++)
	//		{
	//			if (SignalClass.SyncInput.at(i)=="SYNC")
	//			{
	//				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.InputValor[i];
	//				Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.InputList[i]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.InputValor[i]));
	//				CyclesProcess.append(Ciclos);
	//				Ciclos.clear();
	//			}
	//		}
	//		Input=i;
	//	}
	//	if(SignalClass.OutputValor.size()!=0)
	//	{
	//		int j=0;
	//		for(i=Input; i<Input+SignalClass.OutputValor.size(); i++)
	//		{
	//			if (SignalClass.SyncOutput.at(j)=="SYNC")
	//			{
	//				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.OutputValor[j];

	//				Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.OutputList[j]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.OutputValor[j]));
	//				CyclesProcess.append(Ciclos);
	//				Ciclos.clear();

	//				//j++;
	//			}
	//			j++;
	//		}
	//		Input=i;
	//	}
	//	if(SignalClass.InoutValor.size()!=0)
	//	{
	//		int j=0;
	//		for(i=Input; i<Input+SignalClass.InoutValor.size(); i++)
	//		{
	//			if (SignalClass.SyncInout.at(j)=="SYNC")
	//			{
	//				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.InoutValor[j];
	//				

	//				Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.InoutList[j]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.InoutValor[j]));
	//				CyclesProcess.append(Ciclos);
	//				Ciclos.clear();

	//				//j++;
	//			}
	//			j++;
	//		}
	//		Input=i;
	//	}
	//	if(SignalClass.WireValor.size()!=0)
	//	{
	//		int j=0;
	//		for(i=Input; i<Input+SignalClass.WireValor.size(); i++)
	//		{
	//			if (SignalClass.SyncWire.at(j)=="SYNC")
	//			{
	//				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.WireValor[j];

	//				Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.WireList[j]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.WireValor[j]));
	//				CyclesProcess.append(Ciclos);
	//				Ciclos.clear();

	//				//j++;
	//			}
	//			j++;
	//		}
	//	}
	//}
	//guarda todas
	//else if (tipo == 2)
	//{
		Ciclos.append("TODAS:\n");
		int i, Input;
		if(SignalClass.InputValor.size()!=0)
		{
			for(i=0; i<SignalClass.InputValor.size(); i++)
			{
					SIMULACION.ValoresSim[CYCLE][i]=SignalClass.InputValor[i];
					Ciclos.append("\t-  ");
					Ciclos.append(SignalClass.InputList[i]);
					Ciclos.append(":\t");
					Ciclos.append(QString::number(SignalClass.InputValor[i]));
				CyclesProcess.append(Ciclos);
				Ciclos.clear();
			}
			Input=i;
		}
		if(SignalClass.OutputValor.size()!=0)
		{
			int j=0;
			for(i=Input; i<Input+SignalClass.OutputValor.size(); i++)
			{
				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.OutputValor[j];

				Ciclos.append("\t-  ");
				Ciclos.append(SignalClass.OutputList[j]);
				Ciclos.append(":\t");
				Ciclos.append(QString::number(SignalClass.OutputValor[j]));
				CyclesProcess.append(Ciclos);
				Ciclos.clear();

				j++;
			}
			Input=i;
		}
		if(SignalClass.InoutValor.size()!=0)
		{
			int j=0;
			for(i=Input; i<Input+SignalClass.InoutValor.size(); i++)
			{
				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.InoutValor[j];

				Ciclos.append("\t-  ");
				Ciclos.append(SignalClass.InoutList[j]);
				Ciclos.append(":\t");
				Ciclos.append(QString::number(SignalClass.InoutValor[j]));
				CyclesProcess.append(Ciclos);
				Ciclos.clear();

				j++;
			}
			Input=i;
		}
		if(SignalClass.WireValor.size()!=0)
		{
			int j=0;
			for(i=Input; i<Input+SignalClass.WireValor.size(); i++)
			{
				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.WireValor[j];

				Ciclos.append("\t-  ");
				Ciclos.append(SignalClass.WireList[j]);
				Ciclos.append(":\t");
				Ciclos.append(QString::number(SignalClass.WireValor[j]));
				CyclesProcess.append(Ciclos);
				Ciclos.clear();

				j++;
			}
		}
	//}
	textProcess->addItems(CyclesProcess);
	CyclesProcess.clear();
}

//Copia de la anterior, pero esta se encarga de guardar los valores actuales en la simulacion operativa (Asyncronas)
void MainWindow::GuardarActual(int CYCLE)
{
	QString Ciclos;
	QStringList CyclesProcess;
	
	CyclesProcess.append("//------------SAVE CYCLE AHORA-----------//");
	Ciclos.append("- Ciclo: ");
	Ciclos.append(QString::number(CYCLE));
	CyclesProcess.append(Ciclos);
	Ciclos.clear();

	////Solo guardamos las asncronas
	//if (tipo == 0)
	//{
	//	Ciclos.append("ASYNCRONAS:\n");
	//	int i, Input=0;
	//	if(SignalClass.InputValor.size()!=0)
	//	{
	//		for(i=0; i<SignalClass.InputValor.size(); i++)
	//		{
	//			Ciclos.append(QString::number(SIMULACION.ValoresSim[CYCLE][i]));
	//			if (SignalClass.SyncInput.at(i)=="ASYNC")
	//			{
	//				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.InputValor[i];

	//				Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.InputList[i]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.InputValor[i]));
	//			}
	//			CyclesProcess.append(Ciclos);
	//			Ciclos.clear();
	//		}
	//		Input=i;
	//	}
	//	if(SignalClass.OutputValor.size()!=0)
	//	{
	//		int j=0;
	//		for(i=Input; i<Input+SignalClass.OutputValor.size(); i++)
	//		{
	//			Ciclos.append(QString::number(SIMULACION.ValoresSim[CYCLE][i]));
	//			Ciclos.append(QString::number(SignalClass.OutputValor[j]));
	//			if (SignalClass.SyncOutput.at(j)=="ASYNC")
	//			{
	//				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.OutputValor[j];
	//				//SIMULACION.ValoresSim[CYCLE][i]=SignalClass.OutputValor[j]);

	//				Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.OutputList[j]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.OutputValor[j]));
	//				

	//				
	//			}
	//			j++;
	//			CyclesProcess.append(Ciclos);
	//			Ciclos.clear();
	//			
	//		}
	//		Input=i;
	//	}
	//	if(SignalClass.InoutValor.size()!=0)
	//	{
	//		int j=0;
	//		for(i=Input; i<Input+SignalClass.InoutValor.size(); i++)
	//		{
	//			Ciclos.append(QString::number(SIMULACION.ValoresSim[CYCLE][i]));
	//			if (SignalClass.SyncInout.at(j)=="ASYNC")
	//			{
	//	//			SIMULACION.ValoresSim[CYCLE][i]=SignalClass.InoutValor[j];
	//	//			
	//				Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.InoutList[j]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.InoutValor[j]));

	//				
	//			}
	//			j++;
	//			CyclesProcess.append(Ciclos);
	//			Ciclos.clear();
	//		}
	//		Input=i;
	//	}
	//	if(SignalClass.WireValor.size()!=0)
	//	{
	//		int j=0;
	//		for(i=Input; i<Input+SignalClass.WireValor.size(); i++)
	//		{
	//			Ciclos.append(QString::number(SIMULACION.ValoresSim[CYCLE][i]));
	//			if (SignalClass.SyncWire.at(j)=="ASYNC")
	//			{
	//	//			SIMULACION.ValoresSim[CYCLE][i]=SignalClass.WireValor[j];
	//				
	//				//Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.WireList[j]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.WireValor[j]));

	//				//j++;
	//			}
	//			j++;
	//			CyclesProcess.append(Ciclos);
	//			Ciclos.clear();
	//		}
	//	}
	//}
	////Solo guardamos las sncronas
	//else if (tipo == 1)
	//{
	//	Ciclos.append("SYNCRONAS:\n");
	//	int i, Input;
	//	if(SignalClass.InputValor.size()!=0)
	//	{
	//		for(i=0; i<SignalClass.InputValor.size(); i++)
	//		{
	//			if (SignalClass.SyncInput.at(i)=="SYNC")
	//			{
	//				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.InputValor[i];
	//				Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.InputList[i]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.InputValor[i]));
	//				CyclesProcess.append(Ciclos);
	//				Ciclos.clear();
	//			}
	//		}
	//		Input=i;
	//	}
	//	if(SignalClass.OutputValor.size()!=0)
	//	{
	//		int j=0;
	//		for(i=Input; i<Input+SignalClass.OutputValor.size(); i++)
	//		{
	//			if (SignalClass.SyncOutput.at(j)=="SYNC")
	//			{
	//				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.OutputValor[j];

	//				Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.OutputList[j]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.OutputValor[j]));
	//				CyclesProcess.append(Ciclos);
	//				Ciclos.clear();

	//				//j++;
	//			}
	//			j++;
	//		}
	//		Input=i;
	//	}
	//	if(SignalClass.InoutValor.size()!=0)
	//	{
	//		int j=0;
	//		for(i=Input; i<Input+SignalClass.InoutValor.size(); i++)
	//		{
	//			if (SignalClass.SyncInout.at(j)=="SYNC")
	//			{
	//				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.InoutValor[j];
	//				

	//				Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.InoutList[j]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.InoutValor[j]));
	//				CyclesProcess.append(Ciclos);
	//				Ciclos.clear();

	//				//j++;
	//			}
	//			j++;
	//		}
	//		Input=i;
	//	}
	//	if(SignalClass.WireValor.size()!=0)
	//	{
	//		int j=0;
	//		for(i=Input; i<Input+SignalClass.WireValor.size(); i++)
	//		{
	//			if (SignalClass.SyncWire.at(j)=="SYNC")
	//			{
	//				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.WireValor[j];

	//				Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.WireList[j]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.WireValor[j]));
	//				CyclesProcess.append(Ciclos);
	//				Ciclos.clear();

	//				//j++;
	//			}
	//			j++;
	//		}
	//	}
	//}
	//guarda todas
	//else if (tipo == 2)
	//{
		int i, Input;
		if(SignalClass.InputValor.size()!=0)
		{
			for(i=0; i<SignalClass.InputValor.size(); i++)
			{

					SIMULACION.ValoresSim[CYCLE][i]=SignalClass.InputValor[i];
					Ciclos.append("\t-  ");
					Ciclos.append(SignalClass.InputList[i]);
					Ciclos.append(":\t");
					Ciclos.append(QString::number(SignalClass.InputValor[i]));
				
				CyclesProcess.append(Ciclos);
				Ciclos.clear();
			}
			Input=i;
		}
		if(SignalClass.OutputValor.size()!=0)
		{
			int j=0;
			for(i=Input; i<Input+SignalClass.OutputValor.size(); i++)
			{
				
				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.OutputValor[j];

				Ciclos.append("\t-  ");
				Ciclos.append(SignalClass.OutputList[j]);
				Ciclos.append(":\t");
				Ciclos.append(QString::number(SignalClass.OutputValor[j]));
				
				CyclesProcess.append(Ciclos);
				Ciclos.clear();

				j++;
			}
			Input=i;
		}
		if(SignalClass.InoutValor.size()!=0)
		{
			int j=0;
			for(i=Input; i<Input+SignalClass.InoutValor.size(); i++)
			{
				
				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.InoutValor[j];

				Ciclos.append("\t-  ");
				Ciclos.append(SignalClass.InoutList[j]);
				Ciclos.append(":\t");
				Ciclos.append(QString::number(SignalClass.InoutValor[j]));
				
				CyclesProcess.append(Ciclos);
				Ciclos.clear();

				j++;
			}
			Input=i;
		}
		if(SignalClass.WireValor.size()!=0)
		{
			int j=0;
			for(i=Input; i<Input+SignalClass.WireValor.size(); i++)
			{
				
				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.WireValor[j];

				Ciclos.append("\t-  ");
				Ciclos.append(SignalClass.WireList[j]);
				Ciclos.append(":\t");
				Ciclos.append(QString::number(SignalClass.WireValor[j]));
				
				CyclesProcess.append(Ciclos);
				Ciclos.clear();

				j++;
			}
		}
	//}
	textProcess->addItems(CyclesProcess);
	CyclesProcess.clear();
}

//Copia de la anterior, pero esta se encarga de guardar los valores de inicio del siguiente ciclo en la simulacion operativa
void MainWindow::GuardarDespues(int CYCLE)
{
	QString Ciclos;
	QStringList CyclesProcess;
	
	CyclesProcess.append("//------------SAVE CYCLE DESPUES-----------//");
	Ciclos.append("- Ciclo: ");
	Ciclos.append(QString::number(CYCLE));
	CyclesProcess.append(Ciclos);
	Ciclos.clear();

	////Solo guardamos las asncronas
	//if (tipo == 0)
	//{
	//	Ciclos.append("ASYNCRONAS:\n");
	//	int i, Input=0;
	//	if(SignalClass.InputValor.size()!=0)
	//	{
	//		for(i=0; i<SignalClass.InputValor.size(); i++)
	//		{
	//			Ciclos.append(QString::number(SIMULACION.ValoresSim[CYCLE][i]));
	//			if (SignalClass.SyncInput.at(i)=="ASYNC")
	//			{
	//				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.InputValor[i];

	//				Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.InputList[i]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.InputValor[i]));
	//			}
	//			CyclesProcess.append(Ciclos);
	//			Ciclos.clear();
	//		}
	//		Input=i;
	//	}
	//	if(SignalClass.OutputValor.size()!=0)
	//	{
	//		int j=0;
	//		for(i=Input; i<Input+SignalClass.OutputValor.size(); i++)
	//		{
	//			Ciclos.append(QString::number(SIMULACION.ValoresSim[CYCLE][i]));
	//			Ciclos.append(QString::number(SignalClass.OutputValor[j]));
	//			if (SignalClass.SyncOutput.at(j)=="ASYNC")
	//			{
	//				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.OutputValor[j];
	//				//SIMULACION.ValoresSim[CYCLE][i]=SignalClass.OutputValor[j]);

	//				Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.OutputList[j]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.OutputValor[j]));
	//				

	//				
	//			}
	//			j++;
	//			CyclesProcess.append(Ciclos);
	//			Ciclos.clear();
	//			
	//		}
	//		Input=i;
	//	}
	//	if(SignalClass.InoutValor.size()!=0)
	//	{
	//		int j=0;
	//		for(i=Input; i<Input+SignalClass.InoutValor.size(); i++)
	//		{
	//			Ciclos.append(QString::number(SIMULACION.ValoresSim[CYCLE][i]));
	//			if (SignalClass.SyncInout.at(j)=="ASYNC")
	//			{
	//	//			SIMULACION.ValoresSim[CYCLE][i]=SignalClass.InoutValor[j];
	//	//			
	//				Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.InoutList[j]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.InoutValor[j]));

	//				
	//			}
	//			j++;
	//			CyclesProcess.append(Ciclos);
	//			Ciclos.clear();
	//		}
	//		Input=i;
	//	}
	//	if(SignalClass.WireValor.size()!=0)
	//	{
	//		int j=0;
	//		for(i=Input; i<Input+SignalClass.WireValor.size(); i++)
	//		{
	//			Ciclos.append(QString::number(SIMULACION.ValoresSim[CYCLE][i]));
	//			if (SignalClass.SyncWire.at(j)=="ASYNC")
	//			{
	//	//			SIMULACION.ValoresSim[CYCLE][i]=SignalClass.WireValor[j];
	//				
	//				//Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.WireList[j]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.WireValor[j]));

	//				//j++;
	//			}
	//			j++;
	//			CyclesProcess.append(Ciclos);
	//			Ciclos.clear();
	//		}
	//	}
	//}
	////Solo guardamos las sncronas
	//else if (tipo == 1)
	//{
	//	Ciclos.append("SYNCRONAS:\n");
	//	int i, Input;
	//	if(SignalClass.InputValor.size()!=0)
	//	{
	//		for(i=0; i<SignalClass.InputValor.size(); i++)
	//		{
	//			if (SignalClass.SyncInput.at(i)=="SYNC")
	//			{
	//				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.InputValor[i];
	//				Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.InputList[i]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.InputValor[i]));
	//				CyclesProcess.append(Ciclos);
	//				Ciclos.clear();
	//			}
	//		}
	//		Input=i;
	//	}
	//	if(SignalClass.OutputValor.size()!=0)
	//	{
	//		int j=0;
	//		for(i=Input; i<Input+SignalClass.OutputValor.size(); i++)
	//		{
	//			if (SignalClass.SyncOutput.at(j)=="SYNC")
	//			{
	//				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.OutputValor[j];

	//				Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.OutputList[j]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.OutputValor[j]));
	//				CyclesProcess.append(Ciclos);
	//				Ciclos.clear();

	//				//j++;
	//			}
	//			j++;
	//		}
	//		Input=i;
	//	}
	//	if(SignalClass.InoutValor.size()!=0)
	//	{
	//		int j=0;
	//		for(i=Input; i<Input+SignalClass.InoutValor.size(); i++)
	//		{
	//			if (SignalClass.SyncInout.at(j)=="SYNC")
	//			{
	//				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.InoutValor[j];
	//				

	//				Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.InoutList[j]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.InoutValor[j]));
	//				CyclesProcess.append(Ciclos);
	//				Ciclos.clear();

	//				//j++;
	//			}
	//			j++;
	//		}
	//		Input=i;
	//	}
	//	if(SignalClass.WireValor.size()!=0)
	//	{
	//		int j=0;
	//		for(i=Input; i<Input+SignalClass.WireValor.size(); i++)
	//		{
	//			if (SignalClass.SyncWire.at(j)=="SYNC")
	//			{
	//				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.WireValor[j];

	//				Ciclos.append("\t-  ");
	//				Ciclos.append(SignalClass.WireList[j]);
	//				Ciclos.append(":\t");
	//				Ciclos.append(QString::number(SignalClass.WireValor[j]));
	//				CyclesProcess.append(Ciclos);
	//				Ciclos.clear();

	//				//j++;
	//			}
	//			j++;
	//		}
	//	}
	//}
	//guarda todas
	//else if (tipo == 2)
	//{
		int i, Input;
		if(SignalClass.InputValor.size()!=0)
		{
			for(i=0; i<SignalClass.InputValorDesp.size(); i++)
			{
				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.InputValorDesp[i];
				Ciclos.append("\t-  ");
				Ciclos.append(SignalClass.InputList[i]);
				Ciclos.append(":\t");
				Ciclos.append(QString::number(SignalClass.InputValorDesp[i]));
				CyclesProcess.append(Ciclos);
				Ciclos.clear();
			}
			Input=i;
		}
		if(SignalClass.OutputValor.size()!=0)
		{
			int j=0;
			for(i=Input; i<Input+SignalClass.OutputValorDesp.size(); i++)
			{
				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.OutputValorDesp[j];

				Ciclos.append("\t-  ");
				Ciclos.append(SignalClass.OutputList[j]);
				Ciclos.append(":\t");
				Ciclos.append(QString::number(SignalClass.OutputValorDesp[j]));
				
				CyclesProcess.append(Ciclos);
				Ciclos.clear();

				j++;
			}
			Input=i;
		}
		if(SignalClass.InoutValor.size()!=0)
		{
			int j=0;
			for(i=Input; i<Input+SignalClass.InoutValorDesp.size(); i++)
			{
		
				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.InoutValorDesp[j];

				Ciclos.append("\t-  ");
				Ciclos.append(SignalClass.InoutList[j]);
				Ciclos.append(":\t");
				Ciclos.append(QString::number(SignalClass.InoutValorDesp[j]));
				
				CyclesProcess.append(Ciclos);
				Ciclos.clear();

				j++;
			}
			Input=i;
		}
		if(SignalClass.WireValor.size()!=0)
		{
			int j=0;
			for(i=Input; i<Input+SignalClass.WireValorDesp.size(); i++)
			{
				SIMULACION.ValoresSim[CYCLE][i]=SignalClass.WireValorDesp[j];

				Ciclos.append("\t-  ");
				Ciclos.append(SignalClass.WireList[j]);
				Ciclos.append(":\t");
				Ciclos.append(QString::number(SignalClass.WireValorDesp[j]));
				
				CyclesProcess.append(Ciclos);
				Ciclos.clear();

				j++;
			}
		}
	
	textProcess->addItems(CyclesProcess);
	CyclesProcess.clear();
}

//Esta funcin recible el ciclo en el que esta y se encarga de mostrar el valor de todas las seales a la salida del ciclo
void MainWindow::MostrarSignalPorCicloSimulacion()
{
	QString Ciclos, CiclosTXT;
	QStringList CyclesProcess, CyclesProcessTXT;
	
	Ciclos.append("\n//-------------------------SIMULACION----------------------//");
	CyclesProcess.append(Ciclos);
	Ciclos.clear();

	CiclosTXT.append("\n//-------------------------SIMULACION----------------------//");
	CyclesProcessTXT.append(CiclosTXT);
	CiclosTXT.clear();

	//VISTA EN HORIZONTAL
	Ciclos.append("\t");
	CiclosTXT.append("\t\t");
	for(int i=0; i<=CYCLE;i++)
	{
		Ciclos.append("[");
		Ciclos.append(QString::number(i));
		Ciclos.append("]\t");

		CiclosTXT.append("[");
		CiclosTXT.append(QString::number(i));
		CiclosTXT.append("]\t\t");
	}
	CyclesProcess.append(Ciclos);
	Ciclos.clear();
	CyclesProcessTXT.append(CiclosTXT);
	CiclosTXT.clear();

	for(int i=0; i<SIMULACION.SignalSim.size();i++)
	{
		Ciclos.append(SIMULACION.SignalSim.at(i));
		Ciclos.append("\t");

		CiclosTXT.append(SIMULACION.SignalSim.at(i));
		CiclosTXT.append("\t\t");
		for(int j=0; j<=CYCLE; j++)
		{
			QString Number;
			Number.append(QString::number(SIMULACION.ValoresSim[j][i]));
			Ciclos.append(QString::number(SIMULACION.ValoresSim[j][i]));
			CiclosTXT.append(QString::number(SIMULACION.ValoresSim[j][i]));

			Ciclos.append("\t");

			if(Number.size()>7)
			{
				CiclosTXT.append("\t");
			}
			else
			{
				CiclosTXT.append("\t\t");
			}
		}
		CyclesProcess.append(Ciclos);
		Ciclos.clear();

		CyclesProcessTXT.append(CiclosTXT);
		CiclosTXT.clear();
	}

	//VISTA EN VERTICAL
	/*Ciclos.append("    + SignalName:\t\t");
	for(int i=0; i<SIMULACION.SignalSim.size();i++)
	{
		Ciclos.append(SIMULACION.SignalSim.at(i));
		Ciclos.append("\t");
	}
	CyclesProcess.append(Ciclos);
	Ciclos.clear();
	
	Ciclos.append("    + SignalValor:");
	for(int i=0; i<52;i++)
	{
		if(i==0)
		{
			Ciclos.append("\t[");
			Ciclos.append(QString::number(i));
			Ciclos.append("]\t");
		}
		else
		{
			Ciclos.append("\t\t[");
			Ciclos.append(QString::number(i));
			Ciclos.append("]\t");
		}
		for(int j=0; j<SIMULACION.SignalSim.size(); j++)
		{
			Ciclos.append(QString::number(SIMULACION.ValoresSim[i][j]));
			Ciclos.append("\t");
		}
		CyclesProcess.append(Ciclos);
		Ciclos.clear();
	}
	*/
	textOut->addItems(CyclesProcess);
	TXTOut(CyclesProcessTXT);
	CyclesProcess.clear();
	CyclesProcessTXT.clear();
}

//Esta funcion se encarga de mostrar todas las verificaciones que hay que realizar
void MainWindow::MostrarVerificacionesARealizar()
{
	QString Verifico;
	QStringList VerificacionProcess;
	
	Verifico.append("\n//-------------------------VERIFICACIONES------------------------//");
	VerificacionProcess.append(Verifico);
	Verifico.clear();

	//Primero compruebo que no hay verificaciones erroneas o iguales (que me pida comprobar un valor diferente para una misma variable en el mismo ciclo)
	for(int i=0; i<VERIFICACIONES.Ciclo.size(); i++)
	{
		for(int j=0; j<VERIFICACIONES.Ciclo.size(); j++)
		{
			if (VERIFICACIONES.Ciclo.at(i)==VERIFICACIONES.Ciclo.at(j))
			{
				if(VERIFICACIONES.NameSignal.at(i)==VERIFICACIONES.NameSignal.at(j))
				{
					if(VERIFICACIONES.Valor.at(i)!=VERIFICACIONES.Valor.at(j))
					{
						//Remplazo por el valor de i que al salir primero esta guardado como evento aislado
						VERIFICACIONES.Valor.replace(j,VERIFICACIONES.Valor.at(i));
					}
				}
			}
		}
	}


	Verifico.append("Ciclo de actuacion:  \t  ");
	for(int i=0; i<VERIFICACIONES.Ciclo.size(); i++)
	{
		Verifico.append(QString::number(VERIFICACIONES.Ciclo.at(i)));
		Verifico.append("\t");
	}
	Verifico.append("\nVariable a verificar: \t");
	for(int i=0; i<VERIFICACIONES.NameSignal.size(); i++)
	{
		Verifico.append(VERIFICACIONES.NameSignal.at(i));
		Verifico.append("\t");
	}
	Verifico.append("\nValor a verificar:    \t  ");
	for(int i=0; i<VERIFICACIONES.Valor.size(); i++)
	{
		Verifico.append(QString::number(VERIFICACIONES.Valor.at(i)));
		Verifico.append("\t");
	}
	VerificacionProcess.append(Verifico);
	textOut->addItems(VerificacionProcess);
	TXTOut(VerificacionProcess);
	Verifico.clear();
}

//Esta funcion se encarga de recorrer todas las verificaciones y comprobar si son correctas o no
void MainWindow::RealizarVerificaciones()
{
	QString Verificacion;
	QStringList VerificacionProcess;
	
	VerificacionProcess.append(Verificacion);
	Verificacion.clear();

	int NumeroCiclo;
	QString NombreSenal;
	int ValorSenalVerificar;
	int ValorSenalReal;
	int CORRECTO=0, ERRONEO=0;

	for(int i=0; i<VERIFICACIONES.Ciclo.size(); i++)
	{
		//Actualizo
		NumeroCiclo=VERIFICACIONES.Ciclo.at(i);
		NombreSenal=VERIFICACIONES.NameSignal.at(i);
		ValorSenalVerificar=VERIFICACIONES.Valor.at(i);
		
		for(int l=0; l<SIMULACION.SignalSim.size(); l++)
		{
			if(SIMULACION.SignalSim.at(l)==NombreSenal)		
			{	
				ValorSenalReal=SIMULACION.ValoresSim[NumeroCiclo][l];

				if(ValorSenalReal == ValorSenalVerificar)
				{
					Verificacion.append("Verificacion CORRECTA ->\tCiclo: ");
					Verificacion.append(QString::number(NumeroCiclo));
					Verificacion.append(",\tSeal: ");
					Verificacion.append(NombreSenal);
					Verificacion.append(",\t    Valor: ");
					Verificacion.append(QString::number(ValorSenalVerificar));
					VerificacionProcess.append(Verificacion);
					Verificacion.clear();
					CORRECTO=CORRECTO+1;
				}
				else
				{
					Verificacion.append("Verificacion ERRONEA  ->\tCiclo: ");
					Verificacion.append(QString::number(NumeroCiclo));
					Verificacion.append(",\tSeal: ");
					Verificacion.append(NombreSenal);
					Verificacion.append(",\t    Valor a verificar: ");
					Verificacion.append(QString::number(ValorSenalVerificar));
					Verificacion.append("   !=   Valor simulado: ");
					Verificacion.append(QString::number(ValorSenalReal));
					VerificacionProcess.append(Verificacion);
					Verificacion.clear();
					ERRONEO=ERRONEO+1;
				}
			}
		}
	}
	textProcess->addItems(VerificacionProcess);
	Verificacion.clear();
	VerificacionProcess.clear();

	Verificacion.append("\n//-------------------------VERIFICACIONES------------------------//\n");
	Verificacion.append("Verificaciones correctas: ");
	Verificacion.append(QString::number(CORRECTO));
	Verificacion.append("\nVerificaciones erroneas: ");
	Verificacion.append(QString::number(ERRONEO));
	VerificacionProcess.append(Verificacion);
	textOut->addItems(VerificacionProcess);
	TXTOut(VerificacionProcess);
	Verificacion.clear();
	VerificacionProcess.clear();
}

//FUNCIONES PARA GUARDAR LOS DATOS DEL ANALISIS DE ESTRUCTURAS

//Esta funcin crea una matriz para guardar los textos de los diferentes box
//Es una matriz de 50 filas y 300 columnas. En cada columna va el texto de un box y en cada fila sus diferentes text de sus %CR%
//			
//					------------------------------------------------------------------
// iteradorBox ->	|	0	1	2	3	4	5	6	7	8	9	10	11	12	13	14	...
//					------------------------------------------------------------------
// Text				|	
// %CR%				|
// %CR% 			|
// ...				|
//
//					------------------------------------------------------------------
void MatrizTexto()
{
	for(int i=0; i<filasMatricesTexto; i++)
	{
		QVector<QString> TEXTVector;
		QString a="null";
		for(int j=0; j<300; j++)
		{
			TEXTVector.push_back(a);
		}
		BOX.TEXTMatrix.push_back(TEXTVector);
	}

	//EJEMPLO PARA MODIFICAR UNA DE LAS CASILLAS Y PONER UN 25:
	//(BOX.TEXTMatrix[2][1])="25";
}

//Esta funcin crea una matriz para guardar los textos UP de los diferentes box
void MatrizTextoUp()
{
	for(int i=0; i<filasMatricesTextoUp; i++)
	{
		QVector<QString> TEXTUPVector;
		QString a="null";
		for(int j=0; j<300; j++)
		{
			TEXTUPVector.push_back(a);
		}
		BOX.TEXTUPMatrix.push_back(TEXTUPVector);
	}

	//EJEMPLO PARA MODIFICAR UNA DE LAS CASILLAS Y PONER UN 25:
	//(BOX.TEXTUPMatrix[2][1])="25";
}

//Esta funcin crea una matriz para guardar los textos DOWN de los diferentes box
void MatrizTextoDown()
{
	for(int i=0; i<filasMatricesTexto; i++)
	{
		QVector<QString> TEXTDOWNVector;
		QString a="null";
		for(int j=0; j<300; j++)
		{
			TEXTDOWNVector.push_back(a);
		}
		BOX.TEXTDOWNMatrix.push_back(TEXTDOWNVector);
	}

	//EJEMPLO PARA MODIFICAR UNA DE LAS CASILLAS Y PONER UN 25:
	//(BOX.TEXTDOWNMatrix[2][1])="25";
}

//Esta funcin crea una matriz para guardar los ID de los estados de los diferentes padres para la simulacion
void MatrizIDStatesPadres()
{
	for(int i=0; i<100; i++)
	{
		QVector<int> StatesPadresVector;
		int a=0;
		for(int j=0; j<ORDENBOX.IdCabeceras.size(); j++)
		{
			StatesPadresVector.push_back(a);
		}
		ORDENBOX.IDStatePadresMatrix.push_back(StatesPadresVector);
	}

	//EJEMPLO PARA MODIFICAR UNA DE LAS CASILLAS Y PONER UN 25:
	//(BOX.TEXTDOWNMatrix[2][1])="25";
}

//Esta funcin crea una matriz para guardar el ITERADOR de los estados de los diferentes padres para la simulacion
void MatrizITStatesPadres()
{
	for(int i=0; i<100; i++)
	{
		QVector<int> StatesPadresVector;
		int a=0;
		for(int j=0; j<ORDENBOX.IdCabeceras.size(); j++)
		{
			StatesPadresVector.push_back(a);
		}
		ORDENBOX.ITStatePadresMatrix.push_back(StatesPadresVector);
	}

	//EJEMPLO PARA MODIFICAR UNA DE LAS CASILLAS Y PONER UN 25:
	//(BOX.TEXTDOWNMatrix[2][1])="25";
}

//Esta funcion crea la matriz para guardar todos los valores de las seales a lo largo de los ciclos.
//Por defecto se inicializa todo a -1 y el numero de ciclos a 300
void MatrizDatosSimulacionFinal(int NumSignals)
{
	int NumCiclos=300;

	for(int i=0; i<=NumCiclos; i++)
	{
		QVector<int> SimVector;
		int a=-1;
		for(int j=0; j<NumSignals; j++)
		{
			SimVector.push_back(a);
		}
		SIMULACION.ValoresSim.push_back(SimVector);
	}

	//EJEMPLO PARA MODIFICAR UNA DE LAS CASILLAS Y PONER UN 25:
	//(BOX.TEXTMatrix[2][1])="25";
}

//FUNCIONES PARA ANALIZAR LAS EXPRESIONES A TRAVES DE TOKENS

//Esta funcion recibe un String y lo prepara comprobando si esta vacio o no
void MainWindow::ExprPreviaToken(QString &Frase)
{
	QString frase=Frase; 
	int Token;

	HayLogica=0;
	HayComparacion=0;
	NumDec=0;

	TokenProcess.append("\n//-----------EXPRESION PREVIA A ANALIZAR-------------//");
	QString TokenProc;
	TokenProc.append("CICLO DE SIMULACION: ");
	TokenProc.append(QString::number(CYCLE));
	TokenProcess.append(TokenProc);
	TokenProc.clear();

	//Mostramos la frase inicial de entrada
	TokenProcess.append(Frase);
	textProcess->addItems(TokenProcess);
	TokenProcess.clear();

	//Conseguimos el token
	Token=GetToken(Frase,FALSE);

	//Si la frase esta vacia, se devuelve el valor de TOKEN_NULL y se muestra
	if ( Token==TOKEN_NULL )
	{
		TokenProcess.append("Frase vacia");
		textProcess->addItems(TokenProcess);
		TokenProcess.clear();
	}
	//Sino, mientras no este vacio o detecta %CR%, o ; o manda nueva expresion
	else
	{
		if ( Token==TOKEN_RETORNOCARRO )
		{
			TokenProcess.append("Retorno de carro");
			GetToken(Frase,TRUE);
				
		}
		else if ( Token==TOKEN_PUNTOCOMA )
		{
			TokenProcess.append("Fin de frase ;");
			GetToken(Frase,TRUE);
			//break;
		}
		else if ( Token==TOKEN_COMA )
		{
			TokenProcess.append("Coma");
			GetToken(Frase,TRUE);
			//continue;
		}
		else
		{
			//Mandamos la frase al analizador de expresiones iniciales
			ExprToken(Frase);
			//Guardamos la frase devuelta en la frase que llegaba a esta funcion
			//frase=Frase;
		}
		Token=GetToken(Frase,FALSE);
	}

	//Calcula y muestra que falta por analizar
	int longInicial=0, longFinal=0, longitud=0;
	QString sinAnalizar="NO ANALIZADO: ", final;

	//Calculo de longitud de frase que esta sin analizar
	longFinal=frase.length();
	longInicial=Frase.length();
	longitud=longInicial-longFinal;

	//Se guarda la parte sin analizar y se muestra si hay algo sin analizar
	final=Frase;
	final=final.remove(0,longitud);
	if (GetToken(final,FALSE)==TOKEN_NULL)	
	{ 
		sinAnalizar = "ANALIZADO TODO";
		//Se agrega y se muestra
		TokenProcess.append(sinAnalizar);
		textProcess->addItems(TokenProcess);
		TokenProcess.clear();
	}
	else
	{
		final=Frase;
		final=final.remove(0,longitud);
		sinAnalizar.push_back(final);
		//Se agrega y se muestra
		TokenProcess.append(sinAnalizar);
		textProcess->addItems(TokenProcess);
		TokenProcess.clear();
	}
	
}

//Esta funcion analiza expresiones CONDICIONALES del tipo (expr <= exprA ? exprA : exprA...)
//siendo expr <= exprA (logica) ? valor_si_cierto : valor_si_falso;
void MainWindow::ExprToken(QString &frase)
{
	//QStringList TokenProcess;
	int NumAntes, NumAhoraCierto, NumAhoraFalso;
	bool LogicaAntes;
	//TokenProcess.append("Expr");
	//TokenProcess.append(frase);
	//textOut->addItems(TokenProcess);
	//TokenProcess.clear();

	//Primera iteracin
	ExprAToken(frase);
	NumAntes=NumDec;
	LogicaAntes=ComparacionExpresiones;
	int i=1;

	while(1) {
		//Las expresiones estan formadas por una ExprA seguido o no de ?

		//Si es la segunda iteracion
		if (i>1)
		{
			//Mandamos la expresion A
			ExprAToken(frase);
			NumAhoraFalso=NumDec;
			
			if(LogicaAntes==true)	{	NumDec=NumAhoraCierto;	}
			else					{	NumDec=NumAhoraFalso;	}
		}

		//Recibimos una frase, si esa tiene a continuacion un ?
		if (GetToken(frase, FALSE) == TOKEN_QUESTION)
		{
			//Elimina ?
			TokenProcess.append("Encontrado ?");
			GetToken(frase, TRUE);
			TokenProcess.append(frase);
			//Vuelve a mandar la frase ya sin ?, porq despues de ? aparece otra exprA
			ExprAToken(frase);
			NumAhoraCierto=NumDec;
			//Guardamos la condicin de vuelta
			//Despues de ExprA tiene que aparecer :, si no aparecen muestra seal, si aparece elimina
			if (GetToken(frase, TRUE) != TOKEN_DOSPUNTOS)
			{
				TokenProcess.append("Syntax error: looking for ':'");
				break;
			}
			TokenProcess.append("Encontrado :");
			i++;
			continue;
		}
		break;
	}
	//TokenProcess.append(frase);
	textProcess->addItems(TokenProcess);
	TokenProcess.clear();
}

//Esta funcion analiza expresiones LOGICAS && o ||
void MainWindow::ExprAToken(QString &frase)
{
	//QStringList TokenProcess;
	int Token, YLOG;
	bool ExprAntes, ExprAhora, ExprPor, SumExpr, ExprDespues;
	//TokenProcess.append("ExprA");

	//Las expresiones estan formadas por una ExprA seguido o no de ?
	//Mandamos la expresion B
	ExprBToken(frase);
	ExprAntes=ComparacionExpresiones;
	SumExpr=0+ExprAntes;

	while(1) {
		Token = GetToken(frase, FALSE);
		//Recibimos una frase, si esa tiene a continuacion un && lo elimina
		if (Token == TOKEN_YLOGIC)
		{
			HayLogica=1;

			TokenProcess.append("Encontrado &&");
			GetToken(frase, TRUE);
			ExprBToken(frase);
			ExprAhora=ComparacionExpresiones;
			LogicaExpr(SumExpr,ExprAhora,1,SumExpr);
			continue;
		}
		//Si tiene un || lo elimina
		if (Token == TOKEN_OLOGIC)
		{
			HayLogica=1;

			TokenProcess.append("Encontrado ||");
			GetToken(frase, TRUE);
			ExprBToken(frase);
			ExprAhora=ComparacionExpresiones;

			//Leemos el siguiente token. Si es && tenemos que realizar esa operacin primero
VUELTAAND:
			Token = GetToken(frase, FALSE);
			if (Token == TOKEN_YLOGIC)
			{
				YLOG=1;
				GetToken(frase, TRUE);
				ExprBToken(frase);
				ExprDespues=ComparacionExpresiones;
				LogicaExpr(ExprAhora,ExprDespues,1,ExprPor);
				ExprAhora=ExprPor;
				goto VUELTAAND;
			}
			//Sino se realiza la suma entre el acumulado de sumas y numero ahora
			else
			{
				if(YLOG==1)
				{
					ExprAhora=ExprPor;
					ExprPor=0;
					YLOG=0;
				}
				LogicaExpr(SumExpr,ExprAhora,2,SumExpr);
			}
			continue;
		}
		break;
	}

	if (HayLogica==1)
	{
		ComparacionExpresiones=SumExpr;
		if(ComparacionExpresiones==1)	{ TokenProcess.append("Logica TRUE");	}
		if(ComparacionExpresiones==0)	{ TokenProcess.append("Logica FALSE");	}
	}
	textProcess->addItems(TokenProcess);
	TokenProcess.clear();
}

//Esta funcion analiza expresiones LOGICAS DE BIT (&(AND),^,~^,^~(XOR,NOT XOR),|(OR))
//Recibe un string con una expresion B que son una concatenacion de expresiones C separadas o no de las expresiones logicas
void MainWindow::ExprBToken(QString &frase)
{
	//QStringList TokenProcess;
	int Token, AND, POR, XOR;
	int NumAntes, NumAhora, SumNum, NumPor, NumDespues;
	//TokenProcess.append("ExprB");

	//Las expresiones estan formadas por una ExprC seguido o no de &, |, ^, ~^, ^~
	//Mandamos la expresion C
	ExprCToken(frase);
	NumAntes=NumDec;
	SumNum=0+NumAntes;
	while(1) {
		
		Token = GetToken(frase, FALSE);

		//Recibimos una frase, si esa tiene a continuacion un & lo elimina
		if (Token == TOKEN_AND)
		{
			TokenProcess.append("Encontrado &");
			GetToken(frase, TRUE);
			ExprCToken(frase);
			NumAhora=NumDec;
			LogicaBit(SumNum,NumAhora,1,SumNum);
			continue;
		}
		//Si tiene ^,  ~^, ^~ lo elimina
		if ((Token == TOKEN_XOR) || (Token == TOKEN_NXOR))
		{
			if (Token == TOKEN_NXOR)
			{
				TokenProcess.append("Encontrado ~^");
				GetToken(frase,TRUE);
				ExprCToken(frase);
				NumAhora=NumDec;

				//Leemos el siguiente token. Si es & realizamos esto primero
VUELTAY:
				Token = GetToken(frase, FALSE);
				if (Token == TOKEN_AND)
				{
					AND=1;
					GetToken(frase, TRUE);
					ExprCToken(frase);
					NumDespues=NumDec;
					LogicaBit(NumAhora,NumDespues,1,NumPor);
					NumAhora=NumPor;
					goto VUELTAY;
				}
				//Sino se realiza la suma entre el acumulado
				else
				{
					if(AND==1)
					{
						NumAhora=NumPor;
						NumPor=0;
						POR=0;
					}
					LogicaBit(SumNum,NumAhora,2,SumNum);
				}
				continue;
			}
			else
			{
				TokenProcess.append("Encontrado ^");
				//Leemos el siguiente token. Si es & realizamos esto primero
VUELTAY2:
				Token = GetToken(frase, FALSE);
				if (Token == TOKEN_AND)
				{
					AND=1;
					GetToken(frase, TRUE);
					ExprCToken(frase);
					NumDespues=NumDec;
					LogicaBit(NumAhora,NumDespues,1,NumPor);
					NumAhora=NumPor;
					goto VUELTAY2;
				}
				//Sino se realiza la suma entre el acumulado
				else
				{
					if(AND==1)
					{
						NumAhora=NumPor;
						NumPor=0;
						POR=0;
					}
					LogicaBit(SumNum,NumAhora,3,SumNum);
				}
				continue;
			}
		}
		//Si tiene un | lo elimina
		if (Token == TOKEN_OR)
		{
			//////////////////////////////////////FALTA METER XOR Y NXOR ENTRE OR Y AND
			AND=0;
			XOR=0;
			TokenProcess.append("Encontrado |");
			GetToken(frase, TRUE);
			ExprCToken(frase);
			NumAhora=NumDec;

			//Leemos el siguiente token. Si es & tenemos que realizar esa operacin primero 
VUELTAYOR:
			Token = GetToken(frase, FALSE);
			if (Token == TOKEN_AND)
			{
				AND=1;
				GetToken(frase, TRUE);
				ExprCToken(frase);
				NumDespues=NumDec;
				LogicaBit(NumAhora,NumDespues,1,NumPor);
				NumAhora=NumPor;
				goto VUELTAYOR;
			}
			//Sino se realiza la OR entre el acumulado y numero ahora
			else
			{
				if(AND==1)
				{
					NumAhora=NumPor;
					NumPor=0;
					AND=0;
				}
				LogicaBit(SumNum,NumAhora,4,SumNum);
			}
			continue;
		}
		break;
	}

	NumDec=SumNum;

	//TokenProcess.append(QString::number(NumDec));
	textProcess->addItems(TokenProcess);
	TokenProcess.clear();
}

//Esta funcion analiza expresiones de IGUALDAD (==, !=, ===, !==) o relacionales (<,<=,>,>=)
//La expresion C que llega puede estar formada por 1 expresion D o 2 comparadas
void MainWindow::ExprCToken(QString &frase)
{
	//QStringList TokenProcess;
	int Token, tipo=0;
	int NumAntes, NumAhora;
	//TokenProcess.append("ExprC");
	//Las expresiones estan formadas por una ExprD seguido o no de ==, <, >, != y otra ExprD
	//Mandamos la expresion D
	ExprDToken(frase);
	NumAntes=NumDec;
	Token = GetToken(frase, FALSE);

	if ((Token == TOKEN_DOSIGUAL) || (Token == TOKEN_NOTIGUAL))		{ tipo=1; }
	if ((Token == TOKEN_MENOR) || (Token == TOKEN_MENORIGUAL))		{ tipo=2; }
	if ((Token == TOKEN_MAYOR) || (Token == TOKEN_MAYORIGUAL))		{ tipo=3; }
	if ((Token == TOKEN_TRESIGUAL) || (Token == TOKEN_NOTDOSIGUAL))	{ tipo=4; }

	switch (tipo)
	{
		case 1:
			if (Token == TOKEN_DOSIGUAL)	
			{ 
				HayComparacion=1;

				TokenProcess.append("Encontrado ==");
				GetToken(frase, TRUE);
				//Mandamos la expresion D
				ExprDToken(frase);
				NumAhora=NumDec;
				Comparacion(NumAntes,NumAhora,3,ComparacionExpresiones);
			}
			if (Token == TOKEN_NOTIGUAL)
			{ 
				HayComparacion=1;

				TokenProcess.append("Encontrado !=");
				GetToken(frase, TRUE);
				//Mandamos la expresion D
				ExprDToken(frase);
				NumAhora=NumDec;
				Comparacion(NumAntes,NumAhora,4,ComparacionExpresiones);
			}
			break;
		case 2:
			if (Token == TOKEN_MENOR)		
			{ 
				HayComparacion=1;

				TokenProcess.append("Encontrado <");
				GetToken(frase, TRUE);
				//Mandamos la expresion D
				ExprDToken(frase);
				NumAhora=NumDec;
				Comparacion(NumAntes,NumAhora,6,ComparacionExpresiones);
			}
			if (Token == TOKEN_MENORIGUAL)	
			{ 
				HayComparacion=1;

				TokenProcess.append("Encontrado <=");
				GetToken(frase, TRUE);
				//Mandamos la expresion D
				ExprDToken(frase);
				NumAhora=NumDec;
				Comparacion(NumAntes,NumAhora,5,ComparacionExpresiones);
			}
			break;
		case 3:
			if (Token == TOKEN_MAYOR)		
			{ 
				HayComparacion=1;

				TokenProcess.append("Encontrado >");
				GetToken(frase, TRUE);
				//Mandamos la expresion D
				ExprDToken(frase);	
				NumAhora=NumDec;
				Comparacion(NumAntes,NumAhora,8,ComparacionExpresiones);
			}
			if (Token == TOKEN_MAYORIGUAL)
			{ 
				HayComparacion=1;

				TokenProcess.append("Encontrado >=");
				GetToken(frase, TRUE);
				//Mandamos la expresion D
				ExprDToken(frase);
				NumAhora=NumDec;
				Comparacion(NumAntes,NumAhora,7,ComparacionExpresiones);
			}
			break;
		case 4:
			if (Token == TOKEN_TRESIGUAL)
			{ 
				HayComparacion=1;

				TokenProcess.append("Encontrado ===");
				GetToken(frase, TRUE);
				//Mandamos la expresion D
				ExprDToken(frase);	
				NumAhora=NumDec;
				Comparacion(NumAntes,NumAhora,1,ComparacionExpresiones);
			}
			if (Token == TOKEN_NOTDOSIGUAL)
			{ 
				HayComparacion=1;

				TokenProcess.append("Encontrado !==");
				GetToken(frase, TRUE);
				//Mandamos la expresion D
				ExprDToken(frase);
				NumAhora=NumDec;
				Comparacion(NumAntes,NumAhora,2,ComparacionExpresiones);
			}	
			break;  
		default:
			HayComparacion=0;
			break;
	}

	if (HayComparacion==1)
	{
		if(ComparacionExpresiones==1)	{ TokenProcess.append("Comparacion TRUE");	}
		if(ComparacionExpresiones==0)	{ TokenProcess.append("Comparacion FALSE");	}
	}
	//TokenProcess.append(QString::number(ComparacionExpresiones));
	textProcess->addItems(TokenProcess);
	TokenProcess.clear();
}

//Esta funcion analiza expresiones de DESPLAZAMIENTO DE BITS (<<, <<<, >>, >>>)
//Recibe un string con una expresion D, que puede contener <<, <<<, >>, >>> o no. 
void MainWindow::ExprDToken(QString &frase)
{
	int Token, tipo=0;
	int NumAntes, NumAhora;
	//TokenProcess.append("ExprD");
	//Las expresiones estan formadas por una ExprE seguido o no de <<, <<<, >>, >>> y otra ExprE
	//Mandamos la expresion E
	ExprEToken(frase);
	NumAntes=NumDec;
	Token = GetToken(frase, FALSE);

	if ((Token == TOKEN_DOSIZQ) || (Token == TOKEN_TRESIZQ))	{ tipo=1; }
	if ((Token == TOKEN_DOSDER) || (Token == TOKEN_TRESDER))	{ tipo=2; }

	switch (tipo)
	{
		case 1:
			if (Token == TOKEN_DOSIZQ)
			{
				TokenProcess.append("Encontrado <<");
				GetToken(frase, TRUE);
				ExprEToken(frase);
				NumAhora=NumDec;
				Desplazamiento(NumAntes,NumAhora,1,NumDec);
			}
			else
			{
				TokenProcess.append("Encontrado <<<");
				GetToken(frase, TRUE);
				ExprEToken(frase);
				NumAhora=NumDec;
				Desplazamiento(NumAntes,NumAhora,1,NumDec); ///////////////////CAMBIAR 1 POR 3 CUANDO SE MODIFIQUE LA FUNCION EN simulador.cpp
			}
			break;

		case 2:
			if (Token == TOKEN_DOSDER)
			{
				TokenProcess.append("Encontrado >>");
				GetToken(frase, TRUE);
				ExprEToken(frase);
				NumAhora=NumDec;
				Desplazamiento(NumAntes,NumAhora,2,NumDec);
			}
			else
			{
				TokenProcess.append("Encontrado >>>");
				GetToken(frase, TRUE);
				ExprEToken(frase);
				NumAhora=NumDec;
				Desplazamiento(NumAntes,NumAhora,2,NumDec);  ///////////////////CAMBIAR 2 POR 4 CUANDO SE MODIFIQUE LA FUNCION EN simulador.cpp
			}
			break;

		default:
			break;
	}
	//TokenProcess.append(QString::number(NumDec));
	textProcess->addItems(TokenProcess);
	TokenProcess.clear();
}

//Esta funcion analiza expresiones ARITMETICAS (+, -, *, /, %)
//Recibe un string con una expresion E y analiza que simbolo hay
void MainWindow::ExprEToken(QString &frase)
{
	int Token, POR=0;
	int SumNum=0, NumAntes=0, NumAhora=0, NumDespues=0, NumPor=0;
	//TokenProcess.append("ExprE");

	//Las expresiones estan formadas por una ExprF seguido o no de +, -, *, /, % y otra ExprF o no
	//Mandamos la expresion F
	ExprFToken(frase);
	NumAntes=NumDec;
	SumNum=0+NumAntes;
	
	while(1) {

		Token = GetToken(frase, FALSE);
		//Recibimos una frase, si esa tiene a continuacion un *, % o / lo elimina
		if ((Token == TOKEN_POTENCIA) || (Token == TOKEN_DIVISION) || (Token == TOKEN_PORCENTAJE))
		{
			if (Token == TOKEN_POTENCIA)
			{
				TokenProcess.append("Encontrado *");
				GetToken(frase, TRUE);
				ExprFToken(frase);
				NumAhora=NumDec;
				Multiplicacion(SumNum,NumAhora,SumNum);
				continue;
			}
			
			if (Token == TOKEN_DIVISION)
			{
				TokenProcess.append("Encontrado /");
				GetToken(frase, TRUE);
				ExprFToken(frase);
				NumAhora=NumDec;
				Division(SumNum,NumAhora,SumNum);
				continue;
			}

			else
			{
				TokenProcess.append("Encontrado %");
				GetToken(frase, TRUE);
				ExprFToken(frase);
				NumAhora=NumDec;
				Porcentaje(SumNum,NumAhora,SumNum);
				continue;
			}
		}
		//Si tiene un + o - lo elimina
		if ((Token == TOKEN_MAS) || (Token == TOKEN_MENOS))
		{
			if (Token == TOKEN_MAS)
			{
				TokenProcess.append("Encontrado +");
				GetToken(frase, TRUE);
				ExprFToken(frase);
				NumAhora=NumDec;


				//Leemos el siguiente token. Si es *, / o % tenemos que realizar esa operacin primero entre NumAhora y 
				//NumSiguiente.
VUELTAPOR:
				Token = GetToken(frase, FALSE);
				if ((Token == TOKEN_POTENCIA) || (Token == TOKEN_DIVISION) || (Token == TOKEN_PORCENTAJE))
				{
					POR=1;
					GetToken(frase, TRUE);
					ExprFToken(frase);
					NumDespues=NumDec;
					if (Token == TOKEN_POTENCIA)	{ Multiplicacion(NumAhora,NumDespues,NumPor);	}
					if (Token == TOKEN_DIVISION)	{ Division(NumAhora,NumDespues,NumPor);			}
					if (Token == TOKEN_PORCENTAJE)	{ Porcentaje(NumAhora,NumDespues,NumPor);		}
					NumAhora=NumPor;
					goto VUELTAPOR;
				}
				//Sino se realiza la suma entre el acumulado de sumas y numero ahora
				else
				{
					if(POR==1)
					{
						NumAhora=NumPor;
						NumPor=0;
						POR=0;
					}
					Suma(SumNum,NumAhora,SumNum);
				}
				continue;
			}
			else
			{
				TokenProcess.append("Encontrado -");
				GetToken(frase, TRUE);
				ExprFToken(frase);
				NumAhora=NumDec;


				
				//Leemos el siguiente token. Si es *, / o % tenemos que realizar esa operacin primero entre NumAhora y 
				//NumSiguiente.
VUELTAPORMenos:
				Token = GetToken(frase, FALSE);
				if ((Token == TOKEN_POTENCIA) || (Token == TOKEN_DIVISION) || (Token == TOKEN_PORCENTAJE))
				{
					POR=1;
					GetToken(frase, TRUE);
					ExprFToken(frase);
					NumDespues=NumDec;
					if (Token == TOKEN_POTENCIA)	{ Multiplicacion(NumAhora,NumDespues,NumPor);	}
					if (Token == TOKEN_DIVISION)	{ Division(NumAhora,NumDespues,NumPor);			}
					if (Token == TOKEN_PORCENTAJE)	{ Porcentaje(NumAhora,NumDespues,NumPor);		}
					NumAhora=NumPor;
					goto VUELTAPORMenos;
				}
				//Sino se realiza la suma entre el acumulado de sumas y numero ahora
				else
				{
					if(POR==1)
					{
						NumAhora=NumPor;
						NumPor=0;
						POR=0;
					}
					Resta(SumNum,NumAhora,SumNum);
				}
				continue;
			}
		}
		break;
	}

	NumDec=SumNum;

	//TokenProcess.append(QString::number(NumDec));
	textProcess->addItems(TokenProcess);
	TokenProcess.clear();
}

//Esta funcion analiza expresiones con **. Le llegan exprexiones F, y determina si hay una ExprG o 2 separadas por **
void MainWindow::ExprFToken(QString &frase)
{
	int Token;
	int NumAntes, NumAhora;
	//TokenProcess.append("ExprF");
	//Las expresiones estan formadas por una ExprG seguido o no de ** y otra ExprG
	//Mandamos la expresion G
	ExprGToken(frase);
	//Se guarda el numero decimal que llega
	NumAntes=NumDec;
	Token = GetToken(frase, FALSE);

	if (Token == TOKEN_EXPO) 
	{
		TokenProcess.append("Encontrado **");
		GetToken(frase, TRUE);
		ExprGToken(frase);
		//Se guarda el numero decimal que llega
		NumAhora=NumDec;
		//Se opera
		Exponencia(NumAntes,NumAhora,NumDec);
	}

	//TokenProcess.append(QString::number(NumDec));
	textProcess->addItems(TokenProcess);
	TokenProcess.clear();
}

//Esta funcion analiza las expresiones G, que pueden ser {GROUP}, o ( ,!,~,+,-) seguido de (EXPR), SIGNAL, CONSTANT
//LAS EXPRESIONES !,~,+,- SON SIMBOLOS DE SIGNO
void MainWindow::ExprGToken(QString &frase)
{
	int Token, tipo=0, i, j, parametro;
	QString NameSignal;
	//TokenProcess.append("ExprG");
	//Las expresiones estan formadas por un GROUP o +,-,!, ~, o nada seguido de (expr), signal o constant
	Token = GetToken(frase, FALSE);

	if (Token == TOKEN_MAS)			{ tipo=1; }
	if (Token == TOKEN_MENOS)		{ tipo=2; }
	if (Token == TOKEN_NEGBIT)		{ tipo=3; }
	if (Token == TOKEN_NEGLOG)		{ tipo=4; }
	if (Token == TOKEN_CORCHOPEN)	{ tipo=5; }

	switch (tipo)
	{
		case 1:
			TokenProcess.append("Encontrado +");
			GetToken(frase, TRUE);

			//Buscamos a continuacin si es expresion, signal o constant
			Token = GetToken(frase, FALSE);
			
			// ( EXPR )
			if (Token == TOKEN_PARENOPEN)
			{ 
				//Eliminamos (
				GetToken(frase, TRUE);
				ExprToken(frase);

				//Eliminamos )
				GetToken(frase, TRUE);
				break;
			}

			//SIGNAL
			if ( (Token == TOKEN_WORD)) 
			{
				//Se saca la palabra
				j=0;
				parametro=1;
				while (frase[j].isLetter()) 
				{
					NameSignal.push_back(frase[j]);
					j++;	
				}
				NameSignal=NameSignal.toUpper();

				//Si la seal esta dentro de la lista de seales es un signal. Sino un parametro
				for (i=0; i<SignalClass.InputList.size(); i++)
				{
					if(SignalClass.InputList[i]==NameSignal)
					{
						SignalToken(frase);
						parametro=0;
					}
				}
				for (i=0; i<SignalClass.OutputList.size(); i++)
				{
					if(SignalClass.OutputList[i]==NameSignal)
					{
						SignalToken(frase);
						parametro=0;
					}
				}
				for (i=0; i<SignalClass.InoutList.size(); i++)
				{
					if(SignalClass.InoutList[i]==NameSignal)
					{
						SignalToken(frase);
						parametro=0;
					}
				}
				for (i=0; i<SignalClass.WireList.size(); i++)
				{
					if(SignalClass.WireList[i]==NameSignal)
					{
						SignalToken(frase);
						parametro=0;
					}
				}
				
				if (parametro==1)
				{
					ConstantToken(frase);
				}
				break;
			}

			// CONSTANT
			if	(	(Token == TOKEN_NUM)
				||	(Token == TOKEN_APOSTROFE) 
				||	(Token == TOKEN_BINARIO) 
				||	(Token == TOKEN_DECIMAL) 
				||	(Token == TOKEN_OCTAL) 
				||	(Token == TOKEN_HEXADECIMAL) )
			{
				ConstantToken(frase);
				NumDec=+NumDec;
				break;
			}
			break;
		case 2:
			TokenProcess.append("Encontrado -");
			GetToken(frase, TRUE);

			//Buscamos a continuacin si es expresion, signal o constant
			Token = GetToken(frase, FALSE);
			
			// ( EXPR )
			if (Token == TOKEN_PARENOPEN)
			{ 
				//Eliminamos (
				GetToken(frase, TRUE);
				ExprToken(frase);

				//Eliminamos )
				GetToken(frase, TRUE);
				break;
			}

			//SIGNAL
			if ( (Token == TOKEN_WORD))
			{
				j=0;
				parametro=1;
				while (frase[j].isLetter()) 
				{
					NameSignal.push_back(frase[j]);
					j++;	
				}
				NameSignal=NameSignal.toUpper();
				//Si la seal esta dentro de la lista de seales es un signal. Sino un parametro
				for (i=0; i<SignalClass.InputList.size(); i++)
				{
					if(SignalClass.InputList[i]==NameSignal)
					{
						SignalToken(frase);
						parametro=0;
					}
				}
				for (i=0; i<SignalClass.OutputList.size(); i++)
				{
					if(SignalClass.OutputList[i]==NameSignal)
					{
						SignalToken(frase);
						parametro=0;
					}
				}
				for (i=0; i<SignalClass.InoutList.size(); i++)
				{
					if(SignalClass.InoutList[i]==NameSignal)
					{
						SignalToken(frase);
						parametro=0;
					}
				}
				for (i=0; i<SignalClass.WireList.size(); i++)
				{
					if(SignalClass.WireList[i]==NameSignal)
					{
						SignalToken(frase);
						parametro=0;
					}
				}
				
				if (parametro==1)
				{
					ConstantToken(frase);
				}
				break;
			}

			// CONSTANT
			if	(	(Token == TOKEN_NUM)
				||	(Token == TOKEN_APOSTROFE) 
				||	(Token == TOKEN_BINARIO) 
				||	(Token == TOKEN_DECIMAL) 
				||	(Token == TOKEN_OCTAL) 
				||	(Token == TOKEN_HEXADECIMAL) )
			{
				ConstantToken(frase);
				NumDec=-NumDec;
				break;
			}
			break;
		case 3:
			TokenProcess.append("Encontrado ~");
			GetToken(frase, TRUE);

			//Buscamos a continuacin si es expresion, signal o constant
			Token = GetToken(frase, FALSE);
			
			// ( EXPR )
			if (Token == TOKEN_PARENOPEN)
			{ 
				//Eliminamos (
				GetToken(frase, TRUE);
				ExprToken(frase);

				//Eliminamos )
				GetToken(frase, TRUE);
				break;
			}

			//SIGNAL
			if ( (Token == TOKEN_WORD)) 
			{
				j=0;
				parametro=1;
				while (frase[j].isLetter()) 
				{
					NameSignal.push_back(frase[j]);
					j++;	
				}
				NameSignal=NameSignal.toUpper();
				//Si la seal esta dentro de la lista de seales es un signal. Sino un parametro
				for (i=0; i<SignalClass.InputList.size(); i++)
				{
					if(SignalClass.InputList[i]==NameSignal)
					{
						SignalToken(frase);
						parametro=0;
					}
				}
				for (i=0; i<SignalClass.OutputList.size(); i++)
				{
					if(SignalClass.OutputList[i]==NameSignal)
					{
						SignalToken(frase);
						parametro=0;
					}
				}
				for (i=0; i<SignalClass.InoutList.size(); i++)
				{
					if(SignalClass.InoutList[i]==NameSignal)
					{
						SignalToken(frase);
						parametro=0;
					}
				}
				for (i=0; i<SignalClass.WireList.size(); i++)
				{
					if(SignalClass.WireList[i]==NameSignal)
					{
						SignalToken(frase);
						parametro=0;
					}
				}
				
				if (parametro==1)
				{
					ConstantToken(frase);
				}
				break;
			}

			// CONSTANT
			if	(	(Token == TOKEN_NUM)
				||	(Token == TOKEN_APOSTROFE) 
				||	(Token == TOKEN_BINARIO) 
				||	(Token == TOKEN_DECIMAL) 
				||	(Token == TOKEN_OCTAL) 
				||	(Token == TOKEN_HEXADECIMAL) )
			{
				ConstantToken(frase);
				NumDec=~NumDec;
				break;
			}
			break;
		case 4:
			TokenProcess.append("Encontrado !");
			GetToken(frase, TRUE);

			//Buscamos a continuacin si es expresion, signal o constant
			Token = GetToken(frase, FALSE);
			
			// ( EXPR )
			if (Token == TOKEN_PARENOPEN)
			{ 
				//Eliminamos (
				GetToken(frase, TRUE);
				ExprToken(frase);

				//Eliminamos )
				GetToken(frase, TRUE);
				break;
			}
			//SIGNAL
			if ( (Token == TOKEN_WORD))
			{
				j=0;
				parametro=1;
				while (frase[j].isLetter()) 
				{
					NameSignal.push_back(frase[j]);
					j++;	
				}
				NameSignal=NameSignal.toUpper();
				//Si la seal esta dentro de la lista de seales es un signal. Sino un parametro
				for (i=0; i<SignalClass.InputList.size(); i++)
				{
					if(SignalClass.InputList[i]==NameSignal)
					{
						SignalToken(frase);
						parametro=0;
					}
				}
				for (i=0; i<SignalClass.OutputList.size(); i++)
				{
					if(SignalClass.OutputList[i]==NameSignal)
					{
						SignalToken(frase);
						parametro=0;
					}
				}
				for (i=0; i<SignalClass.InoutList.size(); i++)
				{
					if(SignalClass.InoutList[i]==NameSignal)
					{
						SignalToken(frase);
						parametro=0;
					}
				}
				for (i=0; i<SignalClass.WireList.size(); i++)
				{
					if(SignalClass.WireList[i]==NameSignal)
					{
						SignalToken(frase);
						parametro=0;
					}
				}
				
				if (parametro==1)
				{
					ConstantToken(frase);
				}
				break;
			}

			// CONSTANT
			if	(	(Token == TOKEN_NUM)
				||	(Token == TOKEN_APOSTROFE) 
				||	(Token == TOKEN_BINARIO) 
				||	(Token == TOKEN_DECIMAL) 
				||	(Token == TOKEN_OCTAL) 
				||	(Token == TOKEN_HEXADECIMAL) )
			{
				ConstantToken(frase);
				NumDec=!NumDec;
				break;
			}
			break;
		case 5:
			GroupToken(frase);
			break;
		default:
			//NO HAY +,-,! ni ~
			//Buscamos a continuacin si es expresion, signal o constant
			Token = GetToken(frase, FALSE);
			
			// ( EXPR )
			if (Token == TOKEN_PARENOPEN)
			{ 
				//Eliminamos (
				GetToken(frase, TRUE);
				ExprToken(frase);

				//Eliminamos )
				GetToken(frase, TRUE);
				break;
			}

			//SIGNAL
			if ( (Token == TOKEN_WORD))
			{
				j=0;
				parametro=1;
				while (frase[j].isLetter()) 
				{
					NameSignal.push_back(frase[j]);
					j++;	
				}
				NameSignal=NameSignal.toUpper();
				//Si la seal esta dentro de la lista de seales es un signal. Sino un parametro
				for (i=0; i<SignalClass.InputList.size(); i++)
				{
					if(SignalClass.InputList[i]==NameSignal)
					{
						SignalToken(frase);
						parametro=0;
					}
				}
				for (i=0; i<SignalClass.OutputList.size(); i++)
				{
					if(SignalClass.OutputList[i]==NameSignal)
					{
						SignalToken(frase);
						parametro=0;
					}
				}
				for (i=0; i<SignalClass.InoutList.size(); i++)
				{
					if(SignalClass.InoutList[i]==NameSignal)
					{
						SignalToken(frase);
						parametro=0;
					}
				}
				for (i=0; i<SignalClass.WireList.size(); i++)
				{
					if(SignalClass.WireList[i]==NameSignal)
					{
						SignalToken(frase);
						parametro=0;
					}
				}
				
				if (parametro==1)
				{
					ConstantToken(frase);
				}
				break;
			}

			// CONSTANT
			if	(	(Token == TOKEN_NUM)
				||	(Token == TOKEN_APOSTROFE) 
				||	(Token == TOKEN_BINARIO) 
				||	(Token == TOKEN_DECIMAL) 
				||	(Token == TOKEN_OCTAL) 
				||	(Token == TOKEN_HEXADECIMAL) )
			{
				ConstantToken(frase);
				break;
			}
			break;
	}

	//TokenProcess.append(QString::number(NumDec));
	textProcess->addItems(TokenProcess);
	TokenProcess.clear();
}

//Funcion que analiza las CONSTANTES. Puede ser un parametro, un numero decimal solo o seguido de '
void MainWindow::ConstantToken(QString &frase)
{
	int Token;
	int TipoConversion;
	bool ok;
	QString Parametro, NumDecString;
	//TokenProcess.append("CONSTANT");
	//Las constantes pueden ser numeros decimales seguidos o no de ', parametros o '
	Token = GetToken(frase, FALSE);
	//TokenProcess.append(QString::number(Token));

	if (Token == TOKEN_NUM) 
	{
		int l=0;
		while (frase[l].isDigit() ) 
		{	
			NumDecString[l]=frase[l];
			l++;	
		}
		//Guardamos el numero y borramos
		NumDec=NumDecString.toInt(&ok,10);
		GetToken(frase, TRUE);

		//Despues puede ir un apostrofe o no
		Token = GetToken(frase, FALSE);
		if	(	(Token == TOKEN_APOSTROFE) 
			||	(Token == TOKEN_BINARIO) 
			||	(Token == TOKEN_DECIMAL) 
			||	(Token == TOKEN_OCTAL) 
			||	(Token == TOKEN_HEXADECIMAL) )
		{
			GetToken(frase, TRUE);
			
			//Dependiendo el tipo de unidad nos aparece una letra a continuacion:
			//			-	d o sd: Decimal (TipoConversion = 0)
			//			-	b o sb: Binario (TipoConversion = 1)
			//			-	o o so: Octal (TipoConversion = 2)
			//			-	h o sh: Hexadecimal (TipoConversion = 3)

			if(Token == TOKEN_BINARIO)
			{
				TokenProcess.append("Numero binario");
				TipoConversion=1;
				NumDec=ConvertirUnidades(TipoConversion,frase);
				TokenProcess.append(QString::number(NumDec));
				TokenProcess.append(frase);
			}
			if(Token == TOKEN_DECIMAL)
			{
				TokenProcess.append("Numero decimal");
				TipoConversion=0;
				NumDec=ConvertirUnidades(TipoConversion,frase);
				TokenProcess.append(QString::number(NumDec));
				TokenProcess.append(frase);
			}
			if(Token == TOKEN_OCTAL)
			{
				TokenProcess.append("Numero octal");
				TipoConversion=2;
				NumDec=ConvertirUnidades(TipoConversion,frase);
				TokenProcess.append(QString::number(NumDec));
				TokenProcess.append(frase);
			}
			if(Token == TOKEN_HEXADECIMAL)
			{
				TokenProcess.append("Numero hexadecimal");
				TipoConversion=3;
				NumDec=ConvertirUnidades(TipoConversion,frase);
				TokenProcess.append(QString::number(NumDec));
				TokenProcess.append(frase);
			}
		}
		else
		{
			TokenProcess.append("Numero decimal");
			TokenProcess.append(QString::number(NumDec));
		}
	}
	else if	(	(Token == TOKEN_APOSTROFE) 
			||	(Token == TOKEN_BINARIO) 
			||	(Token == TOKEN_DECIMAL) 
			||	(Token == TOKEN_OCTAL) 
			||	(Token == TOKEN_HEXADECIMAL) )
	{
		GetToken(frase, TRUE);
			
		//Dependiendo el tipo de unidad nos aparece una letra a continuacion:
		//			-	d o sd: Decimal (TipoConversion = 0)
		//			-	b o sb: Binario (TipoConversion = 1)
		//			-	o o so: Octal (TipoConversion = 2)
		//			-	h o sh: Hexadecimal (TipoConversion = 3)

		if(Token == TOKEN_BINARIO)
		{
			TokenProcess.append("Numero binario");
			TipoConversion=1;
			NumDec=ConvertirUnidades(TipoConversion,frase);
			TokenProcess.append(QString::number(NumDec));
		}
		if(Token == TOKEN_DECIMAL)
		{
			TokenProcess.append("Numero decimal");
			TipoConversion=0;
			NumDec=ConvertirUnidades(TipoConversion,frase);
			TokenProcess.append(QString::number(NumDec));
		}
		if(Token == TOKEN_OCTAL)
		{
			TokenProcess.append("Numero octal");
			TipoConversion=2;
			NumDec=ConvertirUnidades(TipoConversion,frase);
			TokenProcess.append(QString::number(NumDec));
		}
		if(Token == TOKEN_HEXADECIMAL)
		{
			TokenProcess.append("Numero hexadecimal");
			TipoConversion=3;
			NumDec=ConvertirUnidades(TipoConversion,frase);
			TokenProcess.append(QString::number(NumDec));
		}
	}
	
	////////////Faltan parametros///////////////////////////////////////////////////////////
	if (Token == TOKEN_WORD)
	{
		TokenProcess.append("Parametro");
		int k=0;
		while (frase[k].isLetter() ) 
		{	
			Parametro[k]=frase[k];
			k++;	
		}

		//Guardamos el parametro en la variable dedicada para ello en ParametroToken
		ParametroToken.FraseParametro = Parametro;
		ParametroToken.VariableList.push_back(ParametroToken.FraseParametro);
		//SaveSignals(Parametro);

		//TokenProcess.append(ParametroToken.FraseParametro);
		GetToken(frase, TRUE);
		//ExprToken(frase);
	}
	//TokenProcess.append(frase);
	textProcess->addItems(TokenProcess);
	TokenProcess.clear();
}

//Funcion que analiza las SIGNALS. Sera una seal (INPUT, OUTPUT, INOUT o WIRE) seguido de [ exp : exp ] o no
void MainWindow::SignalToken(QString &frase)
{
	int Token;
	int NumDigito, NumDigitoIni=-1, NumDigitoFin=-1, Necesito, NecesitoFin;
	bool ok;
	QString NumDigitoString, NumDigitoBinString, NumDecStringNecesito;
	QString caracteristicas, NameSignal;
	QString SignalProcess;

	SignalProcess.append("SIGNALPROCESS.\tFrase entrada: ");
	SignalProcess.push_back(frase);
	Token = GetToken(frase, FALSE);

	//Se guarda el nombre de la seal y se elimina la palabra
	if (Token == TOKEN_WORD)
	{ 
		int j=0;
		while (frase[j].isLetter()) 
		{
			NameSignal.push_back(frase[j]);
			j++;	
		}
		NameSignal=NameSignal.toUpper();
		GetToken(frase, TRUE);	
	}
	SignalProcess.append(",\tNombre de seal detectado: ");
	SignalProcess.append(NameSignal);
	
	NameSignalTOKEN=NameSignal;
	//Busca si despues del tipo de variable hay o no parentesis
	Token = GetToken(frase, FALSE);
	
	//Si hay necesitamos el valor del digito de la posicion entre []
	if (Token == TOKEN_QPARENOPEN) 
	{
		SignalProcess.append("\t[");
		//TokenProcess.append("Encontrado [");
		GetToken(frase, TRUE);

		//Para guardar las caracteristicas vamos almacenando en un String
		caracteristicas="[";

		//Se manda la expresin
		ExprToken(frase);
		NumDigitoIni=NumDec;
		SignalProcess.append("\tNumDigitoIni: ");
		SignalProcess.append(QString::number(NumDigitoIni));

		//Comprobamos si hay : expr o ]
		Token = GetToken(frase, FALSE);

		if (Token == TOKEN_DOSPUNTOS)
		{
			SignalProcess.append("\t:");
			GetToken(frase, TRUE);
			caracteristicas.push_back(":");

			//Se manda la expresin
			ExprToken(frase);
			NumDigitoFin=NumDec;
			SignalProcess.append("\tNumDigitoFin: ");
			SignalProcess.append(QString::number(NumDigitoFin));

			Token = GetToken(frase, FALSE);
			if (Token == TOKEN_QPARENCLOSE)	
			{
				SignalProcess.append("\t]");
				GetToken(frase, TRUE);
				caracteristicas.push_back("]");
			}
		}

		else if (Token == TOKEN_QPARENCLOSE)
		{
			SignalProcess.append("\t]");
			GetToken(frase, TRUE);
			caracteristicas.push_back("]");

		}

		SignalClass.CaractSignal=caracteristicas;

		//Recorremos las listas de variables para saber el valor actual de NameSignal
		int i;
		if(SignalClass.InputValor.size()!=0)
		{
			for(i=0; i<SignalClass.InputValor.size(); i++)
			{
				if(NameSignalTOKEN==SignalClass.InputList[i])
				{
					NumDigito=SignalClass.InputValor[i];
				}
			}
		}
		if(SignalClass.OutputValor.size()!=0)
		{
			for(i=0; i<SignalClass.OutputValor.size(); i++)
			{
				if(NameSignalTOKEN==SignalClass.OutputList[i])
				{
					NumDigito=SignalClass.OutputValor[i];
				}
			}
		}
		if(SignalClass.InoutValor.size()!=0)
		{
			for(i=0; i<SignalClass.InoutValor.size(); i++)
			{
				if(NameSignalTOKEN==SignalClass.InoutList[i])
				{
					NumDigito=SignalClass.InoutValor[i];
				}
			}
		}
		if(SignalClass.WireValor.size()!=0)
		{
			for(i=0; i<SignalClass.WireValor.size(); i++)
			{
				if(NameSignalTOKEN==SignalClass.WireList[i])
				{
					NumDigito=SignalClass.WireValor[i];
				}
			}
		}

		//Tenemos:
		//	-	NumDigito = El valor de la seal
		//	-	NumDigitoInit = El numero del bit que nos interesa [ NumDigitoInit ]
		//	-	NumDigitoFin = El numero del bit que nos interesa [ NumDigitoInit : NumDigitoFin ]
		//NumDigitoInit y Fin son -1 si no hay.
		//Hay que pasar el int a String, convertirlo a Binario, tomar el numero de digitos del valor total que nos interesa y devolverlo con NumDec

		//Mostramos numero inicial a convertir
		SignalProcess.append("\nNumero inicial: ");
		SignalProcess.append(QString::number(NumDigito));

		//Convertimos a binario y mostramos
		NumDigitoString.append(QString::number(NumDigito,2));
		SignalProcess.append("\tNumero binario inicial: ");
		SignalProcess.append(NumDigitoString);

		//Calculo que rango de bits necesitamos. El digito donde empiezo y donde acabo en el QString
		//
		//		Numero binario:							1	0	0	1	1	0	1
		//		Posicion en QString (con l inicial=1):	1	2	3	4	5	6	7		
		//		Posicion en numero binario:				6	5	4	3	2	1	0

		Necesito=((NumDigitoString.size())-NumDigitoIni)-1;
		NecesitoFin=((NumDigitoString.size())-NumDigitoFin)-1;

		//Si necesito es negativo es porque hemos pedido un bit superior al que hay (posicion 8 en el ejemplo superior
		//y como maximo hay 6. En ese caso aadimos un 0. Si el numero de la seal es negativo tambien
		if((Necesito<0) || (NumDigito<0))
		{
			NumDec=0;
			SignalProcess.append("\tBit pedido superior a tamao o negativo. 0 por defecto");
		}
		//Sino, dependiendo de si NumDigitoFin es -1 (valor inicializacion que indica que no hay rango de bits 10:5]
		//o no calcula el numero de la posicion indicada o los numeros binarios entre posiciones
		else
		{
			//Caso [Num]
			if(NumDigitoFin==-1)
			{
				SignalProcess.append("\tNecesito el bit: ");
				SignalProcess.append(QString::number(Necesito));
				SignalProcess.append(" del String");

				for(int l=0; l<=NumDigitoString.size(); l++)
				{
					if (l==Necesito)
					{
						NumDecStringNecesito=NumDigitoString[l];
						SignalProcess.append("\tNumero binario final: ");
						SignalProcess.append(NumDecStringNecesito);
						NumDec=NumDecStringNecesito.toInt(&ok,2);
						SignalProcess.append("\tNumero decimal final: ");
						SignalProcess.append(QString::number(NumDec));
					}
				}
			}
			//Caso [Num:Num]
			else
			{
				SignalProcess.append("\tNecesito los bits del ");
				SignalProcess.append(QString::number(NumDigitoIni));
				SignalProcess.append(" al ");
				SignalProcess.append(QString::number(NumDigitoFin));
				SignalProcess.append(" del String");

				for(int l=0; l<=NumDigitoString.size(); l++)
				{
					if (l==Necesito)
					{
						while(l!=NecesitoFin)
						{
							NumDecStringNecesito.push_back(NumDigitoString[l]);
							l++;
						}
						NumDecStringNecesito.push_back(NumDigitoString[l]);

						SignalProcess.append("\tNumero binario final: ");
						SignalProcess.append(NumDecStringNecesito);
						NumDec=NumDecStringNecesito.toInt(&ok,2);
						SignalProcess.append("\tNumero decimal final: ");
						SignalProcess.append(QString::number(NumDec));
					}
				}
			}
		}

	}
	//Sino hay [ ] simplemente devuelve la seal
	else
	{
		caracteristicas=(" ");

		SignalClass.CaractSignal=caracteristicas;

		//Recorremos las listas de variables para saber el valor actual de NameSignal
		int i;
		if(SignalClass.InputValor.size()!=0)
		{
			for(i=0; i<SignalClass.InputValor.size(); i++)
			{
				if(NameSignalTOKEN==SignalClass.InputList[i])
				{
					NumDec=SignalClass.InputValor[i];
				}
			}
		}
		if(SignalClass.OutputValor.size()!=0)
		{
			for(i=0; i<SignalClass.OutputValor.size(); i++)
			{
				if(NameSignalTOKEN==SignalClass.OutputList[i])
				{
					NumDec=SignalClass.OutputValor[i];
				}
			}
		}
		if(SignalClass.InoutValor.size()!=0)
		{
			for(i=0; i<SignalClass.InoutValor.size(); i++)
			{
				if(NameSignalTOKEN==SignalClass.InoutList[i])
				{
					NumDec=SignalClass.InoutValor[i];
				}
			}
		}
		if(SignalClass.WireValor.size()!=0)
		{
			for(i=0; i<SignalClass.WireValor.size(); i++)
			{
				if(NameSignalTOKEN==SignalClass.WireList[i])
				{
					NumDec=SignalClass.WireValor[i];
				}
			}
		}
	}

	TokenProcess.append(SignalProcess);
	textProcess->addItems(TokenProcess);
	TokenProcess.clear();
}

//Funcion para analizar GRUPOS DE EXPRESIONES separadas por , o {
//A esta funcion llega una expresion del tipo: {expr {expr,expr...}}
void MainWindow::GroupToken(QString &frase)
{
	int Token;
	//TokenProcess.append("GROUP");
	//Los grupos empiezan por {
	Token = GetToken(frase, FALSE);

	if (Token == TOKEN_CORCHOPEN) 
	{
		GetToken(frase, TRUE);
		ExprToken(frase);

		Token = GetToken(frase, FALSE);
		//Si vuelve a haber otro corchete {
		if (Token == TOKEN_CORCHOPEN)
		{
			//Borra el corchete
			GetToken(frase, TRUE);
			while(1)
			{
				//Manda la expresion
				ExprToken(frase);

				//Puede haber otra expresion separada por una coma
				Token = GetToken(frase, FALSE);
				if (Token == TOKEN_COMA)
				{
					GetToken(frase, TRUE);
					continue;
				}
				break;
			}

			//Borra todos los corchetes finales }
			Token = GetToken(frase, FALSE);
			if (Token == TOKEN_CORCHCLOSE)
			{
				GetToken(frase, TRUE);
				Token = GetToken(frase, FALSE);
				if (Token == TOKEN_CORCHCLOSE)
				{
					GetToken(frase, TRUE);
				}
			}
		}
		else
		{
			while(1)
			{
				//Manda la expresion
				ExprToken(frase);

				//Puede haber otra expresion separada por una coma
				Token = GetToken(frase, FALSE);
				if (Token == TOKEN_COMA)
				{
					GetToken(frase, TRUE);
					continue;
				}
				break;
			}
			//Elimina corchete final
			Token = GetToken(frase, FALSE);
			if (Token == TOKEN_CORCHCLOSE)
			{
				GetToken(frase, TRUE);
			}
		}
	}
		
	//TokenProcess.append(frase);
	textProcess->addItems(TokenProcess);
	TokenProcess.clear();
}
